/****************************************************************************
 *
 * /home/bat/bat/fast3202_4_0_21_3_3_1/rg/vendor/adi/ad6843/linux-2.6/modules/mtd/mtd.c
 * 
 * Copyright (C) Jungo LTD 2004
 * 
 * This program is free software; you can redistribute it
 * and/or modify it under the terms of the GNU General 
 * Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at
 * your option) any later version.
 * 
 * This program is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the Free
 * Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 * MA 02111-1307, USA.
 *
 * Developed by Jungo LTD.
 * Residential Gateway Software Division
 * www.jungo.com
 * info@jungo.com
 */

#include <linux/module.h>
#include <linux/init.h>
#include <linux/mtd/map.h>
#include <linux/mtd/partitions.h>
#include <linux/ioport.h>
#include <asm/io.h>
#include <linux/sbi_k.h>
#define WINDOW_SIZE ((FLASH_SIZE) * 1024 * 1024)
#define WINDOW_BASE (0x20000000 - WINDOW_SIZE)
#define BUSWIDTH 2

#define NB_OF(x)  (sizeof(x)/sizeof(x[0]))

#define LOG(f, p...) printk("%s: " f "\n", fusiv_mtd_map.name , ## p)
#ifdef DEBUG_VERBOSE
# define LOG_VERBOSE(f, p...) printk("%s: " f "\n", fusiv_mtd_map.name , ## p)
#else  /*!DEBUG_VERBOSE*/
# define LOG_VERBOSE(f, p...)
#endif /*!DEBUG_VERBOSE*/ 


static struct map_info fusiv_mtd_map = {
    .name = "fusiv_flash",
    .bankwidth = BUSWIDTH,
    .size = 8 * 1024 * 1024,
    .phys = 0x20000000 - (8 * 1024 * 1024),

};

static struct mtd_partition fusiv_mtd_partitions[] = {
    { .name = "openrg", .offset = 0x00000000, .size = 8 * 1024 * 1024, },
    { .name = "jffs2",  .offset = 0x1a000, .size = 0x100, },
#if defined CONFIG_SAGEM_LBV2ULT || defined CONFIG_SAGEM_BOXER
    { .name = "jffs1",  .offset = 0x00000000, .size = 16 * 1024 * 1024, },
#endif
	/* Warning - keep these values in sync with OpenRG flash layout */
	/* in vendor/sagem/livebox directory (look for JFFS partition) */
};

static struct mtd_info *fusiv_mtd_info;

void fusiv_mtd_uninit(void)
{
    if (fusiv_mtd_info)
	map_destroy(fusiv_mtd_info);

    if (fusiv_mtd_map.virt)
        iounmap((void *)fusiv_mtd_map.virt);
}

#if defined(CONFIG_SAGEM_LBV2BABY)
int __init fusiv_mtd_init(void)
{
	int nb_partitions = 2;

	sbi->flash_size=8;
	fusiv_mtd_map.size=0x800000;
	fusiv_mtd_map.phys=0x1f800000;

	fusiv_mtd_map.virt = ioremap(fusiv_mtd_map.phys,fusiv_mtd_map.size);
	if(!fusiv_mtd_map.virt)
	{
		printk(KERN_CRIT "ioremap(%#lx, %#lx) failed", fusiv_mtd_map.phys, fusiv_mtd_map.size);
		return -EIO;
	}
	fusiv_mtd_info = do_map_probe("cfi_probe",&fusiv_mtd_map);
	if (!fusiv_mtd_info)
	{
		printk(KERN_CRIT "do_map_probe() failed\n");
		return -ENODEV;
	}

	fusiv_mtd_partitions[0].size = fusiv_mtd_map.size;
	fusiv_mtd_partitions[1].size = 0x000a0000;
	fusiv_mtd_partitions[1].offset = 0x00120000;

	if (add_mtd_partitions(fusiv_mtd_info, fusiv_mtd_partitions, nb_partitions))
	{
		LOG("add_mtd_partitions() failed\n");
		return -ENOMEM;
	}

	return 0;
}
#else
int __init fusiv_mtd_init(void)
{
    int res = -EIO;
	int nb_partitions=2;
	
    if (sbi) sbi->flash_size = 128;

    while (res && FLASH_SIZE>=4) {
	sbi->flash_size = FLASH_SIZE>>1;
	fusiv_mtd_map.size = WINDOW_SIZE;
	fusiv_mtd_map.phys = WINDOW_BASE;
    fusiv_mtd_map.virt = ioremap(fusiv_mtd_map.phys, fusiv_mtd_map.size);
    if (!fusiv_mtd_map.virt)
    {
		LOG_VERBOSE("ioremap(%#lx, %#lx) failed",
	    fusiv_mtd_map.phys, fusiv_mtd_map.size);
		res = -EIO;
		continue;
    }

    fusiv_mtd_info = do_map_probe("cfi_probe", &fusiv_mtd_map);
    if (!fusiv_mtd_info)
    {
		LOG_VERBOSE("do_map_probe() failed");
		res = -ENODEV;
		continue;
	} else {
		res = 0;
		break;
	}
	iounmap((volatile void __iomem *)fusiv_mtd_map.phys);
    }
    if (FLASH_SIZE<4)
	goto Error;

    printk("<3>\n");
    printk("<3>\t.----------------------------------------------------------------.\n");
    printk("<3>\t|           DYNAMICALLY DETECTED FLASH_SIZE = %-3d Mbytes         |\n", FLASH_SIZE);
    printk("<3>\t`----------------------------------------------------------------'\n");
    printk("<3>\n");
	    
	fusiv_mtd_partitions[1].size = MTD_PARTITION_JFFS2_SIZE;
	fusiv_mtd_partitions[1].offset = MTD_PARTITION_JFFS2_OFFSET;
#if defined(CONFIG_SAGEM_LBV2ULT) || defined (CONFIG_SAGEM_BOXER)
	if (FLASH_SIZE==32)
	{
		fusiv_mtd_partitions[0].size = 0x1000000;
		fusiv_mtd_partitions[0].offset = 0x1000000;
		fusiv_mtd_partitions[1].offset +=  0x1000000;
		nb_partitions=3;
	}
	else
#endif
	{
		fusiv_mtd_partitions[0].size = WINDOW_SIZE;
	}

    if (add_mtd_partitions(fusiv_mtd_info, fusiv_mtd_partitions, nb_partitions))
    {
		LOG("add_mtd_partitions() failed\n");
		res = -ENOMEM;
		goto Error;
    }

    LOG("detected at %#lx size %ld bytes", fusiv_mtd_map.phys,
	fusiv_mtd_map.size);

    return 0;

Error:
    fusiv_mtd_uninit();
    return res;
}
#endif

module_init(fusiv_mtd_init);
module_exit(fusiv_mtd_uninit);
