/**
 * @note Copyright (c) 2006  Sagem Communication All rights reserved
 *
 * @file sbi_160.c
 * @version 1.0
 * @date 25/11/2006
 * @author Sagem Communication
 * @brief 
**/
#include <linux/sbi_k.h>
#include <linux/sbi_k_160.h>

struct interrupt_numbers vox160_int_nums = 
{
	.adi_6843_max_ints	= 36,
	.ap_dma_int		= 32,
	.arb2_int		= 12,
	.arb3_int		= 27,
	.arb_int		=  1,
	.bmu_int		= 31,
	.dma_int		=  0,
	.dsp0_fl0_int		= 17,
	.dsp0_fl1_int		= 18,
	.dsp0_pf0_int		= 19,
	.dsp0_pf1_int		= 20,
	.dsp1_fl0_int		= 21,
	.dsp1_fl1_int		= 22,
	.dsp1_pf0_int		= 23,
	.dsp1_pf1_int		= 24,
	.emac1_int		= 14,
	.emac2_int		= 13,
	.emac3_int		= 36,
	.gpio2_int		=  4,
	.gpio_int		=  8,
	.gpt1_int		=  9,
	.gpt2_int		= 10,
	.gpt3_int		= 11,
	.idma1_int		= 16,
	.idma2_int		= 15,
	.idma3_int		= 33,
	.pci_int		= 25,
	.per_int		= 30,
	.scu_int		=  2,
	.spa_int		= -1,
	.sphy_int		= 26,
	.spi_int		=  7,
	.sport2_rx_int		= 28,
	.sport2_tx_int		= 34,
	.uart1_int		=  6,
	.uart2			= 29,
	.uart2_int		= -1,
	.usbh_int		= 35,
	.usb_int		=  3,
	.wiu1_int		=  5,
	.wiu2_int		= -1
	
};


/* Warning - keep these values in sync with OpenRG flash layout  */
/* in vendor/sagem/livebox directory (look for JFFS partition) */
#if defined CONFIG_SAGEM_LBV2BABY
unsigned long vox160_mtd_partition_jffs2_offset = 0x00120000;
unsigned long vox160_mtd_partition_jffs2_size   = 0x000a0000;
#else
unsigned long vox160_mtd_partition_jffs2_offset = 0x001a0000;
unsigned long vox160_mtd_partition_jffs2_size   = 0x000a0000;
#endif

int vox160_first_pci_pin = 20;
int vox160_adsl_gpio_reset_bit = 15;
int num_emac_ports_160 = 2;

int init_sbi_160(void) 
{

	sbi->int_nums = &vox160_int_nums;
	sbi->mtd_partition_jffs2_offset = vox160_mtd_partition_jffs2_offset; 
	sbi->mtd_partition_jffs2_size = vox160_mtd_partition_jffs2_size; 
	sbi->adsl_gpio_reset_bit = vox160_adsl_gpio_reset_bit;
	sbi->first_pci_pin = vox160_first_pci_pin;
	//sbi->sbi_bmap_tasklet = bmap_tasklet_160;
	sbi->mem_detect = vox160_mem_detect;
	sbi->fusiv_init_IRQ = fusiv_init_IRQ_160;
    sbi->num_emac_ports = num_emac_ports_160;
	sprintf(sbi->board_name, "VOX160 - 0x%04x", CPU_ID);

	return 0;
} 

// Modif FR Integration of V31 on Mini 20070316
//DECLARE_TASKLET(bmap_tasklet_160, bmHostTask_160, 0);

int vox160_mem_detect(void)
{
	volatile unsigned int mem_reg = 0, mem_size = 0;
	int nbanks = 0, device_size = 0;		
	
	mem_reg = (*(volatile unsigned int*)0xb9250004);
	// Le boot configure toujours 2 bank actives. 
	// Ce Bug de decalage pas dans le bon sens nous convient bien!! On le laisse!
	nbanks = ((mem_reg<<14)&0x1) ? 2 : 1; 
	switch ((mem_reg>>18)&0x3) {
		case 0x0:	device_size = 512; break;
		case 0x1:	device_size =  64; break;
		case 0x2:	device_size = 128; break;
		case 0x3:	device_size = 256; break;
		default: 	BUG();             break;
	}
	switch ((mem_reg>>16)&0x3) {
		case 0x0:	device_size *= 4; break;	// Individual SDRAM is 4-bit wide
		case 0x1:	device_size *= 2;  break;	// Individual SDRAM is 8-bit wide
	}
	mem_size = (nbanks*device_size) >>3;
	sbi->sdram_size = mem_size;
	return 0;
}
