/**
 * @note Copyright (c) 2006  Sagem Communication All rights reserved
 *
 * @file sbi_k.h
 * @version 1.0
 * @date 25/11/2006
 * @author Sagem Communication
 * @brief 
**/
#ifndef _SBI_K_H
#define _SBI_K_H

#include <linux/interrupt.h>
#include <linux/sbi_defs.h>

/*
 * 	Sagem Board Info / Kernel part
 *	******************************
 */

struct interrupt_numbers {
	int adi_6843_max_ints;
	int ap_dma_int;
	int arb2_int;
	int arb3_int;
	int arb_int;
	int bmu_int;
	int dma_int;
	int dsp0_fl0_int;
	int dsp0_fl1_int;
	int dsp0_pf0_int;
	int dsp0_pf1_int;
	int dsp1_fl0_int;
	int dsp1_fl1_int;
	int dsp1_pf0_int;
	int dsp1_pf1_int;
	int emac1_int;
	int emac2_int;
	int emac3_int;
	int gpio2_int;
	int gpio_int;
	int gpt1_int;
	int gpt2_int;
	int gpt3_int;
	int idma1_int;
	int idma2_int;
	int idma3_int;
	int pci_int;
	int per_int;
	int scu_int;
	int spa_int;
	int sphy_int;
	int spi_int;
	int sport2_rx_int;
	int sport2_tx_int;
	int uart1_int;
	int uart2;
	int uart2_int;
	int usbh_int;
	int usb_int;
	int wiu1_int;
	int wiu2_int;
}; 

struct sagem_board_info {

	char board_name[16];
	
	volatile unsigned int  cpu_id;	//0x683[346]
	unsigned long vendor_id;	//read hw infos from gpios?
	
	int (*mem_detect)(void);
	void (*fusiv_init_IRQ)(void );

	unsigned int sdram_size;		//Mbytes
	unsigned int sdram_size_with_rg_loader;	//Mbytes
	
		
	unsigned int flash_size;		//Mbytes
	unsigned long mtd_partition_jffs2_offset;
	unsigned long mtd_partition_jffs2_size;
	
	int num_emac_ports;
	struct interrupt_numbers * int_nums;
	
	int first_pci_pin;
	
	int adsl_gpio_reset_bit;
	struct gpio_access_struct_t * gpios;
	
	struct tasklet_struct sbi_bmap_tasklet;
		
};

#ifdef IPC_GENERIC_C

typedef struct {
	unsigned long	ipc_id;		/* Interrupt controller ID */
	unsigned long	pad1[0x3f];	/* Reserved */
	unsigned long	cpe_prio[8];	/* CPE Interrupt Priority Registers */
	unsigned long	ap_prio;	/* AP Interrupt Priority Register */
	unsigned long	pad2;		/* Reserved */
	int_stat_t	cpe_stat[11];	/* CPE Interrupt Status Registers*/
	int_stat_t	emac_stat[3];	/* EMAC AP Interrupt Status Registers */
	int_stat_t	bmu_stat;	/* BMU AP Interrupt Status Registers */
	int_stat_t	spa_stat;	/* SPA AP Interrupt Register */
	int_stat_t	pci_stat;	/* PCI Interrupt Registers */
	unsigned long	pad3[0x54];	/* Reserved */
	unsigned long	cpe_per;	/* CPE Peripheral Interrupt */
} ipc_t_200;
typedef struct {
        unsigned long   ipc_id;         /* Interrupt controller ID */
        unsigned long   pad1[0x3f];     /* Reserved */
        unsigned long   cpe_prio[10];   /* CPE Interrupt Priority Registers */
        int_stat_t      cpe_stat[10];   /* CPE Interrupt Status Registers*/
        int_stat_t      emac_stat[3];   /* EMAC AP Interrupt Status Registers */
        int_stat_t      bmu_stat;       /* BMU AP Interrupt Status Registers */
        int_stat_t      peri_stat;      /* Peripheral AP Interrupt Register */
        int_stat_t      pci_stat;       /* PCI Interrupt Registers */
        unsigned long   pad3[0x54];     /* Reserved */
        unsigned long   cpe_per;        /* CPE Peripheral Interrupt */
} ipc_t_160;


#endif /*IPC_GENERIC_C*/



extern struct sagem_board_info sagem_board_info;
extern struct sagem_board_info * sbi; 

extern int init_sbi_cpu(void);

#endif /*_SBI_K_H*/
