/* =============================================================================
 *
 * Copyright (C)  2005  Analog Devices Inc. All rights reserved
 *
 * The information and source code contained herein is the property
 * of Ikanos Communications.
 *
 *  This program is free software; you can redistribute  it and/or modify it
 *  under  the terms of  the GNU General  Public License as published by the
 *  Free Software Foundation;  either version 2 of the  License, or (at your
 *  option) any later version.
 * ===========================================================================*/
 
#include <linux/kernel.h>
#include <linux/init.h>
#include <linux/interrupt.h>
#include <linux/types.h>
#include <linux/interrupt.h>
#include <linux/pci.h>
#include <linux/delay.h>
#include <asm/bootinfo.h>
#include <adi6843.h>
#include <fusivpci.h>
#include <linux/sbi_k.h>

#define PCI_ACCESS_READ  0
#define PCI_ACCESS_WRITE 1
#define PCI_MAX_SLOTS 	4

extern int board_memsize;
static spinlock_t fusiv_pci_lock;


static int __init setpci_pin(char *str)
{
	if (sbi)
		get_option(&str, &sbi->first_pci_pin);
	return 1;
}

static int fusiv_pcibios_config_access(unsigned char access_type,
	struct pci_bus *bus, unsigned int devfn, int where, u32 * val)
{
	unsigned char busnum = bus->number;
	u32 err;
//	printk("bus %d  slot %d fun %d where %x \n",busnum,PCI_SLOT(devfn),PCI_FUNC(devfn),where);
	if (busnum)	return PCIBIOS_DEVICE_NOT_FOUND;
	if (PCI_SLOT(devfn) > PCI_MAX_SLOTS ){
		/* The addressing scheme chosen leaves room for just
		 * 4 devices on the first busnum (besides the PCI
		 * controller itself) */
		return PCIBIOS_DEVICE_NOT_FOUND;
	}

	if (devfn == PCI_DEVFN(0, 0)) {
		/* Access controller registers directly */
		if (access_type == PCI_ACCESS_WRITE) {
		    PCI_CRP_CTL = PCI_CRP_VAL(where, PCI_CMD_CFG_WR, 0);
		    PCI_CRP_WDAT = *val;
		} else {
		    PCI_CRP_CTL = PCI_CRP_VAL(where, PCI_CMD_CFG_RD, 0);
		    *val = PCI_CRP_RDAT;
		}
		return PCIBIOS_SUCCESSFUL;
	}

	/* Clear PCI Error register. This also clears the Error Type
	 * bits in the Control register */
	//PCI_ERR_REG = 0xffffffff;

	/* Setup address */
	PCI_NPC_ADDR = (1 << (PCI_SLOT(devfn)+ FIRST_PCI_PIN -1)) | (where & ~3) | (PCI_FUNC(devfn)<< 8);
	
	if (access_type == PCI_ACCESS_WRITE){
		PCI_NPC_CTL = PCI_NPC_VAL (PCI_CMD_CFG_WR, 0);
		PCI_NPC_WDAT = *val;
	}
	else{
		PCI_NPC_CTL = PCI_NPC_VAL (PCI_CMD_CFG_RD, 0);
		*val = PCI_NPC_RDAT;
	}

	/* Check for master or target abort */
	err = PCI_ERR_REG & 0x103;
	
	if (err){
		PCI_ERR_REG = 0xffffffff;
		return PCIBIOS_DEVICE_NOT_FOUND;
	}

	return PCIBIOS_SUCCESSFUL;
}


/* Bug 20011 NPn => Patch ikanos pour merlin
 * *****************************************/
#define CONFIG_FUSIV_KERNEL_WLAN_ATH_11N /* sebf */
#define CONFIG_FUSIV_KERNEL_WLAN_ATH_11G

static int fusiv_pcibios_read(struct pci_bus *bus, unsigned int devfn,
	int where, int size, u32 * val)
{
	unsigned long flags;
	u32 data = 0;
	int err;
#if defined(CONFIG_FUSIV_KERNEL_WLAN_ATH_11G) || defined(CONFIG_FUSIV_KERNEL_WLAN_ATH_11N)
        int read_count = 0;
        const int MAX_READ_COUNT = 3;
#endif

	if ((size == 2) && (where & 1))
		return PCIBIOS_BAD_REGISTER_NUMBER;
	else if ((size == 4) && (where & 3))
		return PCIBIOS_BAD_REGISTER_NUMBER;

#if defined(CONFIG_FUSIV_KERNEL_WLAN_ATH_11G) || defined(CONFIG_FUSIV_KERNEL_WLAN_ATH_11N)
        /*
         * There is a problem in bring up Atheros 11g MB55 miniPCI card
         * on Ikanos Vox160/180 platform. The PCI controller access to PCI_VENDOR_ID
         * configure register on MB55, then MB55 replies with "retry" command and
         * retrieves the configure data from EEPROM. Before the MB55 have all data
         * ready, the controller enters some state and stop trying.
         * This code re-read the configure register.
         */
#define PCI_INVALID_ID(data) \
        ((data) == 0xffffffff || (data) == 0x00000000 || (data) == 0x0000ffff || (data) == 0xffff0000)

        do {
                spin_lock_irqsave(&fusiv_pci_lock, flags);
                err = fusiv_pcibios_config_access(PCI_ACCESS_READ, bus, devfn, \
                          where, &data);
                spin_unlock_irqrestore(&fusiv_pci_lock, flags);

                if ( (where != PCI_VENDOR_ID) || err) {
                        break;
                }
                mdelay (100);
         } while ( (read_count++ <= MAX_READ_COUNT) && PCI_INVALID_ID(data) );
#else
	spin_lock_irqsave(&fusiv_pci_lock, flags);
	err = fusiv_pcibios_config_access(PCI_ACCESS_READ, bus, devfn, where,
					&data);
	spin_unlock_irqrestore(&fusiv_pci_lock, flags);
#endif

	if (err)
		return err;

	if (size == 1)
		*val = (data >> ((where & 3) << 3)) & 0xff;
	else if (size == 2)
		*val = (data >> ((where & 3) << 3)) & 0xffff;
	else
		*val = data;

	return PCIBIOS_SUCCESSFUL;
}

static int fusiv_pcibios_write(struct pci_bus *bus, unsigned int devfn,
	int where, int size, u32 val)
{
	unsigned long flags;
	u32 data = 0;
	int err;

	if ((size == 2) && (where & 1))
		return PCIBIOS_BAD_REGISTER_NUMBER;
	else if ((size == 4) && (where & 3))
		return PCIBIOS_BAD_REGISTER_NUMBER;

	spin_lock_irqsave(&fusiv_pci_lock, flags);
	err = fusiv_pcibios_config_access(PCI_ACCESS_READ, bus, devfn, where,
	                                  &data);
	spin_unlock_irqrestore(&fusiv_pci_lock, flags);

	if (err)
		return err;

	if (size == 1)
		data = (data & ~(0xff << ((where & 3) << 3))) |
		    (val << ((where & 3) << 3));
	else if (size == 2)
		data = (data & ~(0xffff << ((where & 3) << 3))) |
		    (val << ((where & 3) << 3));
	else
		data = val;

	if (fusiv_pcibios_config_access
	    (PCI_ACCESS_WRITE, bus, devfn, where, &data))
		return -1;

	return PCIBIOS_SUCCESSFUL;
}

unsigned char fusiv_inb(u16 port)
{
	int tmp;
	PCI_NPC_ADDR = port; 
	PCI_NPC_CTL = 0x12;
	tmp = PCI_NPC_RDAT;
	printk("inport %x , data %x\n",port,tmp);
	return (tmp&0xff);
}

void fusiv_outb(unsigned val, u16 port)
{
	
	PCI_NPC_ADDR = port; 
	PCI_NPC_CTL = 0x13;
	PCI_NPC_WDAT =val;
}

int fusiv_inw(u16 port)
{
	int tmp,tmp1;
	PCI_NPC_ADDR = port; 

	if (port & 2)
		tmp1 = 0x32;
	else
		tmp1 = 0xc2;
		
	PCI_NPC_CTL = tmp1;
	tmp = PCI_NPC_RDAT;
	if (port & 2)
		return tmp >>16;
	return (tmp&0xffff);
}
void fusiv_outw(u16 val, u16 port)
{
	
	PCI_NPC_ADDR = port; 
	
	if(port & 2) {
	       PCI_NPC_CTL = 0x33;
	       PCI_NPC_WDAT =val<< 16;
	}
	else {
		PCI_NPC_CTL = 0xc3;
		PCI_NPC_WDAT =val;
	}
}

void fusiv_outl(int val, u16 port)
{
	PCI_NPC_ADDR = port; 
	
       PCI_NPC_CTL = PCI_NPC_VAL (PCI_CMD_IO_WR,0);

	 PCI_NPC_WDAT = (val);
	 
}

struct pci_ops fusiv_pci_ops = {
	.read = fusiv_pcibios_read,
	.write = fusiv_pcibios_write,
};

static struct resource fusiv_pci_mem_resource = {
	.name	= "FUSIV PCI MEM",
	.start	= PCI_MEM_BASE,
	.end		= PCI_MEM_END,
	.flags	= IORESOURCE_MEM,
};

static struct resource fusiv_pci_io_resource = {
	.name	= "FUSIV PCI IO",
	.start	= 0x00000100,
	.end		= 0xFFFF,
	.flags	= IORESOURCE_IO,
};

static struct pci_controller fusiv_pci_controller = {
	.mem_resource	= &fusiv_pci_mem_resource,
	.io_resource	= &fusiv_pci_io_resource,
	.pci_ops		= &fusiv_pci_ops,
};

int __init fusiv_pci_setup(void)
{
 	printk("Fusiv PCI: starting...\n");
	/* reset PCI controller */
	PCI_RST_CONTROL = 0x1;
	mdelay (1);
	PCI_RST_CONTROL = 0x3;
	mdelay (5);
	PCI_RST_CONTROL = 0x1;
	mdelay (5);

	  // Load items used to setup our PCI configuration space
	  // Change the address for PCI_ID, PCI_REV etc., -- VJ
	  PCI_ID = (PCI_OUR_DEV_ID << 16) | PCI_OUR_VENDOR_ID;
	  PCI_REV = (PCI_OUR_CLASS << 8) | PCI_OUR_REV;
	  PCI_SVID = (PCI_OUR_SS_DEV_ID << 16) | PCI_OUR_SS_VENDOR_ID;
	  PCI_CONTROL =  PCI_EN | INTA_EN|INB_ADDR_TR_EN |OUTB_END_EN;			//Leo, this and this only


	  PCI_MBAR_CONF = 0x11;				// standard 
	  PCI_IMBAR1_MASK = 0xf0000000;	// Inbound memory bar1 mask reg 16 M
	  PCI_IMEM1_TRANS = 0x00000000;	// Inbound memory1 trans reg SDRAM Address 
	  PCI_IMBAR2_MASK = 0xff000000;	// Inbound memory bar1 mask reg
	  PCI_IMEM2_TRANS = 0x19000000;	// Inbound memory2 trans reg SDRAM Address 

	  // Setup the PCI control register to allow us to be a bus master. Don't
	  // enable memory or I/O yet as the chip selects need to be setup.
	  //
	  // Also clear all bits set in status register
	  PCI_CRP_CTL = PCI_CRP_VAL (PCI_COMMAND, PCI_CMD_CFG_WR, 0);
	  PCI_CRP_WDAT = PCI_COMMAND_SERR
	    | PCI_COMMAND_PARITY
	    | PCI_COMMAND_MEMORY | PCI_COMMAND_MASTER | (0xffff << 16);


	  PCI_MBAP1 = PCI_MEM_BASE;	
	  PCI_MBAP2 = PCI_MEM_BASE + 0x01000000;	
	  PCI_MBAP3 = PCI_MEM_BASE + 0x02000000;
	  PCI_MBAP4 = PCI_MEM_BASE + 0x03000000;

	/** LUA pci patch bug 1662 **/
	{
	unsigned long chipid = *((unsigned long *)0xb900003c);
	switch (chipid) {
		case 0x6833: /* VX150 */
			printk("PCI reset 0x7FFF000B for VX-150\n");
			PCI_TAR_CTL = 0x7FFF000B; /* Use VX-150 reset value. */
			break;
		default:
			break;
	}
	}
	  mdelay (20);

	register_pci_controller(&fusiv_pci_controller);
        return 0;
}


int __init pcibios_map_irq(struct pci_dev *dev, u8 slot, u8 pin)
{
    PCI_CONTROL |= INTA_EN ;

    switch (slot) {
        case 1:
        case 2:
        case 3:
        case 4:
            return PCI_INT;   /* the only one*/
        default:
            return -1;            /* Illegal */
    }

}

/* Do platform specific device initialization at pci_enable_device() time */
int pcibios_plat_dev_init(struct pci_dev *dev)
{
	return 0;
}

__setup("idsel=", setpci_pin);
arch_initcall(fusiv_pci_setup);
