/* =============================================================================
 *
 * Copyright (C)  2005  Analog Devices Inc. All rights reserved
 *
 * The information and source code contained herein is the property
 * of Ikanos Communications.
 *
 *  This program is free software; you can redistribute  it and/or modify it
 *  under  the terms of  the GNU General  Public License as published by the
 *  Free Software Foundation;  either version 2 of the  License, or (at your
 *  option) any later version.
 * ===========================================================================*/
#include <linux/config.h>
#include <linux/init.h>
#include <linux/sched.h>
#include <linux/pci.h>
#include <linux/interrupt.h>
#include <linux/tty.h>
#include <linux/serial.h>
#include <linux/serial_core.h>
#include <linux/console.h>
#include <linux/irq.h>
#include <linux/delay.h>

#include <asm/serial.h>
#include <asm/io.h>
#include <asm/wbflush.h>
#include <asm/time.h>
#include <adi6843.h>
#include <vendor/epld.h>
#if defined CONFIG_SAGEM_BOXER || defined CONFIG_SAGEM_FAST34XX
#include <vendor/gpio_led_btn.h>
#endif
#include <linux/sbi_k.h>

extern void (*_machine_restart)(char *);
extern void (*_machine_halt)(void);
extern void (*_machine_power_off)(void);
extern void ad6843_startTimer(struct irqaction *irq);
extern void fusiv_init_IRQ(void);
#ifdef CONFIG_HW_BLUETOOTH
extern void ad6843_bt_init(void);
#endif

void  (*softResetPreparation4Vox150_ptr)( void ) =NULL;
unsigned char glo_uncmp_buffer [512*1024];

#define PCI_RST_CONTROL	*(volatile unsigned long *)(KSEG1ADDR(0x19050700))
#define REBOOT_CTRL	*(volatile unsigned long *)(KSEG1ADDR(0x19000000))

// Ajout define
#define CONFIG_SAGEM_WITH_HARDWARE_WATCHDOG_ADI

//hard_watchdog start
#define DEV_ID			*(volatile unsigned long *)(KSEG1ADDR(0x1900003c))
#define VOX200	0x6834
#define VOX150	0x6833
#define VOX160	0x6836
#ifdef CONFIG_SAGEM_WITH_HARDWARE_WATCHDOG_ADI

#define UINT32 unsigned long
#define UINT16 unsigned short
#define SUCCESS          0
#define FAILURE         (-1)

static struct timer_list HardwareWatchdogTimer;
#define CPU_CLOCK_RATE	100000000	/* CPU clock @ 125 MHz */

/* Define the System control watchdog control and status registers */

#define WD_CTRL         *(volatile UINT32 *)(KSEG1ADDR(0x19000008))
#define WD_RELOAD_SHFT  16
#define WD_RELOAD_MSK   0x0000ffff
#define WD_SERV_SHFT    8
#define WD_SERV_MSK     0x0000000f
#define WD_SERV_VAL     0x0000000a      /* Service value */
#define WD_PRESCALE_BIT 1               /* 1 ==> / 16K, 0 ==> /256 */
#define WD_DIS_BIT      0

#define RST_STAT        *(volatile UINT32 *)(KSEG1ADDR(0x19000004))
#define RST_SW_STAT_BIT 0               /* SW reset */
#define RST_HW_STAT_BIT 1               /* HW reset */
#define RST_WD_STAT_BIT 2               /* WDOG reset */
#define RST_POR_STAT_BIT 3              /* Power on reset */
#define RST_TAD_STAT_BIT 4              /* Test unit reset */
#define RST_RTT_STAT_BIT 5              /* Teal time timer reset */
#define RST_EXT_STAT_BIT 6              /* External reset */
#define WD_STAT        *(volatile UINT32 *)(KSEG1ADDR(0x1900000c))
#define WDT_ERR_CNT_SHFT	3
#define WDT_ERR_CNT_MSK		0x00000003

#define WD_PRESCALE_DIV 16384

static UINT32   HardwareWatchdogPacifyValue = 0 ;
static UINT32   HardwareWatchdogPacifyFrequency = 0 ;
static UINT32   HardwareWatchdogReset = 0 ;

//#define TEST_HARDWARE_WATCHDOG
#ifdef TEST_HARDWARE_WATCHDOG
static UINT32   HardwareWatchdogCpt = 0 ;  
#endif

/* ===========================================================================
 * Function name:       HardwareWatchdogPacify
 * Input Parameters:    p: ignored
 * Output Parameters:   none
 * Result:              none
 * Description:         writes into the watchdog register
 *                      restart the timer
 * ===========================================================================*/
void HardwareWatchdogPacify(void *p)
{
//// 	printk("PhT %s jiffies=%d\n",__FUNCTION__,jiffies) ;
   /* Pacify the watchdog */
    WD_CTRL = HardwareWatchdogPacifyValue;
    
#ifdef TEST_HARDWARE_WATCHDOG
if (HardwareWatchdogCpt++ == 30)
{
//	HardwareWatchdogTimer.entry.next = 1 ;	// provoque un BUG() dans le gestion des timer
	HardwareWatchdogPacifyFrequency = 0 ;   // on ne rearme pas le timer 
   	printk("\n%s Ca va tomber volontairement !!!!\n",__FUNCTION__) ;
}
#endif

if (HardwareWatchdogPacifyFrequency)
	{
// 	printk("PhT %s HardwareWatchdogPacifyFrequency=%d\n",__FUNCTION__,HardwareWatchdogPacifyFrequency) ;
	/* Start the timer */
	HardwareWatchdogTimer.expires = jiffies + HardwareWatchdogPacifyFrequency;
	add_timer(&HardwareWatchdogTimer);
	}

}

/* ===========================================================================
 * Function name:       HardwareWatchdogStopPacify
 * Input Parameters:    p: ignored
 * Output Parameters:   none
 * Result:              none
 * Description:         Don't restart the timer
 *                      
 * ===========================================================================*/
int HardwareWatchdogStopPacify(void)
{
	
//printk("%s \n",__FUNCTION__) ;
/* Don't Pacify the watchdog */
HardwareWatchdogPacifyFrequency = 0 ; 
return SUCCESS;

}

/* ===========================================================================
 * Function name:       HardwareWatchdogExit
 * Input Parameters:    p: ignored
 * Output Parameters:   none
 * Result:              none
 * Description:         
 *                      
 * ===========================================================================*/
int HardwareWatchdogExit(void)
{
int ret_val ;
	
//printk("%s \n",__FUNCTION__) ;
/* Stop the watchdog */

WD_CTRL = HardwareWatchdogPacifyValue | (1 << WD_DIS_BIT);

ret_val = HardwareWatchdogStopPacify() ;
del_timer(&HardwareWatchdogTimer);

return ret_val;

}


/* ===========================================================================
 * Function name:       HardwareWatchdogInitialize
 * Input Parameters:    timeout         The watchdog timeout time in ms
 *                      pacify          The time between pacifications in ms
 * Output Parameters:   none
 * Result:              0 - timer set successfully
 *                      non-zero - ERROR
 * Description:         Initialize the watchdog timer for the specified
 *                      period. Then start a software timer to generate
 *                      messages to the WPE at the specified pacify interval.
 *
 *                      The responses to these messages pacifies the timer
 * ===========================================================================*/
int HardwareWatchdogInitialize(UINT16 timeout_ms, UINT16 pacify_ms)
{
    UINT32      timeoutClocks;
//	printk("%s \n",__FUNCTION__) ;

unsigned long chipid = DEV_ID ;
printk("\n") ;	
if (chipid != VOX160)
	{
	printk("%s:: --- WATCHDOG -- NOT ACTIVATED (chip not VOX160 !) \n",__FUNCTION__);
	return FAILURE;		
	}
if (timeout_ms == 0)
	{
	printk("%s:: --- WATCHDOG -- NOT ACTIVATED (timeout=0 !)\n",__FUNCTION__);
	return FAILURE;
	}


if (pacify_ms > 0)
	{
	    /* Make sure the timeout is greater than the reload. If not then don't
     * continue
     */
    if (pacify_ms > timeout_ms)
		{
		printk("%s:: --- WATCHDOG -- NOT ACTIVATED (pacify > timeout !\n)",__FUNCTION__);
		return FAILURE;
		}

    /* Calculate the WD pacify value. The order of this calculation tries
     * to preserve as much precision as possible */
    timeoutClocks = (CPU_CLOCK_RATE / WD_PRESCALE_DIV) * timeout_ms / 1000;

    if (timeoutClocks > WD_RELOAD_MSK)
		{
    	printk("%s:: --- WATCHDOG -- NOT ACTIVATED (timeout > authorized value !\n)",__FUNCTION__);
        return FAILURE;
		}


    /* Form the pacify value to the timer */
    HardwareWatchdogPacifyValue =   (timeoutClocks << WD_RELOAD_SHFT)
                        			| (WD_SERV_VAL << WD_SERV_SHFT)
                        			| (0 << WD_PRESCALE_BIT)
                        			| (0 << WD_DIS_BIT);


    /* Start the watchdog hardware timer */
    WD_CTRL = HardwareWatchdogPacifyValue | (1 << WD_DIS_BIT);
    WD_CTRL = HardwareWatchdogPacifyValue ;
 	printk("%s :: --- WATCHDOG -- INITIALIZED with %d ms\n",__FUNCTION__,timeout_ms) ;
    

	
    /* Start the pacify timer */
    /* Calculate the WD pacify time. Round down to be sure that we
     * don't wait too long between pacifications
     */
	HardwareWatchdogPacifyFrequency = (pacify_ms/1000)*HZ ;
	init_timer(&HardwareWatchdogTimer);
	HardwareWatchdogTimer.function = HardwareWatchdogPacify ;
	HardwareWatchdogTimer.data = 0;
	HardwareWatchdogTimer.expires = jiffies + HardwareWatchdogPacifyFrequency ;
	add_timer(&HardwareWatchdogTimer);
 	printk("%s :: --- WATCHDOG -- Pacify timer of %d ms STARTED\n",__FUNCTION__,pacify_ms) ;
	}
else
	{
	/*  Traitement de pacification du watchdog autre qu'un timer linux */	
	/* pour le moment :    */
	printk("%s:: --- WATCHDOG -- NOT ACTIVATED (Pacify timeout=0 !)\n",__FUNCTION__);
	return FAILURE;
	}
	
return SUCCESS;
}

/* ===========================================================================
 * Function name:       HardwareWatchdogCauseReset
 * Input Parameters:    none
 * Output Parameters:   none
 * Result:              0           Last reset was not caused by watchdog
 *                      non-zero    Last reset was caused by watchdog
 * Description:         Determine whether the previous reset was caused by
 *                      a watchdog timeout or not.
 * ===========================================================================*/
UINT32 HardwareWatchdogCauseReset(void)
{
  	HardwareWatchdogReset = RST_STAT & (1 << RST_WD_STAT_BIT);
	return (HardwareWatchdogReset) ;
}
		
#endif    // CONFIG_SAGEM_WITH_HARDWARE_WATCHDOG_ADI
/* A copy of ad6843_restart with only one difference: due to a bug in VX150, we
 * must call softResetPreparation4Vox150 before doing the actual reset.
 * See remark in fusiv_src/kernel/ap_code/lib/netpro/apload.c:164.
 */
static void ad6833_restart( char *command )
{
if (! IS_CPU_VOX160) {
	/* Do a good, solid PCI bus reset */
	PCI_RST_CONTROL = 0x1;
	mdelay(1);
	PCI_RST_CONTROL = 0x3;
	mdelay(1);
	PCI_RST_CONTROL = 0x1;
	mdelay(1);
	
	/* Mmm, strange HW.
         * Seems that EPLD must be written before issuing sw reset, otherwise
         * hardware won't reset... */
        epld_reg_flags_reset(EPLD_REG0, 0xFF);

	if(softResetPreparation4Vox150_ptr == NULL)
	{
	    printk("\n Error:softResetPreparation4Vox150_ptr. Please restart MANUALLY...\n");
	}
	else
	{
	    (*softResetPreparation4Vox150_ptr)();
	}

	REBOOT_CTRL = 0x1;
	while ( 1 );
}
}

static void ad6843_restart( char *command )
{
	/* Do a good, solid PCI bus reset */
	PCI_RST_CONTROL = 0x1;
	mdelay(1);
	PCI_RST_CONTROL = 0x3;
	mdelay(1);
	PCI_RST_CONTROL = 0x1;
	mdelay(1);

if (! IS_CPU_VOX160) {
	/* Mmm, strange HW.
         * Seems that EPLD must be written before issuing sw reset, otherwise
         * hardware won't reset... */
        epld_reg_flags_reset(EPLD_REG0, 0xFF);
        if(softResetPreparation4Vox150_ptr == NULL)
        {
		printk("\n Error:softResetPreparation4Vox150_ptr. Please restart MANUALLY...\n");
	}
	else
	{
		(*softResetPreparation4Vox150_ptr)( );
	}	       
}
	REBOOT_CTRL = 0x1;
	while ( 1 );
}

static void ad6843_halt( void )
{
	printk("AD6843 Halted\n");
	while ( 1 );
}

#if defined CONFIG_SAGEM_BOXER || defined CONFIG_SAGEM_FAST34XX
static void ad6843_power_off( void )
{
       printk("AD6843 Power Off\n");
#ifdef CONFIG_SAGEM_BOXER
       gpio_led_btn_close();
       // set the power off GPIO
       gpio_led_btn_set(GPIO_CMD_POWER_OFF);
#endif
       while ( 1 );
}
#endif

/* Do AD6843-specific initialization in here */
void __init plat_setup(void)
{
	struct uart_port req;

// LUA reset patch bug 1663
	/* Identify the exact Fusiv CPU to set correct functions, currently
	 * only the restart differs.
	 * The identification algorithm used is a copy of cpu_probe_fusiv()
	 */
	unsigned long chipid = *((unsigned long *)0xb900003c);

	switch (chipid) {
	case 0x6834: /* VX200 */
		board_timer_setup = ad6843_startTimer;
		_machine_restart = ad6843_restart;
		_machine_halt = ad6843_halt;
		_machine_power_off = ad6843_halt;
	break;
	case 0x6833: /* VX150 */
		board_timer_setup = ad6843_startTimer;
		_machine_restart = ad6833_restart;
		_machine_halt = ad6843_halt;
		_machine_power_off = ad6843_halt;
	break;
	case 0x6836:
		board_timer_setup = ad6843_startTimer;
		_machine_restart = ad6843_restart;
		_machine_halt = ad6843_halt;
#ifdef CONFIG_SAGEM_BOXER
		_machine_power_off = ad6843_power_off;
#else
		_machine_power_off = ad6843_halt;
#endif
	break; /* VX160 */
	default:
	break;
	}

//hard_watchdog start
#ifdef CONFIG_SAGEM_WITH_HARDWARE_WATCHDOG_ADI

  	// Read watchdog status
	if (HardwareWatchdogCauseReset() == 0)
	{
		printk (":: NORMAL BOOT\n\n");
	}
	else
	{
		printk (":: - WARNING - CURRENT BOOT WAS CAUSED BY WATCHDOG !\n\n",__FUNCTION__);
	}
#endif	
//hard_watchdog end
	/* Setup serial port */
	memset(&req, 0, sizeof(req));
	req.line = 0;
	req.type = PORT_16450;
	req.irq = UART1_INT;
	req.flags = STD_COM_FLAGS;
	req.uartclk = 100 * 1000 * 1000;
	req.iotype = SERIAL_IO_MEM;
	req.membase = (unsigned char *)((ADI_6843_UART0_ADDR));
	req.mapbase = (unsigned long)(ADI_6843_UART0_ADDR);
	req.regshift = 2;
	req.fifosize = 1;
	early_serial_setup(&req);

	set_io_port_base(0);
if (!IS_CPU_VOX160)
	epld_reg_init();
#ifdef CONFIG_HW_BLUETOOTH
	ad6843_bt_init();
#endif
//#if defined(CONFIG_HW_USB_HOST_OHCI) || defined(CONFIG_HW_USB_HOST_EHCI)// OPN 20070919 BUG 6843 DECT 
//	ad6843_usbh_init();
//#endif
}

//void  (*softResetPreparation4Vox150_ptr)( void ) =NULL;
unsigned char glo_uncmp_buffer [512*1024];

const char *get_system_type(void)
{
	return "ADI Fusiv Core";
}

void __init arch_init_irq(void)
{
//	fusiv_init_IRQ();
	sbi->fusiv_init_IRQ();
}

#if CONFIG_FUSIV_KERNEL_PROFILER_MODULE

int (*loggerFunction)(unsigned long event) = NULL;

int loggerProfile(unsigned long event)
{
  if(loggerFunction != NULL)
    return loggerFunction(event);
  return -1;
}
void  loggerRegFunction( int (*func)(unsigned long event))
{
  loggerFunction = func;
}

EXPORT_SYMBOL(loggerRegFunction);
#endif
EXPORT_SYMBOL(softResetPreparation4Vox150_ptr);
EXPORT_SYMBOL(glo_uncmp_buffer);
#ifdef CONFIG_SAGEM_WITH_HARDWARE_WATCHDOG_ADI
EXPORT_SYMBOL(HardwareWatchdogInitialize);
EXPORT_SYMBOL(HardwareWatchdogExit);
#endif
