/******************************************************************************
*
*   Copyright (C) 2002 Analog Devices India (P) Ltd. All rights are reserved.
*
*   The information and source code contained herein is the exclusive
*   property of Analog Devices India (P) Ltd. and may not be disclosed,
*   examined or reproduced in whole or in part without explicit written
*   authorization from the compnay.
*
******************************************************************************/
                                                                                

#include <oswrapper.h>
#include <voicelib.h>
#include "../osdep/dspdrv.h"


#if PROFILE
extern int myTestValue[10000];
extern int myTestValueCnt;
#endif

UINT16  maxFrames[MAX_VOICE_CHANNELS];
UINT16  gDownloadOpts[MAX_DSP_NUM];                                                                                
#if FXO_MODULE
VoiceOpt_t      gVoiceOption[MAX_VOICE_CHANNELS + 1];
CHAR gFXOCallerIdDetect;
#else
VoiceOpt_t      gVoiceOption[MAX_VOICE_CHANNELS];
#endif

JitterParams_t  gJitterParams;
INT8            gaSlicPortStatus[MAX_VOICE_CHANNELS];

// Sagem 20080226
UINT16   guiCadenceOn[NUM_COUNTRY]={200,	// USA
                                   150,		// FRANCE 
                                   100, 	// NETHERLANDS 
                                   100, 	// POLAND 
                                   150,		// SPAIN 
                                   100	 	// BELGIUM 
                                  };
UINT16   guiCadenceOff[NUM_COUNTRY]={400, 	// USA 
                                    350, 	// FRANCE 
                                    400, 	// NETHERLANDS 
                                    400,  	// POLAND 
                                    350, 	// SPAIN 
                                    300 	// BELGIUM 
                                   };
/* First RING ON period */
UINT16   guiFirstCadenceOn[NUM_COUNTRY]={200,	// USA 
                                         25,	// FRANCE 
                                         25,	// NETHERLANDS 
                                         25,	// POLAND 
                                         25, 	// SPAIN 
                                         25 	// BELGIUM 
                                        };
/* First RING OFF period */
UINT16   guiFirstCadenceOff[NUM_COUNTRY]={400,	// USA 
                                          250,	// FRANCE 
                                          250,	// NETHERLANDS 
                                          250,	// POLAND
                                          250,	// SPAIN
                                          250	// BELGIUM
                                         };

UINT16 guiCountry;
UINT16 guiCountryBackup;
UCHAR  gStoreConfig = 0;
#if THREEWAY_CONFERENCE
UCHAR gDSPConference[MAX_SLIC_CHANNELS];
#endif

/* SAGEM -> [Warning] { */
#if DECT_DEBUG
extern void dectPrintDectStat(void);
#endif
/* } */

/*************************************************************************
 *  Function Name : confInitVoiceOptions.                                *
 *  Description   : Function to initialize Gain Controls.                *
 *  Input         : Port Number.                                         *
 *  Output        : None.                                                *
 *  Return Value  : None.                                                *
 ************************************************************************/
void confInitGainControl(UCHAR ucPort)
{
  INT16 uiGain;
  INT16 uiBlkGain;
  if(gStoreConfig == 1)
  {
    uiGain = ( (6- gVoiceOption[ucPort].lGrx) << 8 | 
             (6- gVoiceOption[ucPort].lGtx));
    uiBlkGain= (6- gVoiceOption[ucPort].lGec) & 0xFF;
    dspipcGainCntrl(ucPort,uiGain,uiBlkGain);
  }
}

void voiceCliConfigStored(void)
{
  /* DSP Ports are not Initialized yet */
  UINT16 iIndex;
  for(iIndex = 0; iIndex < MAX_VOICE_CHANNELS; iIndex++)
    dspipcInitPortDone(iIndex,0);
  gStoreConfig = TRUE;
} 



INT16 voiceCliCmd(VOICELS *pVoice, UINT32 cmd)
{
  pVoice->bErr = FALSE;
  /* Port Number should not exceed MAX Port Numbers */
#if FXO_MODULE 
  if(pVoice->ucPortNumber >= (MAX_VOICE_CHANNELS+1))
#else
  if(pVoice->ucPortNumber >= MAX_VOICE_CHANNELS)
#endif
    pVoice->ucPortNumber =0;
  switch(cmd)
  {
    case DSP_SET_VOICEOPT: /* Set Voice Options */
      gVoiceOption[pVoice->ucPortNumber].bVad = pVoice->bVad;
      gVoiceOption[pVoice->ucPortNumber].bPLC = pVoice->bPLC;
      gVoiceOption[pVoice->ucPortNumber].bEC  = pVoice->bEC;
      gVoiceOption[pVoice->ucPortNumber].uiTailLen = pVoice->uiTailLen;
//VAD2
      gVoiceOption[pVoice->ucPortNumber].ucVadType = pVoice->ucVadType;
      gVoiceOption[pVoice->ucPortNumber].ucPcmMode = pVoice->ucPcmMode;
      if(gVoiceOption[pVoice->ucPortNumber].ucPcmMode > 2)
        gVoiceOption[pVoice->ucPortNumber].ucPcmMode = SPORT_CODEC;
      DEBUG(PRINT("DSP_SET_VOICEOPT => \n Port: %d\n VAD: %d\n G711VadType: %d\n PLC: %d\n EC: %d\n TailLength: %d\n PCM-Mode: %d\n", pVoice->ucPortNumber,pVoice->bVad,pVoice->ucVadType, pVoice->bPLC,pVoice->bEC, pVoice->uiTailLen,pVoice->ucPcmMode), DEBUG_CONF);
      break;
    case DSP_SET_JBPARAMS: /* Set Jitter Buffer Type */
      if((pVoice->uiThreshold == 0) && (pVoice->bJBType == FJB))
      {
         gJitterParams.uiThreshold=20;
      }
      else
        gJitterParams.uiThreshold = pVoice->uiThreshold;
      if(pVoice->uiPassThrThreshold == 0)
        gJitterParams.uiPassThrThreshold = 100;      
      else 
        gJitterParams.uiPassThrThreshold = pVoice->uiPassThrThreshold;
      gJitterParams.bJBType =  pVoice->bJBType;
      if(pVoice->bJBType == FJB)
        switchJitterBuffToFjb(pVoice->ucPortNumber);
      DEBUG(PRINT("DSP_SET_JBPARAMS => \n Port: %d\n JB Type: %d\n Threshold: %d\n Pass-Through Threshold: %d\n",
		pVoice->ucPortNumber,
                gJitterParams.bJBType,
                pVoice->uiThreshold,
                pVoice->uiPassThrThreshold), DEBUG_CONF);
                
      break;
    case DSP_SET_COUNTRYOPT: /* Set Country Options */
      {
        UCHAR ucPort;
        guiCountry = pVoice->ucCountryOpt;
        /* BUG ID : 4084 */
#if SLIC_MODULES        
        for(ucPort = 0; ucPort< MAX_VOICE_CHANNELS; ucPort++)
        {
          if(pVoice->ucCountryOpt == FRANCE)
          {
/* SAGEM -> [ DIAL_TONE Down Prb ] { */
#if 0
            if(gVoiceOption[ucPort].lGtx > -6)
              gVoiceOption[ucPort].lGtx = -6;
#endif
/* } */
            gVoiceOption[ucPort].lGtx = 0;
          }
          confInitGainControl(ucPort);
       }
#endif
        DEBUG(PRINT("DSP_SET_COUNTRYOPT => \nPort: %d\n Country: %d\n",
                       pVoice->ucPortNumber,pVoice->ucCountryOpt),DEBUG_CONF);
      }
      break;
    case DSP_SET_PCMMODE:
      gVoiceOption[pVoice->ucPortNumber].ucPcmMode = pVoice->ucPcmMode;
      DEBUG(PRINT("DSP_SET_PCMMODE => Port: %d\nPCM-Mode: %d\n",
             pVoice->ucPortNumber,pVoice->ucPcmMode),DEBUG_CONF);
      if(gStoreConfig)
        printk("Save and reboot the system to effect the Codec Mode : %d\n",pVoice->ucPcmMode);
      break;
    case DSP_SET_GCPARAM: /* Set Gain Control  Parameters */
      if(pVoice->lGtx <= -15 || pVoice->lGtx > 6)
        return -1;
      if(pVoice->lGrx <= -15 || pVoice->lGrx > 6)
        return -1;
      if(pVoice->lGec <= -15 || pVoice->lGec >6)
        return -1;
      gVoiceOption[pVoice->ucPortNumber].lGrx = pVoice->lGrx;
      gVoiceOption[pVoice->ucPortNumber].lGtx = pVoice->lGtx;
      gVoiceOption[pVoice->ucPortNumber].lGec = pVoice->lGec;
      confInitGainControl(pVoice->ucPortNumber);
      DEBUG(PRINT("DSP_SET_GCPARAM => \nPort: %d\n GTX: %d\n GRX: %d\n GEC: %d\n",
		pVoice->ucPortNumber,pVoice->lGtx,pVoice->lGrx,pVoice->lGec),DEBUG_CONF);
      break;
    case DSP_GET_VOICEOPT: /* Get Voice Options */
      pVoice->bVad = gVoiceOption[pVoice->ucPortNumber].bVad;
      pVoice->bPLC = gVoiceOption[pVoice->ucPortNumber].bPLC;
      pVoice->bEC  = gVoiceOption[pVoice->ucPortNumber].bEC;
      pVoice->uiTailLen = gVoiceOption[pVoice->ucPortNumber].uiTailLen;
//VAD2
      pVoice->ucVadType = gVoiceOption[pVoice->ucPortNumber].ucVadType;
      pVoice->ucPcmMode = gVoiceOption[pVoice->ucPortNumber].ucPcmMode;
     DEBUG(PRINT("DSP_GET_VOICEOPT  => \n Port: %d\n VAD: %d\n G711VadType: %d\n PLC: %d\n EC: %d\n TailLength: %d\n PCM-Mode: %d\n", pVoice->ucPortNumber,pVoice->bVad, pVoice->ucVadType, pVoice->bPLC,pVoice->bEC, pVoice->uiTailLen,pVoice->ucPcmMode), DEBUG_CONF);
     break;
#if THREEWAY_CONFERENCE    
    case DSP_SET_CONFERENCE:
      gDSPConference[pVoice->ucDspNum] = pVoice->bConference;
      break;
    case DSP_GET_CONFERENCE:
      pVoice->bConference = gDSPConference[pVoice->ucDspNum];
      break;
#endif
    case DSP_GET_JBPARAMS: /* Get Jitter Parameters */
      pVoice->bJBType = gJitterParams.bJBType;
      pVoice->uiThreshold = gJitterParams.uiThreshold;
      pVoice->uiPassThrThreshold = gJitterParams.uiPassThrThreshold;
      DEBUG(PRINT("DSP_GET_GCPARAM => \nPort: %d\n GTX: %d\n GRX: %d\n GEC: %d\n",
                pVoice->ucPortNumber,pVoice->lGtx,pVoice->lGrx,pVoice->lGec),DEBUG_CONF);
      break;
    case DSP_NEXT_GCPARAM:
      if(  (cmd == DSP_NEXT_GCPARAM) && (pVoice->lIndex >= MAX_VOICE_CHANNELS))
      {
        pVoice->lIndex = -1;
        pVoice->bErr = TRUE;
        break;
      }
      pVoice->ucPortNumber = pVoice->lIndex;
      pVoice->lGtx= gVoiceOption[pVoice->lIndex].lGtx;
      pVoice->lGrx = gVoiceOption[pVoice->lIndex].lGrx;
      pVoice->lGec = gVoiceOption[pVoice->lIndex].lGec;
      pVoice->lIndex++;
      DEBUG(PRINT("DSP_NEXT_GCPARAM => \nIndex: %d\n GTX: %d\n GRX: %d\n GEC: %d\n",
                pVoice->ucPortNumber,pVoice->lGtx,pVoice->lGrx,pVoice->lGec),DEBUG_CONF);
      break;
    case DSP_GET_COUNTRYOPT: /* Get Country Options */
      pVoice->ucCountryOpt = guiCountry;
      DEBUG(PRINT("DSP_GET_COUNTRYOPT => \nPort: %d\n Country: %d\n",
                       pVoice->ucPortNumber,pVoice->ucCountryOpt),DEBUG_CONF);
#if DECT_DEBUG
      dectPrintDectStat();
#endif
      break;
    case DSP_SET_DWNLOADOPTS: /* Set Download Options */
      gDownloadOpts[pVoice->ucDspNum] = pVoice->ucDownloadOpts;
      break;   
    case DSP_GET_DWNLOADOPTS: /* Get Download Options */
      pVoice->ucDownloadOpts = gDownloadOpts[pVoice->ucDspNum];
      break;   
    case DSP_GET_PCMMODE: /* Get Sport Codec Type*/
      pVoice->ucPcmMode = gVoiceOption[pVoice->ucPortNumber].ucPcmMode;
      DEBUG(PRINT("DSP_GET_PCMMODE => \n \nPort: %d\n PCM-Mode: %d\n",
		 pVoice->ucPortNumber,pVoice->ucPcmMode), DEBUG_CONF);
      break;
    case DSP_GET_VAD:
      pVoice->bVad = gVoiceOption[pVoice->ucPortNumber].bVad;
//VAD2
      pVoice->ucVadType = gVoiceOption[pVoice->ucPortNumber].ucVadType;
      DEBUG(PRINT("DSP_GET_VAD => \n \nPort: %d\n VAD: %d\n G711VadType: %d\n",
		 pVoice->ucPortNumber,pVoice->bVad,pVoice->ucVadType), DEBUG_CONF);
      break;
    case DSP_GET_PLC:
      pVoice->bPLC = gVoiceOption[pVoice->ucPortNumber].bPLC;
      DEBUG(PRINT("DSP_GET_PLC => \n \nPort: %d\n PLC: %d\n",
		 pVoice->ucPortNumber,pVoice->bPLC), DEBUG_CONF);
      break;
    case DSP_SET_EC:
      gVoiceOption[pVoice->ucPortNumber].bEC = pVoice->bEC;
      gVoiceOption[pVoice->ucPortNumber].uiTailLen = pVoice->uiTailLen;
      DEBUG(PRINT("DSP_SET_EC => \n \nPort: %d\n EC: %d\n TailLength: %d\n",
                 pVoice->ucPortNumber,pVoice->bEC,pVoice->uiTailLen), DEBUG_CONF);

      break;
    case DSP_SET_VAD:
      gVoiceOption[pVoice->ucPortNumber].bVad = pVoice->bVad;
//VAD2
      gVoiceOption[pVoice->ucPortNumber].ucVadType = pVoice->ucVadType;
      DEBUG(PRINT("DSP_SET_VAD => \n \nPort: %d\n VAD: %d\n G711VadType: %d\n",
                 pVoice->ucPortNumber,pVoice->bVad, pVoice->ucVadType), DEBUG_CONF);
      break;
    case DSP_SET_PLC:
      gVoiceOption[pVoice->ucPortNumber].bPLC = pVoice->bPLC ;
      DEBUG(PRINT("DSP_SET_PLC => \n \nPort: %d\n PLC: %d\n",
                 pVoice->ucPortNumber,pVoice->bPLC), DEBUG_CONF);
      break;
    case DSP_GET_EC:
      pVoice->bEC = gVoiceOption[pVoice->ucPortNumber].bEC;
      pVoice->uiTailLen = gVoiceOption[pVoice->ucPortNumber].uiTailLen;
      DEBUG(PRINT("DSP_GET_EC => \n \nPort: %d\n EC: %d\n TailLength: %d\n",
                 pVoice->ucPortNumber,pVoice->bEC,pVoice->uiTailLen), DEBUG_CONF);
      break;

    case DSP_SET_FAX_MODE:
      gVoiceOption[pVoice->ucPortNumber].ucFaxMode = pVoice->ucFaxMode;
      if(pVoice->ucFaxMode == FAX_PASSTHRU_METHOD)
         dspipcFaxSG3ToG3Fb(pVoice->ucPortNumber,0);
      else
         dspipcFaxSG3ToG3Fb(pVoice->ucPortNumber,1);
      DEBUG(PRINT("DSP_SET_FAX_MODE => \n \nPort: %d\n FAX Mode: %d\n",
                 pVoice->ucPortNumber,pVoice->ucFaxMode), DEBUG_CONF);
      break;
    case DSP_GET_FAX_MODE:
      pVoice->ucFaxMode = gVoiceOption[pVoice->ucPortNumber].ucFaxMode;
      DEBUG(PRINT("DSP_GET_FAX_MODE => \n \nPort: %d\n FAX Mode: %d\n",
                 pVoice->ucPortNumber,pVoice->ucFaxMode), DEBUG_CONF);
      break;
    default:
      DEBUG(PRINT("Invalid Command for the Configurations => %x\n",cmd),DEBUG_EXCEPTION);
  }
  return 0;
}

void confSetDSPDownloadOpt(UINT16 dspNum, UCHAR downloadOpt)
{
  gDownloadOpts[dspNum] = downloadOpt;
}

UCHAR confGetDSPDownloadOpt(UINT16 dspNum)
{
  return gDownloadOpts[dspNum];
}


#if THREEWAY_CONFERENCE
UCHAR confGetConference(UINT16 dspNum)
{
  return gDSPConference[dspNum];
}
#endif



UCHAR confGetJBType(void)
{
   return gJitterParams.bJBType;
}

UCHAR confGetJitterThreshold(void)
{
   return gJitterParams.uiThreshold;
}
UINT16 confFirstCadenceOn(UINT16 uiCountry )
{
  return guiFirstCadenceOn[uiCountry];
}
UINT16 confFirstCadenceOff(UINT16 uiCountry )
{
  return guiFirstCadenceOff[uiCountry];
}
UINT16 confCadenceOn(UINT16 uiCountry )
{
  return guiCadenceOn[uiCountry];
}
UINT16 confCadenceOff(UINT16 uiCountry )
{
  return guiCadenceOff[uiCountry];
}
VOID confSetMaxFrames(UINT16 portNum,UINT16 maxFrame)
{
  maxFrames[portNum] = maxFrame;
}
UINT16 confGetMaxFrames(UINT16 portNum)
{
  return maxFrames[portNum];
}

VOID confGetCountryOpts(UINT16 *puiCountry)
{
    *puiCountry=guiCountry;
}

//AN
VOID confSetCountryOpts(UINT16 *puiCountry)
{
    guiCountry = *puiCountry;
}
//AN

UCHAR confGetPCMMode(UINT16 uiPort)
{
  return gVoiceOption[uiPort].ucPcmMode;
}

UCHAR confGetPLC(UINT16 uiPort)
{
  return gVoiceOption[uiPort].bPLC;
}


void confSetJitterThreshold(int Threshold)
{
  gJitterParams.uiThreshold = Threshold;
}



VOID  confSetPLC(UINT16 uiPort,UCHAR plcMode)
{
  gVoiceOption[uiPort].bPLC = plcMode;
}

UCHAR confGetFAXMode(UINT16 uiPort)
{
  return gVoiceOption[uiPort].ucFaxMode;
}
#if FXO_MODULE
void confSetToneDetect(UCHAR port, UCHAR toneEnable)
{
  gVoiceOption[port].fxoToneDetect = toneEnable;
}
UCHAR confGetToneDetect(UCHAR port)
{
  return gVoiceOption[port].fxoToneDetect;
}
void confSetFXOCallerIdDetect(UCHAR port, UCHAR Enable)
{
  gVoiceOption[port].fxoCallerIdDetect = Enable;
  if(Enable)
    gFXOCallerIdDetect++;
  else if(gFXOCallerIdDetect != 0)
    gFXOCallerIdDetect--;
}
UCHAR confGetFXOCallerIdDetectEnabled(void)
{
  if(gFXOCallerIdDetect > 0)
    return 1;
  else 
    return 0;
}

UCHAR confGetFXSCallerIdEnabled(UCHAR port)
{
  return gVoiceOption[port].fxoCallerIdDetect;
}
#endif

//VAD2
UCHAR confGetG711VadType(UINT16 uiPort)
{
  return gVoiceOption[uiPort].ucVadType;
}
/* IKANOS- 07/20/07 */
void confSetJBType(UCHAR type)
{
   gJitterParams.bJBType = type;
}

UCHAR confGetJitterPassThrThreshold(void)
{
   return gJitterParams.uiPassThrThreshold;
}
