/*
 * ============================================================================
 *
 * Copyright : (c) 2005 SAGEM SA - Site Velizy - URD2
 *
 * The information and source code contained herein is the exclusive
 * property of SAGEM and may not be disclosed, examined, or
 * reproduced in whole or in part without explicit written authorization
 * of the company.
 *
 * ============================================================================
 */

/**
 * \file dspSysCtl.c /proc/sys interface
 *
 * This file contain all /rpoc/sys interface.
 */

/*
 * Common RTP Includes
 * ~~~~~~~~~~~~~~~~~~~
 */

#include <linux/string.h>

#include "oswrapper.h"
#include "voicelib.h"
#include "voiceSysCtl.h"

#ifdef VOICE_SYSCTL_STATS

/*
 * Externs
 * ~~~~~~~
 */

/* Voice module */
extern uWord16   AJB_READ_THRESHOLD[MAX_VOICE_CHANNELS];
extern JitBuffType_  SMMJitBuff[MAX_VOICE_CHANNELS];
extern VoiceCirQ_t gVoiceTxQ[MAX_VOICE_CHANNELS];
extern VoiceCirQ_t gaDsp2HostQ;
extern VoiceCirQ_t gaFxo2HostQ;
extern UCHAR dspDataMode[MAX_VOICE_CHANNELS];
extern UCHAR gDataModeEnable;
extern SLICInfo_t gaSlicInfo[MAX_SLIC_CHANNELS];
extern DspStateInfo_t gaDspStateInfo[MAX_VOICE_CHANNELS];

/** sysctl dsp_proc_data_mode_port. */
int dsp_proc_data_mode_port(		ctl_table		*_pTable,
							int			_write,
							struct file	*_pFilp,
		     				void __user 	*_pBuffer,
							size_t		*_pLenp,
							loff_t		*_pPpos);

/** sysctl dsp_proc_init_port. */
int dsp_proc_init_port_port	(	ctl_table		*_pTable,
							int			_write,
							struct file	*_pFilp,
		     				void __user 	*_pBuffer,
							size_t		*_pLenp,
							loff_t		*_pPpos);

/** sysctl dsp_proc_slic_state_port. */
int dsp_proc_slic_state_port(		ctl_table		*_pTable,
							int			_write,
							struct file	*_pFilp,
		     				void __user 	*_pBuffer,
							size_t		*_pLenp,
							loff_t		*_pPpos);

/** sysctl dsp_proc_ring_state_port. */
int dsp_proc_ring_state_port(		ctl_table		*_pTable,
							int			_write,
							struct file	*_pFilp,
		     				void __user 	*_pBuffer,
							size_t		*_pLenp,
							loff_t		*_pPpos);

/** sysctl dsp_proc_data_mode. */
int dsp_proc_data_mode(			ctl_table		*_pTable,
							int			_write,
							struct file	*_pFilp,
		     				void __user 	*_pBuffer,
							size_t		*_pLenp,
							loff_t		*_pPpos);

/** sysctl dsp_proc_stats_endpoint. */
int dsp_proc_stats_endpoint(		ctl_table		*_pTable,
							int			_write,
							struct file	*_pFilp,
		     				void __user 	*_pBuffer,
							size_t		*_pLenp,
							loff_t		*_pPpos);

/** sysctl dsp_proc_stats_network. */
int dsp_proc_stats_network(		ctl_table		*_pTable,
					int			_write,
					struct file	*_pFilp,
		     		void __user 	*_pBuffer,
					size_t		*_pLenp,
					loff_t		*_pPpos);

/** sysctl dsp_proc_device_evts. */
int dsp_proc_device_evts(		ctl_table		*_pTable,
							int			_write,
							struct file	*_pFilp,
		     				void __user 	*_pBuffer,
							size_t		*_pLenp,
							loff_t		*_pPpos);

/** sysctl dsp_proc_voice_ready_port. */
int dsp_proc_voice_ready_port(		ctl_table		*_pTable,
								int			_write,
								struct file	*_pFilp,
		     					void __user 	*_pBuffer,
								size_t		*_pLenp,
								loff_t		*_pPpos);

/** sysctl dsp_proc_ajb. */
int dsp_proc_ajb(		ctl_table		*_pTable,
					int			_write,
					struct file	*_pFilp,
		     		void __user 	*_pBuffer,
					size_t		*_pLenp,
					loff_t		*_pPpos);

/*
 * Globals
 * ~~~~~~~
 */

char dsp_tampon[1024];
int	dsp_port_0		= 0;
int	dsp_port_1		= 1;

/** Base statistics. */
struct dsp_voiceSysCtl_str dsp_voiceSysCtl = {
	.head						= NULL,
	.sagem						= {
		{
			.ctl_name				= VOICE_SYSCTL_Sagem,
			.procname				= "sagem",
			.mode				= 0555,
			.child				= dsp_voiceSysCtl.sagem_voice,
		},
		{	.ctl_name				= 0,
		},
	},
	.sagem_voice							= {
		{
			.ctl_name						= VOICE_SYSCTL_Sagem_voice,
			.procname						= "voice",
			.mode						= 0555,
			.child						= dsp_voiceSysCtl.sagem_voice_list,
		},
		{	.ctl_name						= 0,
		},
	},
	.sagem_voice_list						= {
		{
			.ctl_name						= VOICE_SYSCTL_Sagem_voice_port_0,
			.procname						= "port_0",
			.mode						= 0555,
			.child						= dsp_voiceSysCtl.ports[0].sysctl.sagem_voice_port,
		},
		{
			.ctl_name						= VOICE_SYSCTL_Sagem_voice_port_1,
			.procname						= "port_1",
			.mode						= 0555,
			.child						= dsp_voiceSysCtl.ports[1].sysctl.sagem_voice_port,
		},
		{
			.ctl_name						= VOICE_SYSCTL_Sagem_voice_data_mode,
			.procname						= "dataMode",
			.data						= &(gDataModeEnable),
			.maxlen						= sizeof(gDataModeEnable),
			.mode						= 0666,
			.proc_handler					= &dsp_proc_data_mode,
		},
		{
			.ctl_name						= VOICE_SYSCTL_Sagem_voice_dev_nb_evts,
			.procname						= "device_evts",
			.data						= dsp_tampon,
			.maxlen						= sizeof(dsp_tampon),
			.mode						= 0666,
			.proc_handler					= &dsp_proc_device_evts,
		},
		{	.ctl_name						= 0,
		},
	},
	.ports								= {
		{
			.sysctl.sagem_voice_port			= {
				{
					.ctl_name				= VOICE_SYSCTL_Sagem_voice_port_stats,
					.procname				= "stats",
					.mode				= 0555,
					.child				= dsp_voiceSysCtl.ports[0].sysctl.sagem_voice_port_stats,
				},
				{
					.ctl_name				= VOICE_SYSCTL_Sagem_voice_port_params,
					.procname				= "params",
					.mode				= 0555,
					.child				= dsp_voiceSysCtl.ports[0].sysctl.sagem_voice_port_params,
				},
				{
					.ctl_name				= VOICE_SYSCTL_Sagem_voice_port_ajb,
					.procname				= "ajb",
					.data				= &dsp_port_0,
					.maxlen				= sizeof(dsp_port_0),
					.mode				= 0666,
					.proc_handler			= &dsp_proc_ajb,
				},
				{	.ctl_name				= 0,
				},
			},
			.sysctl.sagem_voice_port_stats	= {
				{
					.ctl_name				= VOICE_SYSCTL_Sagem_voice_port_stats_endPoint,
					.procname				= "endPoint",
					.data				= &(dsp_voiceSysCtl.ports[0].stats.endPoint),
					.maxlen				= sizeof(dsp_voiceSysCtl.ports[0].stats.endPoint),
					.mode				= 0666,
					.proc_handler			= &dsp_proc_stats_endpoint,
				},
				{
					.ctl_name				= VOICE_SYSCTL_Sagem_voice_port_stats_network,
					.procname				= "network",
					.data				= &(dsp_voiceSysCtl.ports[0].stats.network),
					.maxlen				= sizeof(dsp_voiceSysCtl.ports[0].stats.network),
					.mode				= 0666,
					.proc_handler			= &dsp_proc_stats_network,
				},
				{	.ctl_name				= 0,
				},
			},
			.sysctl.sagem_voice_port_params		= {
				{
					.ctl_name				= VOICE_SYSCTL_Sagem_voice_port_params_data_mode,
					.procname				= "dataMode",
					.data				= &(dspDataMode[0]),
					.maxlen				= sizeof(dspDataMode[0]),
					.mode				= 0666,
					.proc_handler			= &dsp_proc_data_mode_port,
				},
				{
					.ctl_name				= VOICE_SYSCTL_Sagem_voice_port_params_init_port,
					.procname				= "initPort",
					.data				= &(gaDspStateInfo[0].bInitDone),
					.maxlen				= sizeof(gaDspStateInfo[0].bInitDone),
					.mode				= 0666,
					.proc_handler			= &dsp_proc_init_port_port,
				},
				{
					.ctl_name				= VOICE_SYSCTL_Sagem_voice_port_params_slic_state,
					.procname				= "slicState",
					.data				= &(gaSlicInfo[0].slicOffTheHook),
					.maxlen				= sizeof(gaSlicInfo[0].slicOffTheHook),
					.mode				= 0666,
					.proc_handler			= &dsp_proc_slic_state_port,
				},
				{
					.ctl_name				= VOICE_SYSCTL_Sagem_voice_port_params_ring_state,
					.procname				= "ringState",
					.data				= &(gaSlicInfo[0].ucSlicState),
					.maxlen				= sizeof(gaSlicInfo[0].ucSlicState),
					.mode				= 0666,
					.proc_handler			= &dsp_proc_ring_state_port,
				},
				{
					.ctl_name				= VOICE_SYSCTL_Sagem_voice_port_params_voice_ready,
					.procname				= "voiceReady",
					.data				= &(gVoiceTxQ[0]),
					.maxlen				= sizeof(gVoiceTxQ[0]),
					.mode				= 0666,
					.proc_handler			= &dsp_proc_voice_ready_port,
				},
				{	.ctl_name				= 0,
				},
			},
		},
		{
			.sysctl.sagem_voice_port			= {
				{
					.ctl_name				= VOICE_SYSCTL_Sagem_voice_port_stats,
					.procname				= "stats",
					.mode				= 0555,
					.child				= dsp_voiceSysCtl.ports[1].sysctl.sagem_voice_port_stats,
				},
				{
					.ctl_name				= VOICE_SYSCTL_Sagem_voice_port_params,
					.procname				= "params",
					.mode				= 0555,
					.child				= dsp_voiceSysCtl.ports[1].sysctl.sagem_voice_port_params
				},
				{
					.ctl_name				= VOICE_SYSCTL_Sagem_voice_port_ajb,
					.procname				= "ajb",
					.data				= &dsp_port_1,
					.maxlen				= sizeof(dsp_port_1),
					.mode				= 0666,
					.proc_handler			= &dsp_proc_ajb,
				},
				{	.ctl_name				= 0,
				},
			},
			.sysctl.sagem_voice_port_stats	= {
				{
					.ctl_name				= VOICE_SYSCTL_Sagem_voice_port_stats_endPoint,
					.procname				= "endPoint",
					.data				= &(dsp_voiceSysCtl.ports[1].stats.endPoint),
					.maxlen				= sizeof(dsp_voiceSysCtl.ports[1].stats.endPoint),
					.mode				= 0666,
					.proc_handler			= &dsp_proc_stats_endpoint,
				},
				{
					.ctl_name				= VOICE_SYSCTL_Sagem_voice_port_stats_network,
					.procname				= "network",
					.data				= &(dsp_voiceSysCtl.ports[1].stats.network),
					.maxlen				= sizeof(dsp_voiceSysCtl.ports[1].stats.network),
					.mode				= 0666,
					.proc_handler			= &dsp_proc_stats_network,
				},
				{	.ctl_name				= 0,
				},
			},
			.sysctl.sagem_voice_port_params		= {
				{
					.ctl_name				= VOICE_SYSCTL_Sagem_voice_port_params_data_mode,
					.procname				= "dataMode",
					.data				= &(dspDataMode[1]),
					.maxlen				= sizeof(dspDataMode[1]),
					.mode				= 0666,
					.proc_handler			= &dsp_proc_data_mode_port,
				},
				{
					.ctl_name				= VOICE_SYSCTL_Sagem_voice_port_params_init_port,
					.procname				= "initPort",
					.data				= &(gaDspStateInfo[1].bInitDone),
					.maxlen				= sizeof(gaDspStateInfo[1].bInitDone),
					.mode				= 0666,
					.proc_handler			= &dsp_proc_init_port_port,
				},
				{
					.ctl_name				= VOICE_SYSCTL_Sagem_voice_port_params_slic_state,
					.procname				= "slicState",
					.data				= &(gaSlicInfo[1].slicOffTheHook),
					.maxlen				= sizeof(gaSlicInfo[1].slicOffTheHook),
					.mode				= 0666,
					.proc_handler			= &dsp_proc_slic_state_port,
				},
				{
					.ctl_name				= VOICE_SYSCTL_Sagem_voice_port_params_ring_state,
					.procname				= "ringState",
					.data				= &(gaSlicInfo[1].ucSlicState),
					.maxlen				= sizeof(gaSlicInfo[1].ucSlicState),
					.mode				= 0666,
					.proc_handler			= &dsp_proc_ring_state_port,
				},
				{
					.ctl_name				= VOICE_SYSCTL_Sagem_voice_port_params_voice_ready,
					.procname				= "voiceReady",
					.data				= &(gVoiceTxQ[1]),
					.maxlen				= sizeof(gVoiceTxQ[1]),
					.mode				= 0666,
					.proc_handler			= &dsp_proc_voice_ready_port,
				},
				{	.ctl_name				= 0,
				},
			},
		},
	}
};

/*
 * Functions
 * ~~~~~~~~~
 */

/** sysctl dsp_proc_data_mode_port.
 *
 *   \par Description
 *	<b>This function allow to print data mode on sysctl.</b>\n
 *
 *	@param _pTable:	Sysctl table reference.
 *	@param _write:		If write.
 *	@param _pFile:		File reference.
 *	@param _pBuffer:	Buffer reference.
 *	@param _pLen:		Length reference.
 *	@param _pPpos:		Position reference.
 *
 *	@return proc_dostring return.
 */
int dsp_proc_data_mode_port(	ctl_table		*_pTable,
						int			_write,
						struct file	*_pFilp,
		     			void __user 	*_pBuffer,
						size_t		*_pLenp,
						loff_t		*_pPpos)
{
	int			_res;
	unsigned char	_modeData		= *((unsigned char *)(_pTable->data));

	switch(_modeData) {
		case DSP_DATA_INACTIVE:
			strcpy(	dsp_tampon,
					"INACTIVE");
			break;

		case DSP_DATA_SEND_ONLY:
			strcpy(	dsp_tampon,
					"SEND_ONLY");
			break;

		case DSP_DATA_RECV_ONLY:
			strcpy(	dsp_tampon,
					"RECV_ONLY");
			break;

		case DSP_DATA_SEND_RECV:
			strcpy(	dsp_tampon,
					"SEND_RECV");
			break;

		default:
			strcpy(	dsp_tampon,
					"INCONNU");
			break;
	}
		
	/* Perform function */
	{
		void 	 	*_pOldData;
		size_t		_pOldMaxlen;

		/* Save old params */
		_pOldData		= _pTable->data;
		_pOldMaxlen	= _pTable->maxlen;

		/* Store new params */
		_pTable->data		= dsp_tampon;
		_pTable->maxlen	= sizeof(dsp_tampon);

		/* Perform function */
		_res = proc_dostring(	_pTable,
							_write,
							_pFilp,
							_pBuffer,
							_pLenp,
							_pPpos);

		/* Restore old params */
		_pTable->data		= _pOldData;
		_pTable->maxlen	= _pOldMaxlen;
	}

	/* End */
	return(_res);
}

/** sysctl dsp_proc_init_port.
 *
 *   \par Description
 *	<b>This function allow to print init port on sysctl.</b>\n
 *
 *	@param _pTable:	Sysctl table reference.
 *	@param _write:		If write.
 *	@param _pFile:		File reference.
 *	@param _pBuffer:	Buffer reference.
 *	@param _pLen:		Length reference.
 *	@param _pPpos:		Position reference.
 *
 *	@return proc_dostring return.
 */
int dsp_proc_init_port_port(	ctl_table		*_pTable,
						int			_write,
						struct file	*_pFilp,
		     			void __user 	*_pBuffer,
						size_t		*_pLenp,
						loff_t		*_pPpos)
{
	int			_res;
	unsigned char	_initDone		= *((unsigned char *)(_pTable->data));

	if (_initDone == FALSE) {
		strcpy(	dsp_tampon,
				"CLOSE");
	}
	else {
		strcpy(	dsp_tampon,
				"OPEN");
	}
		
	/* Perform function */
	{
		void 	 	*_pOldData;
		size_t		_pOldMaxlen;

		/* Save old params */
		_pOldData		= _pTable->data;
		_pOldMaxlen	= _pTable->maxlen;

		/* Store new params */
		_pTable->data		= dsp_tampon;
		_pTable->maxlen	= sizeof(dsp_tampon);

		_res = proc_dostring(	_pTable,
							_write,
							_pFilp,
							_pBuffer,
							_pLenp,
							_pPpos);

		/* Restore old params */
		_pTable->data		= _pOldData;
		_pTable->maxlen	= _pOldMaxlen;
	}

	/* End */
	return(_res);
}

/** sysctl dsp_proc_data_mode.
 *
 *   \par Description
 *	<b>This function allow to print data mode on sysctl.</b>\n
 *
 *	@param _pTable:	Sysctl table reference.
 *	@param _write:		If write.
 *	@param _pFile:		File reference.
 *	@param _pBuffer:	Buffer reference.
 *	@param _pLen:		Length reference.
 *	@param _pPpos:		Position reference.
 *
 *	@return proc_dostring return.
 */
int dsp_proc_data_mode(	ctl_table		*_pTable,
					int			_write,
					struct file	*_pFilp,
		     		void __user 	*_pBuffer,
					size_t		*_pLenp,
					loff_t		*_pPpos)
{
	int			_res;
	unsigned char	_dataModeEnable		= *((unsigned char *)(_pTable->data));

	if (_dataModeEnable == TRUE) {
		strcpy(	dsp_tampon,
				"ENABLE");
	}
	else {
		strcpy(	dsp_tampon,
				"DISABLE");
	}
		
	/* Perform function */
	{
		void 	 	*_pOldData;
		size_t		_pOldMaxlen;

		/* Save old params */
		_pOldData		= _pTable->data;
		_pOldMaxlen	= _pTable->maxlen;

		/* Store new params */
		_pTable->data		= dsp_tampon;
		_pTable->maxlen	= sizeof(dsp_tampon);

		_res = proc_dostring(	_pTable,
							_write,
							_pFilp,
							_pBuffer,
							_pLenp,
							_pPpos);

		/* Restore old params */
		_pTable->data		= _pOldData;
		_pTable->maxlen	= _pOldMaxlen;
	}

	/* End */
	return(_res);
}

/** sysctl dsp_proc_slic_state_port.
 *
 *   \par Description
 *	<b>This function allow to print slic stateon sysctl.</b>\n
 *
 *	@param _pTable:	Sysctl table reference.
 *	@param _write:		If write.
 *	@param _pFile:		File reference.
 *	@param _pBuffer:	Buffer reference.
 *	@param _pLen:		Length reference.
 *	@param _pPpos:		Position reference.
 *
 *	@return proc_dostring return.
 */
int dsp_proc_slic_state_port(		ctl_table		*_pTable,
							int			_write,
							struct file	*_pFilp,
		     				void __user 	*_pBuffer,
							size_t		*_pLenp,
							loff_t		*_pPpos)
{
	int			_res;
	unsigned short	_slicOffTheHook		= *((unsigned short *)(_pTable->data));

	switch (_slicOffTheHook) {
		case SLIC_OFFHOOK:
			strcpy(	dsp_tampon,
					"SLIC_OFFHOOK");
			break;

		case SLIC_ONHOOK:
			strcpy(	dsp_tampon,
					"SLIC_ONHOOK");
			break;

		default:
			strcpy(	dsp_tampon,
					"SLIC_NOT_STEADY");
			break;
	}
		
	/* Perform function */
	{
		void 	 	*_pOldData;
		size_t		_pOldMaxlen;

		/* Save old params */
		_pOldData		= _pTable->data;
		_pOldMaxlen	= _pTable->maxlen;

		/* Store new params */
		_pTable->data		= dsp_tampon;
		_pTable->maxlen	= sizeof(dsp_tampon);

		_res = proc_dostring(	_pTable,
							_write,
							_pFilp,
							_pBuffer,
							_pLenp,
							_pPpos);

		/* Restore old params */
		_pTable->data		= _pOldData;
		_pTable->maxlen	= _pOldMaxlen;
	}

	/* End */
	return(_res);
}

/** sysctl dsp_proc_ring_state_port.
 *
 *   \par Description
 *	<b>This function allow to print ring stateon sysctl.</b>\n
 *
 *	@param _pTable:	Sysctl table reference.
 *	@param _write:		If write.
 *	@param _pFile:		File reference.
 *	@param _pBuffer:	Buffer reference.
 *	@param _pLen:		Length reference.
 *	@param _pPpos:		Position reference.
 *
 *	@return proc_dostring return.
 */
int dsp_proc_ring_state_port(		ctl_table		*_pTable,
							int			_write,
							struct file	*_pFilp,
		     				void __user 	*_pBuffer,
							size_t		*_pLenp,
							loff_t		*_pPpos)
{
	int			_res;
	unsigned char	_slicState		= *((unsigned char *)(_pTable->data));

	switch (_slicState) {
		case SLIC_RINGING_ON:
			strcpy(	dsp_tampon,
					"SLIC_RINGING_ON");
			break;

		case SLIC_RINGING_OFF:
			strcpy(	dsp_tampon,
					"SLIC_RINGING_OFF");
			break;

		case SLIC_NORING_OFFHOOK:
			strcpy(	dsp_tampon,
					"SLIC_NORING_OFFHOOK");
			break;

		case SLIC_NORING_ONHOOK:
			strcpy(	dsp_tampon,
					"SLIC_NORING_ONHOOK");
			break;

		default:
			strcpy(	dsp_tampon,
					"SLIC_RING_NOT_STEADY");
			break;
	}
		
	/* Perform function */
	{
		void 	 	*_pOldData;
		size_t		_pOldMaxlen;

		/* Save old params */
		_pOldData		= _pTable->data;
		_pOldMaxlen	= _pTable->maxlen;

		/* Store new params */
		_pTable->data		= dsp_tampon;
		_pTable->maxlen	= sizeof(dsp_tampon);

		_res = proc_dostring(	_pTable,
							_write,
							_pFilp,
							_pBuffer,
							_pLenp,
							_pPpos);

		/* Restore old params */
		_pTable->data		= _pOldData;
		_pTable->maxlen	= _pOldMaxlen;
	}

	/* End */
	return(_res);
}

/** sysctl dsp_proc_stats_endpoint.
 *
 *   \par Description
 *	<b>This function allow to print stats endpoint sysctl.</b>\n
 *
 *	@param _pTable:	Sysctl table reference.
 *	@param _write:		If write.
 *	@param _pFile:		File reference.
 *	@param _pBuffer:	Buffer reference.
 *	@param _pLen:		Length reference.
 *	@param _pPpos:		Position reference.
 *
 *	@return proc_dostring return.
 */
int dsp_proc_stats_endpoint(		ctl_table		*_pTable,
							int			_write,
							struct file	*_pFilp,
		     				void __user 	*_pBuffer,
							size_t		*_pLenp,
							loff_t		*_pPpos)
{
	int						_res;
	struct dsp_statsEndPoint_str	*_statsEndPoint	= (struct dsp_statsEndPoint_str *)(_pTable->data);
	char						*_pChain			= dsp_tampon;

	snprintf(	_pChain,
			sizeof(dsp_tampon),
			"EXTERNAL -> [ SndPkts = %6d SndBytes = %6d RcvPkts = %6d RcvBytes = %6d ]",
			_statsEndPoint->external.snd.pkts,
			_statsEndPoint->external.snd.bytes,
			_statsEndPoint->external.rcv.pkts,
			_statsEndPoint->external.rcv.bytes);
			
	/* Next */
	_pChain += strlen(_pChain);

	/* Perform function */
	{
		void 	 	*_pOldData;
		size_t		_pOldMaxlen;

		/* Save old params */
		_pOldData		= _pTable->data;
		_pOldMaxlen	= _pTable->maxlen;

		/* Store new params */
		_pTable->data		= dsp_tampon;
		_pTable->maxlen	= sizeof(dsp_tampon);

		_res = proc_dostring(	_pTable,
							_write,
							_pFilp,
							_pBuffer,
							_pLenp,
							_pPpos);

		/* Restore old params */
		_pTable->data		= _pOldData;
		_pTable->maxlen	= _pOldMaxlen;
	}

	/* End */
	return(_res);
}

/** sysctl dsp_proc_stats_network.
 *
 *   \par Description
 *	<b>This function allow to print stats network sysctl.</b>\n
 *
 *	@param _pTable:	Sysctl table reference.
 *	@param _write:		If write.
 *	@param _pFile:		File reference.
 *	@param _pBuffer:	Buffer reference.
 *	@param _pLen:		Length reference.
 *	@param _pPpos:		Position reference.
 *
 *	@return proc_dostring return.
 */
int dsp_proc_stats_network(		ctl_table		*_pTable,
							int			_write,
							struct file	*_pFilp,
		     				void __user 	*_pBuffer,
							size_t		*_pLenp,
							loff_t		*_pPpos)
{
	int						_res;
	struct dsp_statsNetwork_str	*_statsNetwork		= (struct dsp_statsNetwork_str *)(_pTable->data);
	char						*_pChain			= dsp_tampon;

	snprintf(	_pChain,
			sizeof(dsp_tampon),
			"EXTERNAL -> [ SndPkts = %6d SndBytes = %6d RcvPkts = %6d RcvBytes = %6d ]\n",
			_statsNetwork->external.snd.pkts,
			_statsNetwork->external.snd.bytes,
			_statsNetwork->external.rcv.pkts,
			_statsNetwork->external.rcv.bytes);
			
	/* Next */
	_pChain += strlen(_pChain);

	snprintf(	_pChain,
			sizeof(dsp_tampon),
			"DSP -> [ SndSpeechPkts = %6d SndSpeechBytes = %6d SndTonePkts = %6d SndToneBytes = %6d ]\n",
			_statsNetwork->dsp.sndSpeech.pkts,
			_statsNetwork->dsp.sndSpeech.bytes,
			_statsNetwork->dsp.sndTone.pkts,
			_statsNetwork->dsp.sndTone.bytes);
	/* Next */
	_pChain += strlen(_pChain);

	snprintf(	_pChain,
			sizeof(dsp_tampon),
			"DSP -> [ RcvSpeechPkts = %6d RcvSpeechBytes = %6d RcvVasPkts = %6d RcvVadBytes = %6d ]\n",
			_statsNetwork->dsp.rcvSpeech.pkts,
			_statsNetwork->dsp.rcvSpeech.bytes,
			_statsNetwork->dsp.rcvVad.pkts,
			_statsNetwork->dsp.rcvVad.bytes);
			
	/* Next */
	_pChain += strlen(_pChain);

	snprintf(	_pChain,
			sizeof(dsp_tampon),
			"RTP -> [ SndPkts = %6d SndBytes = %6d InitPort = %6d callerId = %6d ]\n",
			_statsNetwork->rtp.snd.pkts,
			_statsNetwork->rtp.snd.bytes,
			_statsNetwork->rtp.snd.initPortDone,
			_statsNetwork->rtp.snd.callerIdDone);

	/* Next */
	_pChain += strlen(_pChain);

	snprintf(	_pChain,
			sizeof(dsp_tampon),
			"RTP -> [ VoiceReady = %6d RcvPkts = %6d RcvBytes = %6d RcvDtmfs = %6d ]\n",
			_statsNetwork->rtp.snd.voiceReady,
			_statsNetwork->rtp.rcv.pkts,
			_statsNetwork->rtp.rcv.bytes,
			_statsNetwork->rtp.rcv.dtmfs);

	/* Next */
	_pChain += strlen(_pChain);

	snprintf(	_pChain,
			sizeof(dsp_tampon),
			"RTP -> [ ModeData_inactive = %3d ModeData_recvOnly = %3d ModeData_sendOnly = %3d ModeData_sendRecv = %6d ]\n",
			_statsNetwork->rtp.rcv.modeData.inactive,
			_statsNetwork->rtp.rcv.modeData.recv_only,
			_statsNetwork->rtp.rcv.modeData.send_only,
			_statsNetwork->rtp.rcv.modeData.send_recv);

	/* Next */
	_pChain += strlen(_pChain);

	snprintf(	_pChain,
			sizeof(dsp_tampon),
			"RTP -> [ ErrSndVoiceNull = %6d ]",
			_statsNetwork->rtp.error.sndVoiceNull);
			
	/* Next */
	_pChain += strlen(_pChain);

	/* Perform function */
	{
		void 	 	*_pOldData;
		size_t		_pOldMaxlen;

		/* Save old params */
		_pOldData		= _pTable->data;
		_pOldMaxlen	= _pTable->maxlen;

		/* Store new params */
		_pTable->data		= dsp_tampon;
		_pTable->maxlen	= sizeof(dsp_tampon);

		_res = proc_dostring(	_pTable,
							_write,
							_pFilp,
							_pBuffer,
							_pLenp,
							_pPpos);

		/* Restore old params */
		_pTable->data		= _pOldData;
		_pTable->maxlen	= _pOldMaxlen;
	}

	/* End */
	return(_res);
}
/** sysctl dsp_proc_device_evts.
 *
 *   \par Description
 *	<b>This function allow to print nb wait events on device.</b>\n
 *
 *	@param _pTable:	Sysctl table reference.
 *	@param _write:		If write.
 *	@param _pFile:		File reference.
 *	@param _pBuffer:	Buffer reference.
 *	@param _pLen:		Length reference.
 *	@param _pPpos:		Position reference.
 *
 *	@return proc_dostring return.
 */
int dsp_proc_device_evts(		ctl_table		*_pTable,
							int			_write,
							struct file	*_pFilp,
		     				void __user 	*_pBuffer,
							size_t		*_pLenp,
							loff_t		*_pPpos)
{
	int						_res;
	char						*_pChain			= dsp_tampon;

	snprintf(	_pChain,
			sizeof(dsp_tampon),
			"DEVICE -> [ Fxs = %6d Fxo = %6d]",
  			gaDsp2HostQ.writeIdx - gaDsp2HostQ.readIdx,
  			gaFxo2HostQ.writeIdx - gaFxo2HostQ.readIdx);
			
	/* Next */
	_pChain += strlen(_pChain);

	/* Perform function */
	{
		void 	 	*_pOldData;
		size_t		_pOldMaxlen;

		/* Save old params */
		_pOldData		= _pTable->data;
		_pOldMaxlen	= _pTable->maxlen;

		/* Store new params */
		_pTable->data		= dsp_tampon;
		_pTable->maxlen	= sizeof(dsp_tampon);

		_res = proc_dostring(	_pTable,
							_write,
							_pFilp,
							_pBuffer,
							_pLenp,
							_pPpos);

		/* Restore old params */
		_pTable->data		= _pOldData;
		_pTable->maxlen	= _pOldMaxlen;
	}

	/* End */
	return(_res);
}

/** sysctl dsp_proc_voice_ready_port.
 *
 *   \par Description
 *	<b>This function allow to print nb wait voice sample ready for Network.</b>\n
 *
 *	@param _pTable:	Sysctl table reference.
 *	@param _write:		If write.
 *	@param _pFile:		File reference.
 *	@param _pBuffer:	Buffer reference.
 *	@param _pLen:		Length reference.
 *	@param _pPpos:		Position reference.
 *
 *	@return proc_dostring return.
 */
int dsp_proc_voice_ready_port(		ctl_table		*_pTable,
								int			_write,
								struct file	*_pFilp,
		     					void __user 	*_pBuffer,
								size_t		*_pLenp,
								loff_t		*_pPpos)
{
	int						_res;
	VoiceCirQ_t				*_pVoiceTxQ		= (VoiceCirQ_t *)(_pTable->data);
	char						*_pChain			= dsp_tampon;

	snprintf(	_pChain,
			sizeof(dsp_tampon),
			"%6d",
    			_pVoiceTxQ->writeIdx - _pVoiceTxQ->readIdx);
			
	/* Next */
	_pChain += strlen(_pChain);

	/* Perform function */
	{
		void 	 	*_pOldData;
		size_t		_pOldMaxlen;

		/* Save old params */
		_pOldData		= _pTable->data;
		_pOldMaxlen	= _pTable->maxlen;

		/* Store new params */
		_pTable->data		= dsp_tampon;
		_pTable->maxlen	= sizeof(dsp_tampon);

		_res = proc_dostring(	_pTable,
							_write,
							_pFilp,
							_pBuffer,
							_pLenp,
							_pPpos);

		/* Restore old params */
		_pTable->data		= _pOldData;
		_pTable->maxlen	= _pOldMaxlen;
	}

	/* End */
	return(_res);
}

/** sysctl dsp_proc_ajb.
 *
 *   \par Description
 *	<b>This function allow to print AJB.</b>\n
 *
 *	@param _pTable:	Sysctl table reference.
 *	@param _write:		If write.
 *	@param _pFile:		File reference.
 *	@param _pBuffer:	Buffer reference.
 *	@param _pLen:		Length reference.
 *	@param _pPpos:		Position reference.
 *
 *	@return proc_dostring return.
 */
int dsp_proc_ajb(		ctl_table		*_pTable,
					int			_write,
					struct file	*_pFilp,
		     		void __user 	*_pBuffer,
					size_t		*_pLenp,
					loff_t		*_pPpos)
{
	int						_res;
	JitBuffType_  				*_pSMMJitBuff		= &(SMMJitBuff[*(int *)(_pTable->data)]);
	char						*_pChain			= dsp_tampon;

	snprintf(	_pChain,
			sizeof(dsp_tampon),
			"[ WtErrFlag = %6X RdErrFlags = %6X threshold = %6u GapOffset = %6u ]\n",
    			_pSMMJitBuff->WtErrFlag,
    			_pSMMJitBuff->RdErrFlag,
			AJB_READ_THRESHOLD[*(int *)(_pTable->data)],
    			_pSMMJitBuff->GapOffset);
			
	/* Next */
	_pChain += strlen(_pChain);

	snprintf(	_pChain,
			sizeof(dsp_tampon),
			"[ FreeRtpCnt = %6u FreeCellCnt = %6u ListRtpCnt = %6u ListCellCnt = %6u ]\n",
    			_pSMMJitBuff->FreeRtpCnt,
    			_pSMMJitBuff->FreeCellCnt,
    			_pSMMJitBuff->ListRtpCnt,
    			_pSMMJitBuff->ListCellCnt);
			
	/* Next */
	_pChain += strlen(_pChain);

	snprintf(	_pChain,
			sizeof(dsp_tampon),
			"[ WtOFDropCnt = %6u WtDupDropCnt = %6u WtNoCellMemDropCnt = %6u WtNoRtpMemDropCnt = %6u ]\n",
    			_pSMMJitBuff->WtOFDropCnt,
    			_pSMMJitBuff->WtDupDropCnt,
    			_pSMMJitBuff->WtNoCellMemDropCnt,
    			_pSMMJitBuff->WtNoRtpMemDropCnt);
			
	/* Next */
	_pChain += strlen(_pChain);

	snprintf(	_pChain,
			sizeof(dsp_tampon),
			"[ CodecType = %6u FrameLen = %6u ExpReadSeqNum = %6u LastReadSeqNum = %6u ]\n",
    			_pSMMJitBuff->CodecType,
    			_pSMMJitBuff->FrameLen,
    			_pSMMJitBuff->ExpReadSeqNum,
    			_pSMMJitBuff->LastReadSeqNum);
			
	/* Next */
	_pChain += strlen(_pChain);

	snprintf(	_pChain,
			sizeof(dsp_tampon),
			"[ OFRtpMarkCnt = %6u OFCellMarkCnt = %6u LateDropCnt = %6u TotalNULLPlayed = %6lu ]\n",
    			_pSMMJitBuff->OFRtpMarkCnt,
    			_pSMMJitBuff->OFCellMarkCnt,
    			_pSMMJitBuff->LateDropCnt,
    			_pSMMJitBuff->TotalNULLPlayed);
			
	/* Next */
	_pChain += strlen(_pChain);

	snprintf(	_pChain,
			sizeof(dsp_tampon),
			"[ ListCellMax = %6u ListCellMin = %6u DelayEstMax = %6u DelayEstMin = %6u ]\n",
    			_pSMMJitBuff->ListCellMax,
    			_pSMMJitBuff->ListCellMin,
    			_pSMMJitBuff->DelayEstMax,
    			_pSMMJitBuff->DelayEstMin);
			
	/* Next */
	_pChain += strlen(_pChain);

	snprintf(	_pChain,
			sizeof(dsp_tampon),
			"[ PlayOutDelay = %6d DelayTrackPktDropCnt = %6d PutDupDropCnt = %6d ]",
    			_pSMMJitBuff->PlayOutDelay,
    			_pSMMJitBuff->DelayTrackPktDropCnt,
    			_pSMMJitBuff->PutDupDropCnt);
			
	/* Next */
	_pChain += strlen(_pChain);

	/* Perform function */
	{
		void 	 	*_pOldData;
		size_t		_pOldMaxlen;

		/* Save old params */
		_pOldData		= _pTable->data;
		_pOldMaxlen	= _pTable->maxlen;

		/* Store new params */
		_pTable->data		= dsp_tampon;
		_pTable->maxlen	= sizeof(dsp_tampon);

		_res = proc_dostring(	_pTable,
							_write,
							_pFilp,
							_pBuffer,
							_pLenp,
							_pPpos);

		/* Restore old params */
		_pTable->data		= _pOldData;
		_pTable->maxlen	= _pOldMaxlen;
	}

	/* End */
	return(_res);
}

#endif

// vim:ts=5:
