/* ============================================================================
 *
 * Copyright (C) 2001 Analog Devices Inc. All rights reserved
 *
 * The information and source code contained herein is the exclusive property
 * of Analog Devices and may not be disclosed, examined or reproduced in whole
 * or in part without the explicit written authorization from Analog Devices.
 *
 * ============================================================================
 */


#include<oswrapper.h>
#if TONE_GEN
#include "../dsplib/ToneGenCons.h"
#include "../dsplib/ToneProto.h"
#endif

#define CPE_TIMER1_BASE_ADDRESS 0xb9070000


volatile UINT32 *dummy_read_addr = (UINT32 *)0xa0000000;
UINT32 dummy_value;

/* Definitions from dspipc */
extern UINT16 voicePacketFlag[];
extern DspStateInfo_t gaDspStateInfo[];
extern DspipcAddr dspipcAddress[MAX_DSP_NUM];
UINT16 InterfaceFlag;
extern Word16 SMM_ToneGenEna[];
extern CommandBuffer gaComandQ[];
extern DspStateInfo_t gaDspStateInfo[MAX_VOICE_CHANNELS];
#if  DTMFRX
extern Word16 HDtmfrx_reset_on[MAX_VOICE_CHANNELS];
extern uWord16  SMMDtmfrxDSP2HostBuff[MAX_VOICE_CHANNELS][DTMFRX_DSP2HOST_BUFF_LEN];
#endif
extern VoiceCirQ_t gaDsp2HostQ;
volatile UINT16 *hp_addr_latch[MAX_DSP_NUM];
volatile UINT32 *pm_data_latch[MAX_DSP_NUM];
volatile UINT16 *dm_data_latch[MAX_DSP_NUM];
unsigned short dspTimeSlot0,dspTimeSlot1;

/***************************************************************
 *  Function Name : dspAssignIDMAReg.                          *
 *  Description   : Function to initialize voice module.       *
 *  Input         : DSP Data to download.                      *
 *  Output        : None.                                      *
 *  Return Value  : None.                                      *
 **************************************************************/
void dspAssignIDMAReg(void)
{
  /* Assign IDMA Addresses */
  hp_addr_latch[0] = (UINT16*)hp_addr_latch_value1;
  hp_addr_latch[1] = (UINT16*)hp_addr_latch_value2;
                                                                                                
  pm_data_latch[0] = (UINT32*)pm_data_latch_value1;
  pm_data_latch[1] = (UINT32*)pm_data_latch_value2;
                                                                                                
  dm_data_latch[0] = (UINT16*)dm_data_latch_value1;
  dm_data_latch[1] = (UINT16*)dm_data_latch_value2;
}

/***************************************************************
 *  Function Name : voiceInit.                                 *
 *  Description   : Function to initialize voice module.       *
 *  Input         : None.                                      *
 *  Output        : None.                                      *
 *  Return Value  : None.                                      *
 **************************************************************/
void dspipcSystemInit(void)
{
  UINT32 temp; 
  UINT16 temp1;
  /* Assign SPORT0 to DSPs */
  temp = *(volatile UINT32*)DSP_SPORT0;
  /* Enabling DSP2 SPORT0 to receive clocks and fs for DSP1 */
  temp |= 0x00000003;
  *(volatile UINT32*)DSP_SPORT0 = temp;

#if SLIC_MODULES
  slicDeviceInitialize();
#endif
  /* Initialize dspipc */
  dspipcInitialize();

  /* Reset ALL DSP Ports */
  dspipcResetAllPorts();
  mdelay(100);
  /* Assign the Interrupts,
     even for 2 DSP, we register only one interrupts , we might need to look
     it again */
  osLockInterrupts();
  osAssignInterrupt(0);
  if(MAX_CHANNELS_ISR == 1)
    osAssignInterrupt(1);
  osUnlockInterrupts();
  mdelay(1000);
  for(temp1 = 0; temp1 < MAX_VOICE_CHANNELS; temp1++)
    confInitGainControl(temp1);
#if DTMFRX
  for(temp1 = 0; temp1 < MAX_VOICE_CHANNELS; temp1++)
    DtmfrxResetDet(temp1);
#endif
}
/**********************************************************************************
 *  Function Name  : dspipcInitDSPBankAddresses.                                  *
 *  Description    : Function to get address for DSP banks.                       *
 *  Input          : DSP Number.                                                  *
 *  Output         : None.                                                        *
 *  Return Value   : None.                                                        *
 *********************************************************************************/
void dspipcInitDSPBankAddresses(INT16 dspnumber)
{
  UINT16 dspIntAddr;
  dspIntAddr = DSP_INTERFACE_ADDR;
  /* get the bank addresses from dsp */
  dspipcAddress[dspnumber].dspBankSelectCntr = dspIntAddr;
  dummy_value = *dummy_read_addr;
  dspIntAddr++;
  dspipcAddress[dspnumber].hostBankSelectCntr = dspIntAddr;
  dummy_value = *dummy_read_addr;
  dspIntAddr++;
  dspipcAddress[dspnumber].bankSelectErrorCntr = dspIntAddr;
  dummy_value = *dummy_read_addr;
  dspIntAddr++;
  dspipcAddress[dspnumber].cmdHost2dspBank0 = dspReadDataMem(dspnumber, dspIntAddr);
  dummy_value = *dummy_read_addr;
  dspIntAddr++;
  dspipcAddress[dspnumber].cmdHost2dspBank1 = dspReadDataMem(dspnumber, dspIntAddr);
  dummy_value = *dummy_read_addr;
  dspIntAddr++;
  dspipcAddress[dspnumber].cmdDsp2hostBank0 = dspReadDataMem(dspnumber, dspIntAddr);
  dummy_value = *dummy_read_addr;
  dspIntAddr++;
  dspipcAddress[dspnumber].cmdDsp2hostBank1 = dspReadDataMem(dspnumber, dspIntAddr);
  dummy_value = *dummy_read_addr;
  dspIntAddr++;

  dspipcAddress[dspnumber].dataHost2dspBank0 = dspReadDataMem(dspnumber, dspIntAddr);
  dummy_value = *dummy_read_addr;
  dspIntAddr++;
  dspipcAddress[dspnumber].dataHost2dspBank1 = dspReadDataMem(dspnumber, dspIntAddr);
  dummy_value = *dummy_read_addr;
  dspIntAddr++;
  dspipcAddress[dspnumber].dataDsp2hostBank0 = dspReadDataMem(dspnumber, dspIntAddr);
  dummy_value = *dummy_read_addr;
  dspIntAddr++;
  dspipcAddress[dspnumber].dataDsp2hostBank1 = dspReadDataMem(dspnumber, dspIntAddr);
  dummy_value = *dummy_read_addr;
  dspIntAddr++;


  InterfaceFlag = dspIntAddr;
#if DTMFRX
  dspipcAddress[dspnumber].DtmfrxDsp2hostBank0 =
         (dspipcAddress[dspnumber].cmdDsp2hostBank0) + (MAX_COMMAND_WORDS_PER_CHANNEL * MAX_CHANNELS_ISR);
  dspipcAddress[dspnumber].DtmfrxDsp2hostBank1 =
         (dspipcAddress[dspnumber].cmdDsp2hostBank1) +  (MAX_COMMAND_WORDS_PER_CHANNEL * MAX_CHANNELS_ISR);
#endif
}

/**********************************************************************************
 *  Function Name  : dspipcInitialize.                                            *
 *  Description    : Function to Initialize dspipc.                               *
 *  Input          : None.                                                        *
 *  Output         : None.                                                        *
 *  Return Value   : None.                                                        *
 *********************************************************************************/
void dspipcInitialize(void)
{
  INT16 iIndex;
  memset(gaDspStateInfo,0,sizeof(gaDspStateInfo));
#if PROFILE
  voxTimer2Init();
#endif
  /* Initialize DSP Bank Addresses */
  for(iIndex =0; iIndex < MAX_DSP_NUM; iIndex++)
  {
    dspipcInitDSPBankAddresses(iIndex);
#if DELAY_OPT
    dspipcGetDSPBankAddress(iIndex);
#endif
  }
  for (iIndex=0; iIndex< MAX_VOICE_CHANNELS; iIndex++)
  {
    dspipcClearQs(iIndex);
    ToneResetVars(iIndex);
    dspipcSetDSPDataMode(iIndex,DSP_DATA_INACTIVE);
    dspipcResetStateInfo(iIndex, 0);
#if DTMFRX
    DtmfrxResetDet(iIndex);
#endif
    gaComandQ[iIndex].readPtr = 0;
    gaComandQ[iIndex].writePtr = 0;
#if T38
    dspipcResetT38Q(iIndex);
#endif

  }
  gaDsp2HostQ.writeIdx = 0;
  gaDsp2HostQ.readIdx = 0;
#if FXO_MODULE
  dspipcResetFXOQ();
#endif

}
/**********************************************************************************
 *  Function Name  : dspDownloadMem                                               *
 *  Description    : Function to download DSP Image .                             *
 *  Input          : control_word1,control_word2,dsp_code_start,count.            *
 *  Output         : None.                                                        *
 *  Return Value   : None.                                                        *
 *********************************************************************************/
void dspDownloadMem(UINT16 dspNum,
                     UINT16 control_word1, 
                     UINT16 control_word2, 
                     UINT16 *dsp_code_start, 
                     UINT16 count)
{
  UINT32 templongint;
  UINT16 i,tempval;

  *hp_addr_latch[dspNum] = control_word1;
  dummy_value = *dummy_read_addr;
  dummy_value = *dummy_read_addr;
  if (control_word1 >= 0x4000)
    control_word2 = control_word2 << 4;
  tempval = 0x8000 | control_word2;
  *hp_addr_latch[dspNum] = tempval;
  dummy_value = *dummy_read_addr;
  dummy_value = *dummy_read_addr;
  if(control_word1 >= 0x4000)
  {                                                                               
    for(i=0; i<count; i++ )
    {
      *dm_data_latch[dspNum] = *dsp_code_start++;
      dummy_value = *dummy_read_addr;
    }
  }                                                                              
  else
  {
    for(i=0; i<count; i=i+2 )
    {
      templongint = (*dsp_code_start++) << 16;
      templongint= (templongint | (*dsp_code_start++)) ;
      *pm_data_latch[dspNum] = templongint;
    }
  }
}
/**********************************************************************************
 *  Function Name  : dspWriteIntoDataMem.                                         *
 *  Description    : Function to write into DSP Data  Memory.                     *
 *  Input          : DSP Number,Address, Value.                                   *
 *  Output         : None.                                                        *
 *  Return Value   : None.                                                        *
 *********************************************************************************/
void dspWriteIntoDataMem(UINT16 dspNum,
                                  UINT16 addr,
                                  UINT16 value)
{
   *hp_addr_latch[dspNum] = addr + 0x4000;
   *dm_data_latch[dspNum] = value;
}
/**********************************************************************************
 *  Function Name  : dspWriteIntoProgramMem.                                      *
 *  Description    : Function to write into DSP program  Memory.                  *
 *  Input          : DSP Number,Address, Value.                                   *
 *  Output         : None.                                                        *
 *  Return Value   : None.                                                        *
 *********************************************************************************/
void dspWriteIntoProgramMem(UINT16 dspNum,
                                  UINT16 addr,
                                  UINT16 value)
{
   *hp_addr_latch[dspNum] = addr;
   *pm_data_latch[dspNum] = value;
}

/**********************************************************************************
 *  Function Name  : dspReadDataMem.                                              *
 *  Description    : Function to read from DSP Data  Memory.                      *
 *  Input          : DSP Number, Address.                                         *
 *  Output         : None.                                                        *
 *  Return Value   : 16 Bit data .                                                *
 *********************************************************************************/
UINT16 dspReadDataMem(UINT16 dspNum, UINT16 addr)
{
   *hp_addr_latch[dspNum] = addr + 0x4000;
   return *dm_data_latch[dspNum];
}

/**********************************************************************************
 *  Function Name  : dspReadProgramMem.                                              *
 *  Description    : Function to read from DSP program Memory.                      *
 *  Input          : DSP Number, Address.                                         *
 *  Output         : None.                                                        *
 *  Return Value   : 32 Bit data .                                                *
 *********************************************************************************/
UINT32 dspReadProgramMem(UINT16 dspNum, UINT16 addr)
{
   *hp_addr_latch[dspNum] = addr ;
   return *pm_data_latch[dspNum];
}

                                                                                                
/*==================================================================================
  Function to Initialize SPORT.                           
 ==================================================================================*/
void dspInitSportCodec(void)
{
  UINT16 temp_value, temp_addr;
  UINT16 dspNum;

  for (dspNum = 0; dspNum < MAX_DSP_NUM; dspNum++)
  {
    temp_addr = 0x3fb4;
    if(dspNum == 0)
      temp_value = DSP0_TIME_SLOT;
    else
      temp_value = DSP1_TIME_SLOT;
    dspWriteIntoDataMem(dspNum, temp_addr, temp_value);
    DEBUG(PRINT("\r\n DSP TIME SLOT Assigned:%d",temp_value),DEBUG_INFO);
    temp_addr = 0x3fb5;
    temp_value = 255;  // 32.768 or 38.912(MU or A law)
    dspWriteIntoDataMem(dspNum, temp_addr, temp_value);
    //--------initialise DSP sport Clock
    temp_addr = 0x3fb6;

    if(*(volatile UINT32*)0xb900003c == 0x6836)
      temp_value = ((DSP_VX160_CLOCK/4.096)-1);
    else if(*(volatile UINT32*)0xb900003c == 0x6833)
      temp_value = ((DSP_VX150_CLOCK/4.096)-1);
    else if(*(volatile UINT32*)0xb900003c == 0x6834)
      temp_value = ((DSP_VX200_CLOCK/4.096)-1);
    else
      printk("UNKNOWN CHIP: ???????????????????????????????????? \n");
    DEBUG(PRINT("\r\n DSP CLock Assigned:%d",temp_value),DEBUG_INFO);
    dspWriteIntoDataMem(dspNum, temp_addr, temp_value);
    //--------initialise DSP sport Codec
    temp_addr = 0x3fb7;
    temp_value = confGetPCMMode(0);
    DEBUG(PRINT("\r\n DSP Codec Type  Assigned:%d",temp_value),DEBUG_INFO);
    dspWriteIntoDataMem(dspNum, temp_addr, temp_value);
  }

}

void dspStartSPORT(void)
{
  int dspNum;
  UINT16 temp_value, temp_addr;
  dspInitSportCodec();
  for (dspNum = 0; dspNum < MAX_DSP_NUM; dspNum++)
  {
    temp_addr = 0x0000;
    temp_value = dspNum;
    dspWriteIntoDataMem(dspNum,temp_addr, temp_value);
    mdelay(10);
    temp_addr = 0x0001;
    temp_value = 0x1234;
    dspWriteIntoDataMem(dspNum,temp_addr, temp_value);
  }
  osUnlockInterrupts();
  DEBUG(PRINT("\r\nADSP218x DOWNLOAD DONE !!!!\n"),DEBUG_INFO);
  temp_value = dspReadDataMem(0,0x3fb8);
  DEBUG(PRINT("\r\nDSP Ver No:%x.%x  \n",(temp_value>>8 & 0xff),
                      (temp_value&0xff)), DEBUG_INFO);
}
