/********************************************************************
Ver       : 0.06
File Name : CIDTx.c
Purpose   : Declaration of data structures for Caller-ID Generation.
Project   : To generate CID CP-FSK Signals.
Complaint : Bellcore 30/31
********************************************************************/
/*******************************************************************
                      RUN CONDITIONS 
********************************************************************/
#ifndef __CIX_TX_CON_H
#define __CIX_TX_CON_H

//---> To selcet Simulation Environment
//#define CID_SIMULATION

#define DECIMATION


#ifdef CID_SIMULATION
  //---> To make active all CiTx code.
  #define  CALLER_IDTx
#endif

//---->To Enable CIDTx string pattern
#define CIDTX_STRING_INTERN

/*****************************************************************
          CALLER ID TRANMITTER MESSAGE FORMAT
******************************************************************/

//#define CIDTX_MSG_TYPE   SPCS_MSG_TYPE_SDMF;
#define CIDTX_MSG_TYPE   SPCS_MSG_TYPE_MDMF;
//#define CIDTX_MSG_TYPE   SPCS_MSG_TYPE_GDMF;

#define CIDTX_TYPE1		1
#define CIDTX_TYPE2		2
/*******************************************************************
         CONTROLLING PARAMETERS/USER SELECTION
//--> Select Source side One Source  Input ---Different Examples
********************************************************************/
//#define SDMF_1
//#define SDMF_2
//#define SDMF_3
//#define  MDMF_1
#define  MDMF_2
//#define  MDMF_3

/*******************************************************************
              DATA TRANSMISSION PROTOCOL MANAGING STATE DEFENETIONS.
********************************************************************/
#define  CID_TX_STATE0		 0
#define  CID_TX_STATE1           1
#define  CID_TX_STATE2           2
#define  CID_TX_STATE3           3
#define  CID_TX_STATE4           4

/*
First Ring to Data Start Generation Event Delay Counter interm of10ms
 */

#define  FIRST_RING_CIDTX_PLAY     90 //260   //--> 150 ms.

#define  END_OF_CID_DISPLAY        35 //200  //constant to give  2nd ring

/******************************************************************
             Caller -ID BELL -30/  Message Layer Type fields
*******************************************************************/

#define  SPCS_MSG_TYPE_SDMF           0x04
#define  SPCS_MSG_TYPE_MDMF           0x80
#define  SPCS_MSG_TYPE_GDMF           0xFF //--> Need to be captured
#define  SPCS_MSG_TYPE_WAITING_IND           0x82 //Message Waiting Indicator

/******************************************************************
     Caller -ID BELL -30/ Presentation Layer Date field Length
*******************************************************************/

#define  DATE_FIELD_LENGTH            0x08

/******************************************************************
 Caller -ID BELL -30/   MDMF Presentation Layer  field Types
if Number and and Name are available ,following Type  for Num and 
	Name will be used			 
******************************************************************/

#define  MDMF_DATE_TIME_TYPE          0x01
#define  MDMF_T_CALL_NUMBER_TYPE      0x02
#define  MDMF_T_CALL_NAME_TYPE        0x07

#define MDMF_VISUAL_IND_NAME_TYPE	0x0B
#define MDMF_NO_OF_MESSAGES_TYPE	0x13
#define MDMF_LASTMSGCLI_TYPE		0x0E
#define MDMF_MSG_IDENTIFICATION_TYPE 0x0D

/*****************************************************************
Caller -ID BELL -30/   MDMF Presentation Layer  field Types
if Number and and Name are not available ,following Type  for Num 
	and Name are used to indicate the reasons for 
		non-availability of Information
******************************************************************/

#define  MDMF_R_CALL_NUMBER_TYPE      0x04
#define  MDMF_R_CALL_NAME_TYPE        0x08


/*****************************************************************
     DATA LAYER Channel Seizure/ Mark signal generation side 
	 Counters   interms of 10 ms Intervals.
******************************************************************/

#define LEN_CHAN_SEIZURE_25x12BITS  25
#define LEN_MARK_SIG_15x12BITS      15
#define CIDCW_MARK_SIG_7x12BITS     7



//#ifdef CID_SIMULATION

/*****************************************************************
                  PHYSICAL LAYER Signal Properties	      
******************************************************************/

/*****************************************************************
   Sampling Frequence Selection
   Fs : 8000 kHz
   Up   Sampling it 3      = 24000 Hz.
   Down Sampling it yields = 8000  Hz.
   Phasor  rate : 24K.
******************************************************************/

#define FS_LINE       8000 
#define FS_USED      24000 

/*****************************************************************
           V.21 Frequency Selection
******************************************************************/
#define ETSI_STD //--->enable this switch to use ETSI standard

#ifdef ETSI_STD

#define FREQ0  2100
#define FREQ1 1300

#else

#define FREQ0  2200
#define FREQ1 1200 

#endif

/*****************************************************************
		 Coefficents Calculation
******************************************************************/

#define PHA_ADV1 (65535 * FREQ0 /FS_USED)

#define PHA_ADV2 (65535 * FREQ1 /FS_USED)


/*****************************************************************
           V.21 BIT TRANSMISSION RATE
		   1200 Baud.
******************************************************************/

#define  BIT_TRANSMISSION_RATE        1200 

/*****************************************************************
W.R.T BIT TRANSMISSION RATE , NO OF SAMPLES PER EACH BIT
******************************************************************/

#define  NO_OF_SAMPLES_PER_BIT      ( FS_USED / BIT_TRANSMISSION_RATE )


/*****************************************************************
                 DOWN_SAMPLE_FACT
W.R.T BIT TRANSMISSION RATE && SUPPOSED FS : 8K && USED FS :24 K
******************************************************************/

#define  DOWN_SAMPLING_MOD_CONST      ( FS_USED / FS_LINE )

//---> Resolving of PayLoad Buffer Length Constants

#ifdef  DECIMATION 
	#define CID_TX_PAY_LOAD_LEN        80
#else
	#define CID_TX_PAY_LOAD_LEN       240
#endif




#define CIDTX_PACKET 4

#define CIDTX_FAILURE -1

#define CIDTX_STRING_LEN 50  //Max. Parameter for both NAme and Number
#define CIDTX_STRING_BUFF_LEN 150


//----------------->CIDCW Timings
#define  CIDCW_TCW_T1 65     //to play CPE alert tone 300ms CW + pause 350ms
#define  CIDCW_TCAS_TOUT 45 //30  //Timeout for ACK TCAS(80ms)+ TOUT(160ms)+ Tb(60ms)
#define  CIDCW_TFSK 15       //to send FSK after pause 60<T<200




/*****************************************************************
                     FEW SIMULATION CONSTANTS
********************************************************************/
#ifdef CID_SIMULATION

#define  MAX_VOICE_CHANNELS          1

#endif

//CidStrPtr Buffer Length

#ifdef CIDTX_STRING_INTERN
#define MAX_CIDTX_STRING_LEN  50
#endif

#endif
