/********************************************************************
File Name		:	JitVar.h
Description     :   Jitter Buffer  Variable Declarations 	
Authors         : sivan.nagireddi@analog.com &rajakumar.kulakarni@analog.com.
Version Number	:	2.0
Date			:	10/June/2002
Company Name	:  	Analog Devices India Pvt Ltd.
********************************************************************/
#ifndef __JitVar_H
#define __JitVar_H

//#define SIMULATION


#include "JitCons.h"

/*
Jitter Buffer Data Structure declaration
*/

//---> To Interface Write Operation.
typedef struct WtRtpPkt_ {               

uWord16  CodecType;
uWord16  SeqNum;
uWord16  FrameLen;
uWord16  WtDelay;

#if 1 //def JIT_DELAY_ESTIMATION
uWord32 SR_TIMESTAMP;
uWord32 RR_TIMESTAMP;
#endif
}WtRtpPktType_;



//-----> Per Channel Jitter Buffer Variables.
typedef struct StJitterBuff_{


/*-----------------------------------------------------------------------
                     LIST TOP
-----------------------------------------------------------------------*/
uWord16 ListTop;
uWord16 OFListTop;



/*-----------------------------------------------------------------------
               //--> AJB Scheduler TableIndex
-----------------------------------------------------------------------*/
uWord16  SchTabelIndex;




/*-----------------------------------------------------------------------
               //--> RTP TO DSP-IPC QUE 
-----------------------------------------------------------------------*/
uWord16  QueWtIndex;
uWord16  QueRdIndex;


/*-----------------------------------------------------------------------
              //---> OVerFlow Buffer Counters
-----------------------------------------------------------------------*/
uWord16 OFCellMarkCnt;
uWord16 OFRtpMarkCnt;


/*-----------------------------------------------------------------------
 //--> RunCodecType  //-->NoOfBasicFramesCnt  //--> BasicFrameInMS.
               CODEC parametrization Initialization Flag.
-----------------------------------------------------------------------*/
uWord16  CodecType;
uWord16 RxFrameCntr;
uWord16 RxFrameMS;
uWord16 RxFrameFlag;
uWord16 FrameLen;

uWord16 DelIntStepMS;//Delay introduce Step in ms.
uWord16 DelRemStepMS;//Delay Remove Step in ms.

uWord16 RdSkipCntr;//Read Skip run time Counter.
uWord16 RdRepCntr; //Read repeat run time Counter.
uWord16 RdSkipThr;//Read Skip Threshold.
uWord16 RdRepThr; //Read repeat Threshold.


/*-----------------------------------------------------------------------
                  VAD CHARACTERSTIC PARARMETRS 
-----------------------------------------------------------------------*/
uWord16  PrevSidEnable;
uWord16  SidRead;

uWord16  SidInterval;
uWord32  SilenceON;
uWord32  SilenceOFF;
/*-----------------------------------------------------------------------
                   VAD CHARACTERSTIC PARARMETRS 
-----------------------------------------------------------------------*/




/*-----------------------------------------------------------------------
                 Sequnce Number Tracking Parameters.
-----------------------------------------------------------------------*/
#ifdef AJB_RTP_DUPLICATE_CHECK_ENABLED
uWord16 HistWtIndex;
uWord16 HistBuffLen;
#endif

//-->  Current reading (or) Last Read packet SeqNum.
uWord16  LastReadSeqNum;
uWord16  ExpReadSeqNum;
/*-----------------------------------------------------------------------
                 Sequnce Number Tracking Parameters.
-----------------------------------------------------------------------*/




///////////////////////////////////////////////////////////////////////
uWord16 JitInitFlag;//-->   JitterBuffer TRIPLE POLE SWITCH 
///////////////////////////////////////////////////////////////////////




/*-----------------------------------------------------------------------
             DSP-IPC Interfce Paramaters for 10ms.
-----------------------------------------------------------------------*/
uWord16  RxPktProp;
//---> Current Pkt FrameLen.
uWord16  RxFrameLen;
//---> Current Reading Packet PayLoad.
UCHAR    RxBuff[AJB_MAX_PAYLOAD_CELL_LEN];

/*-----------------------------------------------------------------------
          End of DSP-IPC Interfce Paramaters for 10ms.
-----------------------------------------------------------------------*/
uWord16  CodecInstProp;
/*-----------------------------------------------------------------------
                 DelayEstimation Module Parameters.
-----------------------------------------------------------------------*/
Word16  EstCumJit;
Word32  EstCumJit32;
Word32  EstInitJit;

uWord32 SR_TIMESTAMP;
uWord32 RR_TIMESTAMP;
Word16  d_hat;
Word16  v_hat;
Word16  p_hat;
Word16  d_hat_prev;
Word16  v_hat_prev;
Word16  nd_prev;
Word16  nd_2;
Word16  var_i;
uWord16 ReadThreshold;
/*-----------------------------------------------------------------------
                 End of DelayEstimation Module Parameters.
-----------------------------------------------------------------------*/



/*-----------------------------------------------------------------------
                 //---> WtJit Vars &Statistics
-----------------------------------------------------------------------*/
uWord16 WtPktCnt;
#if AJB_ENABLE_STATISTICS

uWord16 PreWtPktCnt;
uWord16 PreWtDupDropCnt;
uWord16 WtDupDropCnt;
uWord16 PutDupDropCnt;
uWord16 NoCells2Rtp;

uWord16 PreWtCodecErr;
uWord16 WtCodecErr;

uWord16 PreWtFrameLenErr;
uWord16 WtFrameLenErr;

uWord16 PreWtNoRtpMemDropCnt;
uWord16 WtNoRtpMemDropCnt;

uWord16 PreWtNoCellMemDropCnt;
uWord16 WtNoCellMemDropCnt;

uWord16 PreWtOFDropCnt;
uWord16 WtOFDropCnt;

#endif
uWord16 FreeRtpCnt;
uWord16 FreeCellCnt;
/*-----------------------------------------------------------------------
                 //---> WtJit Statistics.
-----------------------------------------------------------------------*/

/*-----------------------------------------------------------------------
                  //--->  Jit DelayTracking Statistics.
-----------------------------------------------------------------------*/
uWord16 MaxDelay;
uWord16 CurWtDelay;

uWord16 JitDelayEstTimer;
uWord16 DelayDropTimer;



uWord16 DelayTrackFreqCntr;
uWord16 LateDropFreqCntr;
uWord16 NullPlayFreqCntr;

uWord16 DelayTrackPktDropCnt;
#if  AJB_ENABLE_STATISTICS
uWord16 DelayEstMin;
uWord16 DelayEstMax;
uWord16 StatisticsResetTimer;
//--> DelayRdQue Pointers
uWord16  AJBRdCntr;
uWord16  PlayOutDelay;
uWord16 WtErrFlag;
uWord16 RdErrFlag;
#endif

uWord16  GapOffset;
/*-----------------------------------------------------------------------
                  //--->  Jit DelayTracking Statistics.
-----------------------------------------------------------------------*/


/*-----------------------------------------------------------------------
                  //--->  Jit Put Module Statistics.
-----------------------------------------------------------------------*/
uWord16 ListCellCnt;
uWord16 RTPThrCnt;
uWord16 ListRtpCnt;

uWord16 ListCellMax;
uWord16 ListCellMin;

uWord32 TotalNULLPlayed;
/*-----------------------------------------------------------------------
                  //--->  Jit Put Module Statistics.
-----------------------------------------------------------------------*/

/*-----------------------------------------------------------------------
//--->  JitRd Statistics.
-----------------------------------------------------------------------*/
uWord16  RdPktIndex;

uWord32 NullPktCnt;  //--> ContinousNullCnt;
uWord16 ListNullCnt; //--> Null Tot Count.
uWord16  LateDropCnt;
uWord16  PreLateDropCnt;
uWord16 LateDropRunCnt;
/*-----------------------------------------------------------------------
//--->  JitRd Statistics.
-----------------------------------------------------------------------*/


/*-----------------------------------------------------------------------
//--->  Error Statics &Flags
-----------------------------------------------------------------------*/
Word16  ResetRdSeqNumber;
Word16  Reset;
Word16  ResetDelayEstTimer;

uWord16 Rd_state;


uWord32 RxDelayTDropPktCnt;
uWord32 RxLateDropPktCnt;
uWord32 RxNullPlayPktCnt;
uWord32 RxNwLostPktCnt;

Word32  CodecIeOffset;
uWord16 CodecIePolyIndex;
/*-----------------------------------------------------------------------
//--->  Error Statics  &Flags
-----------------------------------------------------------------------*/

} JitBuffType_;


//---> It consists of Each PayLoad Profile To re-order and Manage
typedef struct StructRtpPkt_ {	                

uWord32  SRTimeStamp;
//--> Current Packet Sequence Number					
uWord16  SeqNum;

//--->   WtDelay
uWord16  WtDelay;

//---> Current Packet Property.
uWord16  PktProp;
					
//---> No of packets indirected.
uWord16  PktCnt;

//--> Current Packet Index.
uWord16  PktIndex;

//---> To locate Next Packet in a List.
uWord16  NextIndex;

}RtpPktType_;



typedef struct StRTPPayLoadt_ {	                


//---> Current Packet Property.
uWord16  PktProp;

//--> Current Packet Index.
uWord16  PktIndex;

uWord16  FrameLen;

//---> To locate Next Packet in a List.
uWord16  NextIndex;

UCHAR VoicePkt[AJB_MIN_PAYLOAD_CELL_LEN];

}RTPPayLoadPktType_;

#ifdef AJB_SIMULATION

typedef struct VoiceSt{
                 UCHAR aucPayload[600];

}VoiceQ_t;

typedef enum 
{
PCMU = 0,CODEC_1016,PCMA=8,G722=9,G726_32=2,G729=18,
CODEC_EXT=15,G723AH = 96 , G723AL,G723H,G723L,G729A,G729B,G723=4,
G726_16 = 108, G726_24, G726_40
}Codec_e;

#endif
//---> End of Declaration.


/* 
 * RTP payload type definitions 
 */

#define RTP_PAYLOADTYPE_PCMU         0
#define RTP_PAYLOADTYPE_1016         1
#define RTP_PAYLOADTYPE_G72632       2
#define RTP_PAYLOADTYPE_GSM          3
#define RTP_PAYLOADTYPE_G723         4
#define RTP_PAYLOADTYPE_DVI48000     5
#define RTP_PAYLOADTYPE_DVI416000    6
#define RTP_PAYLOADTYPE_LPC          7
#define RTP_PAYLOADTYPE_PCMA         8
#define RTP_PAYLOADTYPE_G722         9
#define RTP_PAYLOADTYPE_L161CH       10
#define RTP_PAYLOADTYPE_L162CH       11
#define RTP_PAYLOADTYPE_QCELP        12
#define RTP_PAYLOADTYPE_RESERVED1    13
#define RTP_PAYLOADTYPE_MPA          14
#define RTP_PAYLOADTYPE_G728         15
#define RTP_PAYLOADTYPE_DVI411025    16
#define RTP_PAYLOADTYPE_DVI422050    17
#define RTP_PAYLOADTYPE_G729         18
#define RTP_PAYLOADTYPE_RESERVED2    19
#define RTP_PAYLOADTYPE_CELB         25
#define RTP_PAYLOADTYPE_JPEG         26
#define RTP_PAYLOADTYPE_NV           28
#define RTP_PAYLOADTYPE_H261         31
#define RTP_PAYLOADTYPE_MPV          32
#define RTP_PAYLOADTYPE_MP2T         33
#define RTP_PAYLOADTYPE_H263         34


/**********************************************************************
					AJB CLI PARAMETERS
**********************************************************************/
#if AJB_USE_CLI_CONFIGURATIONS
typedef struct AJBCliPara{

uWord16   CLI_AJB_MIN_DELAY_MS;
uWord16   CLI_AJB_MAX_DELAY_MS;
uWord16   CLI_AJB_DEL_OFFSET_MS;

uWord16   CLI_AJB_DELAY_EST_TIMER;
uWord16   CLI_AJB_DELAY_DROP_TIMER;
uWord16   CLI_AJB_ALLOW_PKT_DROP_PER_SLOT;

uWord32   CLI_AJB_RESET_ON_MAX_NULL_PLAY_OUT_TIME;
uWord16   CLI_AJB_RESET_ON_MAX_LATE_PKT_DROP_TIME;
uWord16   CLI_AJB_AllowNullPlayPassFailCntThr;

uWord16   CLI_AJB_ALLOW_NULLS_PER_SLOT[AJB_MAX_NO_OF_DELAY_LEVELS];

uWord16   CLI_AJB_Mode;
uWord16   CLI_AJB_STASTICS_RESET_TIMER;

uWord16   CLI_AJB_DELAY_EST_SIGMA_FACTOR;
uWord16   CLI_AJB_GAP_DELAY_EST_SIGMA_FACTOR;
uWord16   CLI_AJB_DELAY_TRACK_SIGMA_FACTOR;

} AJBCliParaSt_;
#endif

/********************************************************************
				   END AJB CLI PARAMETERS
********************************************************************/

/*---------------------------------------------------------------------
                MARKOV-4 STATE MODEL STRUCT
----------------------------------------------------------------------*/
typedef struct Markov4StateStr {

uWord16  PLCEnaDisFlag;
/*
             2   DURING BURUST- STATE PACKETS GOT RECEIVED -> MARKOVSTATE2
             ^
             |
             3  BURUST-STATE---> PACKETS GETTING LOST --> MARKOVSTATE3
     ----------------------------------------------
	         1  GAP-STATE - NO LOST IN PACKET -MARKOVSTATE1
			 |
			 V
             4  IN GAP-STATE - LOST IN PACKET - MARKOVSTATE4

*/
uWord16 MARKOV4STATE;

uWord32 RxPktCnt;
uWord32 RxLostPktCnt;

uWord32 c11;
uWord32 c13;
uWord32 c14;

uWord32 c22;
uWord32 c23;
uWord32 c33;
uWord32 BurstFreqCntr_c5;

//----> VQMonPktMetrics
uWord16 FrameSizeMS;
UCHAR   BurustLossDensity;
uWord16 BurustLength;

UCHAR   GapLossDensity;
uWord16 GapLength;

UCHAR   AvePktLossRateProb;
UCHAR   AvePktLossRateAbs;
UCHAR   AveNwPktLossRate;
UCHAR   AveDisPktLossRate;

UCHAR   Est_Ie;
Word16  Est_Id;

UCHAR   Rating_Factor;
UCHAR   Rating_FactorExt;
UCHAR   MOS_LQ;
UCHAR   MOS_CQ;

uWord16 JBNominalDelay;
uWord16 JBCurrentDelay; //--> End System delay
uWord16 JBMaxDelay;
 // 01   23      4567
 // Plc  JbType  DelalStepSize.
UCHAR   RxConfig;

//---------------------------------------------------------------------
uWord16 RoundTripDelay; //if not avail fill with zeros.

UCHAR   SigLevel; //-128 to +126 +127:not Avail(-15dBm0 to -20dBm0)
UCHAR   NoiseLevel; //-128 to +126 +127:not Avail(-15dBm0 to -20dBm0)
UCHAR   RERL; //40 to 50dB   +127:not Avail:ERL + ERLE
//---------------------------------------------------------------------
} Markov4StateType_;


typedef enum {
  RATING_FACTOR_MAX = 94, RATING_FACTOR_MIN = 50, MARKOV4_ESTI_SIG_LOST = 8,
  GMin = 16
} Markov4Const_;
/*---------------------------------------------------------------------
			End of  MARKOV-4 STATE MODEL STRUCT
----------------------------------------------------------------------*/


/*---------------------------------------------------------------------
			        Build VQ-MON-rtcpxr Packet
----------------------------------------------------------------------*/
typedef struct VQMonRtcpXR_Str {
	uWord32 VQMonFrame[VQ_MON_RTCP_XR_BLOCK_LEN];
    UCHAR   VQMonRtcpXRBlock[VQ_MON_RTCP_XR_BLOCK_LEN * 4];
}VQMonRtcpXR_Type_;

/*---------------------------------------------------------------------
			    End of  Build VQ-Mon-rtcpxrPacket.
----------------------------------------------------------------------*/
#endif


