/* ============================================================================
 *
 * Copyright (C) 2001 Analog Devices Inc. All rights reserved
 *
 * The information and source code contained herein is the exclusive property
 * of Analog Devices and may not be disclosed, examined or reproduced in whole
 * or in part without the explicit written authorization from Analog Devices.
 *
 * ============================================================================
 */
                                                                                               
#ifndef __VOICE_COMMON_H
#define __VOICE_COMMON_H

#define PROFILE  0
#define A_LAW 0
#define MU_LAW 1
#define LINEAR 2

#define PRINT printk
extern unsigned char gDebugLevel;
#define DEBUG_EXCEPTION 0
#define DEBUG_SLIC      1
#define DEBUGISR        2
#define DEBUG_COMMANDS  3
#define DEBUG_CONF      4
#define DEBUG_EVENTS    5
#define DEBUG_INFO      6
#define DEBUG_ALL       0xff
#define DEBUG_KRTP      7


#if DEBUG_ENABLED
#define DEBUG(s,type)     if((1 << type) & gDebugLevel) s 
#define DEBUG_ISR(s,type) if((1 << type) & gDebugLevel) s 
#else 
#define DEBUG(s,type)   
#define DEBUG_ISR(s, type)
#endif

                                                                                                     

 // Bit operations
  // These operations are used to operaton on the specified bit (or bitfield)
  // within a register.
  //
  // They rely on the following naming conventions:
  //  XX_BIT            The bit position of the single bit XX
  //  YY_FST            The bit position of the LSB of bitfield YY
  //  YY_LEN            The number of bit in the bitfield YY
  //
  // Don't panic too much at the number of operations - the optimizer in the
  // compiler should be able to resolve almost all of them to constants.
                                                                                
#define SETBIT(var, bit, val)        if (val) \
                                         var |= (1 << bit ## _BIT ); \
                                     else \
                                         var &= ~(1 << bit ## _BIT)
                                                                                
#define GETBIT(var, bit)        (((var) >> bit ## _BIT) & 0x1)
                                                                                
#define ZEROMASK(fld)           ((1 << (fld ## _LEN)) - 1)
                                                                                
#define FLDMASK(fld)            (ZEROMASK(fld) << (fld ## _FST))
                                                                                
#define SETFIELD(var, fld, val) (var = (var & ~FLDMASK(fld)) | \
                                (((val) & ZEROMASK(fld)) << fld ## _FST))
                                                                                
#define GETFIELD(val, fld)      (((val) >> (fld ## _FST)) & ZEROMASK(fld))

#define T38RELAY 1

typedef enum
{
  USA=0, FRANCE, NETHERLANDS, POLAND, SPAIN, BELGIUM
}CountryEnum;



#endif
