#ifndef _modem_api_c_h
#define _modem_api_c_h


	typedef char				CHAR8;
	typedef unsigned char		UCHAR8;
//	typedef char				CHAR;
//	typedef unsigned char		UCHAR;

//	typedef short int			INT16;
//	typedef unsigned short int	UINT16;
//	typedef short int			INT;
//	typedef unsigned short int	UINT;
	typedef signed short int	SINT16;


//	typedef int					INT32;
//	typedef unsigned int		UINT32;


	typedef long				LONG;
//	typedef unsigned long		ULONG;

	typedef long				LONG32;
	typedef unsigned long		ULONG32;

/*------------------------------------------------------------------------
 PREPROCESSOR OPTIONS
-------------------------------------------------------------------------*/

//#define T30_T38	//operation in T.30 + T.38 comboned mode
//#define IMPAIRMENT

//#define TCP_MODE          //to select TCP mode of packet transfer
#define UDP_MODE            //to select TCP mode of packet transfer
//#define CISCO_GW



/*------------------------------------------------------------------------
 PREPROCESSOR CONSTANTS
-------------------------------------------------------------------------*/

/* Modes of data transfers*/
#define DATA_L2MSS  0        //Lsb 2 Msb with start/stop addition/deletion
#define DATA_M2L    1        //Msb to Lsb without start/stop bits
#define DATA_HDLC   2        //HDLC mode
#define DATA_L2M    3        //Lsb 2 Msb without start/stop bits


#define PCMLENGTH	80  //length of one PCM Frame

#define FL_5MS  1       //Frame Length - 5ms
#define FL_10MS 2       //Frame Length - 10ms
#define FL_20MS 3       //Frame Length - 20ms



//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

#define DEF_DEBUG	//To enable session logging to the file "Stata.LOG"

//#define GRAPH_DEB	//Enable graphic debugger

//#define DEF_CPP	//To enable CPP part of the host code

//#define DEB_PRINTF	//To enable printf statements

//#define T30LOOPBACK	//2 channel loopback of T.30

//#define T30INTG		//Integrate T.30 modules

//#define TESTVECT	//Test fax data pump using vectors


#define MAXIFPPKTSIZE   1000	//600
#define ORG_MODE		1
#define ANS_MODE		2


#define MAXMSGLEN   64     //maximum number of message parameters

#define MAXFRAMELEN 160     //frame length, in number of samples 
                            //(160 corresponds to 20ms )

#define FRMLENGTH 80

//Message Queue size for sending messages from host to DP
#define T38_IN_MSG_QSIZE      128

//Message Queue size for messages received from the DP
#define T38_OUT_MSG_QSIZE     128

#define TXPKTQLEN 10*400    //US and DS packet queue buffer sizes
#define RXPKTQLEN 10*400

#define TX_IFP_MAXSIZE  200

/* Fax mode, data pump definitions */

#define RATE_V21300      0x000C
#define RATE_V272400     0x000B
#define RATE_V274800     0x000A
#define RATE_V297200     0x0009
#define RATE_V299600     0x0008
#define RATE_V177200L    0x0007
#define RATE_V177200S    0x0006
#define RATE_V179600L    0x0005
#define RATE_V179600S    0x0004
#define RATE_V1712000L   0x0003
#define RATE_V1712000S   0x0002
#define RATE_V1714400L   0x0001
#define RATE_V1714400S   0x0000


	typedef struct {
		INT16 FrameLength;			// Length specified 
		INT16 *INptr;				// PCM samples for receiver 
		INT16 *OUTptr;				// Buffer for filling modulated samples 
	} PCMDATASTRUCT;


	typedef struct {
		INT16 EndOfSession;		//will be set when fax session ends
		INT16 DSPktSize;		//size of down stream packet (set by application)
		INT16 DSValidPkt;		//When '1' indicates presence of valid DS packet(set by application)
		INT16 USPktSize;		//size of up stream packet (set by fax server)
		INT16 USValidPkt;		//When '1' indicates presence of valid US packet(set by fax server)
//Buffer to queue DS packets (filled by application)
		UCHAR8 DSPktBuf[MAXIFPPKTSIZE];
//Buffer to queue US packets (filled by fax server)
		UCHAR8 USPktBuf[MAXIFPPKTSIZE];
	}tIFP_PARAMS;


	typedef struct  {
		INT16 MsgId;
		INT16 MsgLen;
		UCHAR8 MsgParams[64];
	} MESSAGE;


	typedef struct {
		UCHAR8 *INptr;			// Store messages for data pump 
		UCHAR8 *OUTptr;			// Store messages from data pump
		UCHAR8 *InPtrWrite;		// Current write pointer to ip queue
		UCHAR8 *InPtrRead;		// Current read pointer to ip queue 
		UCHAR8 *OutPtrWrite;	// Current write pointer to op queue
		UCHAR8 *OutPtrRead;		// Current read pointer to op queue 
		INT16 InQueLen;			// Length of the input queue 
		INT16 OutQueLen;		// Length of the output queue
	} MSGQUESTRUCT;


    typedef struct {
        INT16 iLen;                 //Length of the individual message
        INT16 iId;                  //Message ID
        UCHAR8 aiParams[MAXMSGLEN]; //Optional message parameters
    } tMESSAGE;
    
    typedef struct {
         UCHAR8 *paiBufPtr;     //Pointer to message queue buffer
         UCHAR8 *paiWrite;      //Write pointer to message queue
         UCHAR8 *paiRead;       //Read pointer to message queue
         UINT16 wSize;          //Message queue buffer size
    } tMSGQSTRUCT;


	typedef    struct
	{
		INT16 iCallMode;      //ORIGINATE = 1 or ANSWER = 2
		UINT16 wFrameLength;
		UINT16 wSizeOfJitterBuf;
		UINT16 LSRedundancy;
		UINT16 HSRedundancy;
	//Pointer to structure that maintains the Queue having messages to T38
		tMSGQSTRUCT *ptT38InMsgQStr;
	//Pointer to structure that maintains the Queue having responsess from T38
		tMSGQSTRUCT *ptT38OutMsgQStr;
	//Pointer to down stream packet queue (Jitter buffer)
		UCHAR8 *JitterBufPtr;

	} tFAXRELAY_ESL_config;


#endif
