#include <oswrapper.h>
#include <voice_bitmap.h>
#include <linux/time.h>
#include "rtpgdef.h"
#include "rtpmsg.h"

#if KRTCP_ENABLED
/*
 *  Minimum time between RTCP packets from this site (in seconds).
 * This time prevents the reports from `clumping' when sessions
 * are small and the law of large numbers isn't helping to smooth
 * out the traffic.  It also keeps the report interval from
 * becoming ridiculously small during transient outages like a
 * network partition.
 */
#define RTCP_MIN_TIME  5

/*
* Fraction of the RTCP bandwidth to be shared among active
* senders.  (This fraction was chosen so that in a typical
* session with one or two active senders, the computed report
* time would be roughly equal to the minimum report time so that
* we don't unnecessarily slow down receiver reports.) The
* receiver fraction must be 1 - the sender fraction.
*/
#define RTCP_SENDER_BW_DIVISER  4


extern SrcInfoTable_t  gaSrcInfoTable[];
extern UCHAR           aucSITBitMask[]; 
CHAR   gaRTCPCompPkt[RTCP_COMP_PKT_SIZE];  /* out going RTCP Compound Pkt */
extern UCHAR           gaRTPBitMap[];
extern RTPInfo_t       gaRTPInfo[];
/*
 * Minimum time between RTCP packets from this site (in seconds).
 * This time prevents the reports from `clumping' when sessions
 * are small and the law of large numbers isn't helping to smooth
 * out the traffic.  It also keeps the report interval from
 * becoming ridiculously small during transient outages like a
 * network partition.
 */
#define RTCP_MIN_TIME  5

/*
* Fraction of the RTCP bandwidth to be shared among active
* senders.  (This fraction was chosen so that in a typical
* session with one or two active senders, the computed report
* time would be roughly equal to the minimum report time so that
* we don't unnecessarily slow down receiver reports.) The
* receiver fraction must be 1 - the sender fraction.
*/
#define RTCP_SENDER_BW_DIVISER  4

/********************************************************************************
 * Function    : rtcpSendData(RTPInfo_t* pRTPSessionInfo, UINT32 *pulPacketSize)*
 * Input       : pRTPSessionInfo, pulPacketSize.                                *
 * Output      : pulPacketSize.                                                 *
 * Result      : 0 : Success and  -1 : Failure.                                 *
 * Description : This function prepares the RTCP compound packet in             *
 * gaRTCPCompPkt[] and sends it over the RTCP transport channel.                *
********************************************************************************/
INT32 rtcpSendData(RTPInfo_t *pRTPSessionInfo, UINT32 *pulPacketSize)
{
  INT16 iSendBytes;
  /* Prepare the RTCP Compound packet, in gaRTCPCompPkt[] */
  if(rtcpBuildCompPkt(pRTPSessionInfo, pulPacketSize) != SUCCESS) 
  {
    printk("failed in rtcpBuildCompPkt\n");
    return(FAILURE);
  }
  iSendBytes = rtpSendData(pRTPSessionInfo->rtcp_sock,
                           gaRTCPCompPkt,
                           *pulPacketSize,
                           pRTPSessionInfo->ulDestIPAddr,
                           pRTPSessionInfo->uiDestRTCPPort);
  if(iSendBytes < 0)
  {
    return FAILURE;
  }
  return SUCCESS;
}
/*******************************************************************************
 * Prototype   : rtcpHandleIntervalTimer.                                      *
 * Input       : pRTPInfo.                                                     *
 * Output      : None.                                                         *
 * Result      : None.                                                         *
 * Description : This function will be called upon each expiry of the RTCP     *
 *   Transmission Interval Timer. It then inturn calls the RTCPSendData        *
 *   function to send out the RTCP Compound Packet.                            *
 ******************************************************************************/
VOID rtcpHandleIntervalTimer(unsigned long data)
{
    RTPInfo_t  *pRTPSessionInfo = (RTPInfo_t*)data;
    SessInfo_t SessInfo;
    SessInfo.rtp_cmd = SEND_RTCP_DATA;
    SessInfo.iSessId = (pRTPSessionInfo-gaRTPInfo);
    pRTPSessionInfo->lRTCPTimerId = -1;
}

VOID sendRTCPData(VOID *pRTPInfo)
{
    UINT32     ulRTCPInterval = 0;
    RTPInfo_t  *pRTPSessionInfo;
    INT32      lMembers = 0;
    INT32      lSenders = 0;
    UINT32     ulRTCPBW = 0;
    UINT32     ulPacketSize = 0;
    BOOL       bWeSent = FALSE;
    INT16      iCount;

    if(pRTPInfo == NULL)
      return;
    pRTPSessionInfo = (RTPInfo_t*) pRTPInfo;

    pRTPSessionInfo->lRTCPTimerId = -1;
    /* Check whether session is already released */
    {
      UINT32 ulCount = 0;
      for(ulCount = 0; ulCount < MAX_RTP_SESSIONS; ulCount++)
      {
        if(CheckElement(gaRTPBitMap,MAX_RTP_SESSIONS,ulCount) == ELE_ALLOCATED)
        {
           RTPInfo_t *pTmp = &gaRTPInfo[ulCount];
           if(pTmp == pRTPSessionInfo)
            break;
        }
      }
      if(ulCount >= MAX_RTP_SESSIONS)
      {
        return;
      }
    }

    /* Prepare and Send out the RTCP Packets, puts the compound pkt 
     * size in ulPacketSize
     */
    rtcpSendData(pRTPSessionInfo, &ulPacketSize);
    /* If BYE packet is just sent then clear the data structures for that session */
    if(pRTPSessionInfo->bEndSession == TRUE) {
      pRTPSessionInfo->lRTCPTimerId = -1;
      pRTPSessionInfo->bEndSession = FALSE;
      printk("releaseSess..releaseSessreleaseSessreleaseSess.. PENDING\r\n");
      pRTPSessionInfo->bTimer = 0;
    } else {  /* if BYE is not sent */
       /* Total number of members in this session (including this member) */
       for(iCount = 0; iCount < MAX_SOURCES; iCount++) {
         if((CheckElement(aucSITBitMask, MAX_SOURCES, iCount) == ELE_ALLOCATED) 
             && gaSrcInfoTable[iCount].bValid 
                 && (gaSrcInfoTable[iCount].iSessionIndex == pRTPSessionInfo->iSessionIndex)) {
                lMembers++;
              
         }
       }

       lMembers++;  /* including this member */

       /* Senders during last RTCP interval */
       for(iCount = 0; iCount < MAX_SOURCES; iCount++) {
         if((CheckElement(aucSITBitMask, MAX_SOURCES, iCount) == ELE_ALLOCATED) 
            && gaSrcInfoTable[iCount].bValid && (gaSrcInfoTable[iCount].bPktRecvdInRTCPInterval)
                && (gaSrcInfoTable[iCount].iSessionIndex == pRTPSessionInfo->iSessionIndex)) {
                lSenders++;
                gaSrcInfoTable[iCount].bPktRecvdInRTCPInterval = FALSE;  /* Reset Flag */
         }
       }
       /* include this member, if it sent any pkts during the interval */
       if(pRTPSessionInfo->bPktSentInRTCPInterval) {
          lSenders++;   
       }    

       /* RTCP BW = 5% of Session BW */
       if(pRTPSessionInfo->iSessionBW == 0)
       {
         pRTPSessionInfo->iSessionBW = 8000;
       }
       ulRTCPBW = 5000;
    
       /* RTCP compound pkt size just sent, including network encapsulation 
        * ie. UDP, IP etc header sizes
        */
       ulPacketSize +=  (UDP_HDR_SIZE + IP_HDR_SIZE + FRAME_HDR_SIZE);
  
      
       /* Flag to indicate if any RTP pkts were sent during the last two intervals */
       if(pRTPSessionInfo->bPktSentInRTCPInterval 
                  || pRTPSessionInfo->bPktSentInLast2Intervals) {
           bWeSent = TRUE;
       } else {
           bWeSent = FALSE;
       }
       /* Calculate RTCP Timer interval */
       ulRTCPInterval = rtcpInterval(lMembers, lSenders, ulRTCPBW, 
                bWeSent, ulPacketSize, &pRTPSessionInfo->ulAvgRTCPSize, FALSE);
       /* Update Flags */
       pRTPSessionInfo->bPktSentInLast2Intervals = pRTPSessionInfo->bPktSentInRTCPInterval;
       pRTPSessionInfo->bPktSentInRTCPInterval = FALSE;
        
       /* Start the RTCP Timer */
       timerStart(pRTPInfo,
                  ulRTCPInterval,
                  rtcpHandleIntervalTimer, 
                  (unsigned long)pRTPInfo);
       pRTPSessionInfo->bTimer = 1;

    } /* end of else clause of "if(pRTPSessionInfo->bEndSession == TRUE)" */
}

/**********************************************************************************
* Prototype   : rtcpBuildCompPkt(RTPInfo_t *pRTPSessionInfo, UINT32 *pulPacketSize)
* Input       : pRTPSessionInfo, pulPacketSize
* Output      : pulPacketSize
* Result      : 0 : Success and  -1 : Failure
* Description : This function prepares the RTCP Compound RTCP packet. 
* pRTPSessionInfo is used to access the needed information from the global data structure 
* SrcInfoTable and RTPInfo.
*********************************************************************************/

INT32 rtcpBuildCompPkt(RTPInfo_t *pRTPSessionInfo, UINT32 *pulPacketSize)
{
    INT16      iCount = 0;
    UINT16     uiShortInt = 0;
    CHAR       *pcRTCPPkt;
    CHAR       *pcByeLen;
    INT16      iSSRCCount = 0;
    RTCPSDES_t *pRTCPSDES; 
    RTCPBYE_t  *pRTCPBye;
    CHAR       cPadCount=0;



    pcRTCPPkt = (CHAR *) gaRTCPCompPkt;
    memset(pcRTCPPkt, 0, RTCP_COMP_PKT_SIZE);
#ifdef NOT_NEEDED
    /* If RTP_MULTICAST or RTP_MULTI_UNICAST, then RR blocks for all the 
     * participants are included as the part of SR and RR (if any) 
     */
    if(pRTPSessionInfo->cMulticast == RTP_MULTICAST 
                    || pRTPSessionInfo->cMulticast == RTP_MULTI_UNICAST) {
#endif

    if((pRTPSessionInfo->cMulticast == RTP_UNICAST) 
                || (pRTPSessionInfo->cMulticast == RTP_MULTICAST)) { 
       if(pRTPSessionInfo->bPktSentInRTCPInterval) { 
	  /* RTP Packets were sent during this RTCP Interval, 
           * so, SR will be sent.
           */
          if(rtcpBuildSR(pRTPSessionInfo, &pcRTCPPkt) != SUCCESS) {
            DEBUG(PRINT("Faied in rtcpBuildSR\n"), DEBUG_EXCEPTION);
          }
          else
          {
#ifdef SNMP_RTP
           pRTPSessionInfo->ulLastSRTime = rtpGetCurrTime();
#endif
         }

       } else { 
	  /* RTP Packets were NOT sent during this RTCP Interval, 
           * so, RR will be sent.
           */
	 if(rtcpBuildRR(pRTPSessionInfo, 0, &pcRTCPPkt) != SUCCESS) {
            DEBUG(PRINT("Faied in rtcpBuildRR\n"), DEBUG_EXCEPTION);
         }
         else
         {
#if SNMP_RTP
           pRTPSessionInfo->ulLastRRTime = rtpGetCurrTime();
#endif
         }

       }  

       /* Prepare the SDES Packet and put into the Compound packet.
        * Every time all available SDES need not be sent, it is done selectively 
        * For the time being we shall send only CNAME every time, which is a must. 
        */
       pRTCPSDES = (RTCPSDES_t*)pcRTCPPkt;

       /* SDES Header */
       pRTCPSDES->Header.uiVersion |= RTP_VERSION;
       pRTCPSDES->Header.uiRRCount |= 0x01;        /* SDES Chunk count */
       pRTCPSDES->Header.uiPacketType |= RTCP_SDES;
 
       pRTCPSDES->ulSRC = pRTPSessionInfo->ulSSRC;
       pRTCPSDES->Header.uiPacketLength = sizeof(pRTCPSDES->ulSRC);
       pcRTCPPkt += (sizeof(pRTCPSDES->ulSRC) + sizeof(pRTCPSDES->Header));

       *pcRTCPPkt = (UCHAR)NULL;
       pcRTCPPkt += 1;
       pRTCPSDES->Header.uiPacketLength += 1;
 
       /* Adjust the location of pcRTCPPkt to point to the location just after the 
        * first SDES item (ie. CNAME), because for the time being we are sending 
        * only CNAME, which is mandatory.
	*/
       cPadCount = (((pRTCPSDES->Header.uiPacketLength % 4) == 0 ) ? 0 
                      : (4 - (pRTCPSDES->Header.uiPacketLength % 4)));
       if(cPadCount > 0) {
           /* Padding Octets, inclusive of the last padding-count octet */
          /* cPadCount-1 is because pcRTCPPkt already points to the next 
           * free location after SDES */
           pcRTCPPkt += cPadCount;  /* Padding Octets */
           pRTCPSDES->Header.uiPacketLength += cPadCount;
       }  
       pRTCPSDES->Header.uiPacketLength /= 4;

       /* BYE Packet */ 
       if(pRTPSessionInfo->bEndSession == TRUE) { 
	  /* Prepare BYE packet and put it into the Compound Packet. */
          pRTCPBye = (RTCPBYE_t*) pcRTCPPkt;

#ifdef NOT_NEEDED 
          /* added, to prevent runtime segmentation fault error */
          *pcRTCPPkt = (CHAR) (((RTP_VERSION << 6) | 0x20) & 0xe0);
          *(pcRTCPPkt+1) = (CHAR) RTCP_BYE;
#endif 

          /* BYE Header */
          pRTCPBye->Header.uiVersion    = 2;  //|= RTP_VERSION;
          pRTCPBye->Header.uiPadding    &= 0x0;       /* No Padding */
          pRTCPBye->Header.uiRRCount    &= 0x1;       /* SSRC Count */
          pRTCPBye->Header.uiPacketType |= RTCP_BYE;

          /* Here put the 'iSSRCCount' number of SSRCs into the BYE Packet.
           * All SSRCs corresponding to this session are included in the Packet
           */
          for(iCount=0; iCount < MAX_SSRC_LIST; iCount++) {
              if((CheckElement(aucSITBitMask, MAX_SOURCES, iCount) == ELE_ALLOCATED) 
                 && gaSrcInfoTable[iCount].bValid && (pRTPSessionInfo->iSessionIndex
                                        == gaSrcInfoTable[iCount].iSessionIndex)) {
                     if(iSSRCCount < MAX_SSRC_LIST)
                     {
                       pRTCPBye->aulSRC[iSSRCCount] =
                                        gaSrcInfoTable[iCount].ulSSRC;
                       iSSRCCount++;
                     }
                     else
                     {
                       DEBUG(PRINT("\r\nError: MAX_SSRC_LIST crossed\r\n"), 
                                    DEBUG_EXCEPTION);
                     }
                 
              }
          } 

          /* To calculate Packet Length, total size of the included SSRCs is
	   * condsidered. The Header length is excluded from the Packet Length 
           * calculation
           */
          /*Store the length field location of byepacket*/
          pcByeLen=pcRTCPPkt+2;
          uiShortInt = sizeof(RTCPBYE_t) - sizeof(RTCPCommonPktHeader_t)
                        - (sizeof(UINT32) * (MAX_SSRC_LIST - iSSRCCount));
          /* Adjust the pcRTCPPkt to point to the location just after the last SSRC
	   * included in the BYE packet
           */
          cPadCount = (((uiShortInt % 4) == 0 ) ? 0 : (4 - (uiShortInt % 4)));
          pcRTCPPkt += (sizeof(RTCPBYE_t) - (sizeof(UINT32) 
                                  * (MAX_SSRC_LIST - iSSRCCount)));
          if(cPadCount > 0) {
              /* Padding Octets, inclusive of the last padding-count octet */
              /* cPadCount-1 is because pcRTCPPkt already points to the next 
               * free location after SDES */
              pcRTCPPkt += cPadCount;  /* Padding Octets */
              uiShortInt += cPadCount;
          }  
          /*Alligning the length to 32-bit boundary*/
          uiShortInt /= 4;
          *(pcByeLen) = (UCHAR)(uiShortInt >> 8);
          *(pcByeLen+1) = (UCHAR)(uiShortInt & 0xff);
       } /* end of BYE packet */
    } 
     
#ifdef NOT_NEEDED   // This may be used for padding if we use Encryption
    /* Padding, to adjust the total length to multiple of 4 octets */
    *pulPacketSize = abs(pcRTCPPkt - gaRTCPCompPkt);
    cPadCount = (((*pulPacketSize % 4) == 0 ) ? 0 : (4 - (*pulPacketSize % 4)));

    if(cPadCount > 0) {
        /* Padding Octets, inclusive of the last padding-count octet */
        pcRTCPPkt += cPadCount;  /* Padding Octets */
        *pcRTCPPkt = cPadCount;  /* Padding count */  
    
        /* Total Compound Pkt size, including Padding Octets */
        *pulPacketSize = abs(pcRTCPPkt - gaRTCPCompPkt);
       /***length of each rtcp packet must be 32-bit aligned*/
       /**************
       if(pRTPSessionInfo->bEndSession == TRUE)
            *pcByeLen += 1;
       else
            pRTCPSDES->Header.uiPacketLength += 1;
       ******************/
       if(pRTPSessionInfo->bEndSession == TRUE)
            *pcByeLen += 1;
           
    } 
#endif

    /* Total Compound Pkt size, including Padding Octets */
    *pulPacketSize = abs(pcRTCPPkt - gaRTCPCompPkt);
#if 0
{
  INT32 x = pcRTCPPkt - gaRTCPCompPkt;
    *pulPacketSize = abs(x);
}
#endif
   return(SUCCESS);

} /* end of RTCPruildCompPkt() */
 
/*****************************************************************************
Function  : INT32 rtcpRRBlocks(INT16 iSessionIndex, RTCPRR_t aRRBlocks[], 
                     INT16 *piRRCount, INT16 iStartIndex, INT16 *pNextIndex)


*****************************************************************************/
INT32 rtcpRRBlocks(INT16 iSessionIndex, RTCPRR_t aRRBlocks[], INT16 *piRRCount, 
               INT16 iStartIndex, INT16 *pNextIndex)
{
    UINT32   ulExtendedMax;
    UINT32   ulTotalExpected;
    UINT32   ulExpectedInInterval;
    UINT32   ulReceivedInInterval;
    INT16    iCount;
    INT16    iCountValid;
    UINT32    ulCurrentTimeSec;
    UINT32    ulCurrentTimeUSec;
   
    

    for(iCount = iStartIndex, iCountValid = 0; (iCount < MAX_SOURCES) 
                             && (iCountValid < MAX_RR_BLOCKS); iCount++) {

      if((CheckElement(aucSITBitMask, MAX_SOURCES, iCount) == ELE_ALLOCATED)
                && gaSrcInfoTable[iCount].bValid 
                && (gaSrcInfoTable[iCount].iSessionIndex == iSessionIndex) 
                && gaSrcInfoTable[iCount].bPktRecvdInRTCPInterval) {
         aRRBlocks[iCountValid].ulSSRC = gaSrcInfoTable[iCount].ulSSRC; // KUMAR
     
         /* Calculate the Cumulative packet loss and  Fraction packet Loss */
         ulExtendedMax = gaSrcInfoTable[iCount].PerSrcInfo.ulCycles + 
                           gaSrcInfoTable[iCount].PerSrcInfo.uiMaxSeqNo;              
         ulTotalExpected = ulExtendedMax - gaSrcInfoTable[iCount].PerSrcInfo.ulBaseSeqNo + 1;
 
         /* total or cumulative packet loss */
         aRRBlocks[iCountValid].lCumulativeLost = ulTotalExpected 
                      - gaSrcInfoTable[iCount].PerSrcInfo.ulPktsReceived;
         /* Since this number is carried in 24 bits, it should be clamped 
          * at 0xFFFFFF,  mask off 8 bits from MSB side 
          */
	 aRRBlocks[iCountValid].lCumulativeLost &= 0x00FFFFFF; 
          
         ulExpectedInInterval = ulTotalExpected - gaSrcInfoTable[iCount].PerSrcInfo
                                                    .ulPktsExpectedAtLastInterval;
         gaSrcInfoTable[iCount].PerSrcInfo.ulPktsExpectedAtLastInterval = ulTotalExpected;

         ulReceivedInInterval = gaSrcInfoTable[iCount].PerSrcInfo.ulPktsReceived 
               - gaSrcInfoTable[iCount].PerSrcInfo.ulPktsReceivedAtLastInterval;

         gaSrcInfoTable[iCount].PerSrcInfo.ulPktsReceivedAtLastInterval 
                           = gaSrcInfoTable[iCount].PerSrcInfo.ulPktsReceived; 
        
         aRRBlocks[iCountValid].uiFractionLost = ulExpectedInInterval 
                                              - ulReceivedInInterval;
         if((ulExpectedInInterval == 0) || (aRRBlocks[iCountValid].uiFractionLost <= 0)) {
             aRRBlocks[iCountValid].uiFractionLost = 0;
         } else {    
             aRRBlocks[iCountValid].uiFractionLost = (aRRBlocks[iCountValid].uiFractionLost 
                                                       << 8) / ulExpectedInInterval;
         }
         aRRBlocks[iCountValid].ulLastSeqNo = gaSrcInfoTable[iCount].ulLastRecvdSeqNo;
         /* shifted rigth by 4 to keep the jitter in terms of Integer, refer 
          * to A.8 in RTP RFC1889 */ 
         aRRBlocks[iCountValid].ulJitter = gaSrcInfoTable[iCount].PerSrcInfo.ulJitter >> 4;
         aRRBlocks[iCountValid].ulLastSRTimestamp = gaSrcInfoTable[iCount].ulLastSRTimestamp;
       if(gaSrcInfoTable[iCount].bSRRecvd == TRUE ) // KUMAR
       {

         if(rtpGetTimeOfDay(&ulCurrentTimeSec, &ulCurrentTimeUSec) != SUCCESS ) {
             /* Error in rtpGetTimeOfDay() */
         }
       aRRBlocks[iCountValid].ulDelaySinceLastSR = time32bit(ulCurrentTimeSec, ulCurrentTimeUSec);
       aRRBlocks[iCountValid].ulDelaySinceLastSR = aRRBlocks[iCountValid].
             ulDelaySinceLastSR - 
                             gaSrcInfoTable[iCount].ulLastSRRecvTimestamp; 
       }
       else
         aRRBlocks[iCountValid].ulDelaySinceLastSR = 0; // KUMAR
         iCountValid++;
      } /* end of if(bValid) */
    } /* end of for loop */
   
    *pNextIndex = iCount; /* for the case where RRCount is more than or equal to 31 */    
    *piRRCount = iCountValid;

    return(SUCCESS);     
} /* end of RTCPRRBlocks() */


/*****************************************************************************
Function  : INT32 rtcpBuildSR(RTPInfo_t *pRTPSessionInfo, CHAR *pcRTCPPkt)


*****************************************************************************/

INT32 rtcpBuildSR(RTPInfo_t *pRTPSessionInfo, CHAR **pcRTCPPkt) 
{
   
    SenderReport_t         *pSR;
    INT16                  iRRCount;
    INT16                  iNextIndex;


    
    pSR = (SenderReport_t*) *pcRTCPPkt;

    /* SR Header */
    pSR->Header.uiVersion    = RTP_VERSION;
    pSR->Header.uiPadding    = 0x0;      /* No Padding */
    pSR->Header.uiPacketType = RTCP_SR;  // Hariv Changed from |= to = 

    pSR->ulSSRC = pRTPSessionInfo->ulSSRC;

    /* NTP and RTP Timestamps */
/*   rtcpTimestamps(&pSR->ulNTPTimestampSec, &pSR->ulNTPTimestampFrac, 
                                            &pSR->ulRTPTimestamp);
*/
    if(rtpGetTimeOfDay((INT32*)&pSR->ulNTPTimestampSec, (INT32*)&pSR->ulNTPTimestampFrac) > 0) {
         /* Error in rtpGetTimeOfDay() */
    }

//    pSR->ulRTPTimestamp = pRTPSessionInfo->ulTimestamp;
    /* RamReddy added this.
     * RTP_Timestamp = (int32)(current_NTP_time * RTP_timestamp_clock_rate) - offset */ 
    
    pSR->ulRTPTimestamp = (INT32)(pSR->ulNTPTimestampSec * 8000) - pRTPSessionInfo->ulOffset; 
   
    /* Total number of RTP Packets sent, in this session */ 
    pSR->ulTotalPacketsSent = pRTPSessionInfo->ulTotalPacketsSent;	

    /* Total number of RTP Octets sent, in this session */ 
    pSR->ulTotalOctetsSent = pRTPSessionInfo->ulTotalOctetsSent;	

    /* Prepare the RR Blocks for this session */
    if(rtcpRRBlocks(pRTPSessionInfo->iSessionIndex, pSR->aRRBlocks, &iRRCount, 
                          0, &iNextIndex) != SUCCESS) {
         DEBUG(PRINT("rtcpBuildSR: error in rtcpRRBlocks()\n"), DEBUG_EXCEPTION);
    }
 
    /* Total number of RR Blocks included inside the SR Report */
     pSR->Header.uiRRCount = iRRCount;
    
    /* To calculate the Packet Length, the size of actual number of RR Blocks 
     * is considered. The Header Size is excluded from this calculation.
     */  
    pSR->Header.uiPacketLength = sizeof(SenderReport_t) - sizeof(RTCPCommonPktHeader_t)
                                 - (sizeof(RTCPRR_t) * (MAX_RR_BLOCKS - iRRCount));

    /* Adjust the pcRTCPPkt to point to the location just after the last 
     * RR Block included 
     */
    *pcRTCPPkt += (sizeof(SenderReport_t) - (sizeof(RTCPRR_t) * 
                                             (MAX_RR_BLOCKS - iRRCount)));
    pSR->Header.uiPacketLength /= 4;   /* in terms of 32 bit words */ 
    /* if number of RR block is beyond 32 then add a RR report to 
     * accomodate rest of the RR blocks  
     */
    if((iRRCount >= 31) && (iNextIndex < MAX_RTP_SESSIONS)) {
        rtcpBuildRR(pRTPSessionInfo, iNextIndex, pcRTCPPkt);
    }

    return(SUCCESS);
}

/*****************************************************************************
Function : INT32 rtcpBuildRR(RTPInfo_t *pRTPSessionInfo, INT16 iNextIndex, 
                  CHAR **pcRTCPPkt)


*****************************************************************************/

INT32 rtcpBuildRR(RTPInfo_t *pRTPSessionInfo, INT16 iStartIndex, CHAR **pcRTCPPkt) 
{
    ReceptionReport_t *pRR;
    INT16           iRRCount;
    INT16           iNextIndex; 

    pRR = (ReceptionReport_t*) *pcRTCPPkt;

    /* RR Header */
    pRR->Header.uiVersion    = RTP_VERSION;
    pRR->Header.uiPadding    = 0x0;           /* No Padding */
    pRR->Header.uiPacketType = RTCP_RR;   // Hari V Changed |= to = 

    pRR->ulSSRC = pRTPSessionInfo->ulSSRC;

    /* Prepare the RR Blocks for this session */
    if(rtcpRRBlocks(pRTPSessionInfo->iSessionIndex, pRR->aRRBlocks, &iRRCount,\
                     iStartIndex, &iNextIndex) != SUCCESS) {
        DEBUG(PRINT("rtcpBuildRR: error in rtcpRRBlocks()\n"), DEBUG_EXCEPTION);
    }
 
    /* Total number of RR Blocks included inside the SR Report */
     pRR->Header.uiRRCount = iRRCount;
    
    /* To calculate the Packet Length, the size of actual number of RR Blocks 
     * is considered. The Header Size is excluded from this calculation.
     */  
    pRR->Header.uiPacketLength = sizeof(ReceptionReport_t) - sizeof(RTCPCommonPktHeader_t)
                                 - (sizeof(RTCPRR_t) * (MAX_RR_BLOCKS - iRRCount));


    /* Adjust the pcRTCPPkt to point to the location just after the last 
     * RR Block included 
     */
    *pcRTCPPkt += (sizeof(ReceptionReport_t) - (sizeof(RTCPRR_t) * 
                                              (MAX_RR_BLOCKS - iRRCount)));

    pRR->Header.uiPacketLength /= 4;   /* in terms of 32 bit words */ 

    return(SUCCESS);
}


/***************************************************************************
Function : rtcpHeaderValidate()

 RTCP Header Validity Checks
   The following checks can be applied to RTCP packets.
        o RTP version field must equal 2.
        o The payload type field of the first RTCP packet in a compound
         packet must be equal to SR or RR.
        o The padding bit (P) should be zero for the first packet of a
         compound RTCP packet because only the last should possibly need
         padding.
        o The length fields of the individual RTCP packets must total to
         the overall length of the compound RTCP packet as received.
         This is a fairly strong check.
   This function performs all of these checks. The packet type is not
   checked for subsequent packets since unknown packet types may
   be present and should be ignored.
*************************************************************************/

INT32 rtcpHeaderValidate(CHAR *pcRTCPInBuf, INT32 lRTCPPktLen,RTPInfo_t *pRtpInfo)
{
       RTCPCommonPktHeader_t   *pHeader;  /* RTCP header */
       RTCPCommonPktHeader_t   *pEnd;    /* end of compound RTCP packet */
       UINT32                  ulSSRC = 0;
       UINT32                  ulNTPTimestampSec;
       UINT32                  ulNTPTimestampFrac;  
       INT32                   iCount;
       BOOL                    bSR = FALSE;
       BOOL                    bBYE = FALSE;
       INT16 iIndex;
       iIndex  = pRtpInfo->iSessionIndex;

       pHeader = (RTCPCommonPktHeader_t*) pcRTCPInBuf;
       if ((*(UINT16 *)pHeader & RTCP_VALID_MASK) != RTCP_VALID_VALUE)
       {
          DEBUG(PRINT("rtcpHeaderValidate: something wrong with packet format\n"),
                     DEBUG_EXCEPTION);
          return(FAILURE);
       }
       /* lRTCPPktLen is the length of RTCP Comppound packet in terms 
        * of 32 bit words 
        */
       pEnd = (RTCPCommonPktHeader_t *)((UINT32 *)pHeader + lRTCPPktLen);
       ulSSRC = *((UINT32 *)pHeader + 1); 

       if(pHeader->uiPacketType == RTCP_SR || pHeader->uiPacketType == RTCP_RR) 
       {
           if(pHeader->uiPadding > 0) 
           {
               DEBUG(PRINT("rtcpHeaderValidate: Padding bit for very first packet should not be SET\n"), DEBUG_EXCEPTION); 
               return(FAILURE);
           }

           /* the consolidated length of all rtcp packets should equal the total 
            * length of the RTCP compound packet. and the Version in all rtcp 
            * packet headers must be equal to 2.
            */ 
           do 
           {
	      if(pHeader->uiPacketType == RTCP_SR) 
              {
                  bSR = TRUE;
                  ulNTPTimestampSec  = *((UINT32 *)pHeader + 2);
                  ulNTPTimestampFrac = *((UINT32 *)pHeader + 3);
#if RTCP_LOG
                  ucRRCnt =0;

                while(ucRRCnt <  pHeader->uiRRCount)
                {
                  pRRInfo = (RTCPRR_t *) (pHeader + 7+((ucRRCnt)*6)); 
                  rtcpLogRR(pRRInfo, pRtpInfo->ucEpIndex);
                  ucRRCnt++;
                }
#endif
              } 
              else if(pHeader->uiPacketType == RTCP_RR) 
              {
#if RTCP_LOG
                while(ucRRCnt <  pHeader->uiRRCount)
                {
                   pRRInfo = (RTCPRR_t *) (pHeader+2+(ucRRCnt*6));
                   rtcpLogRR(pRRInfo, pRtpInfo->ucEpIndex);
                   ucRRCnt++;
                }
#endif
              }
              else if(pHeader->uiPacketType == RTCP_BYE) 
              {
                  bBYE = TRUE;
#if SNMP_RTP
                  gaRTPInfo[iIndex].ulNoOfByes++;
#endif
#if SIP|MGCP
                  if(gaRTPInfo[iIndex].lRTCPTimerId != -1)
                  {
                    if(aditimerStop(gaRTPInfo[iIndex].lRTCPTimerId) == FAILURE)
                     printf("in bye :failed in timer stop\n");
                    gaRTPInfo[iIndex].lRTCPTimerId = -1;
                  }
#endif
              }
 
	      /* the constant '1' in this equation corresponds to the one 
               * 32 bit word ie. Size of the Common Header
               */
              pHeader = (RTCPCommonPktHeader_t *)((UINT32 *)pHeader 
                                       + pHeader->uiPacketLength + 1);

           }while((pHeader < pEnd) && (pHeader->uiVersion == 2));
       } else {
          DEBUG(PRINT("rtcpHeaderValidate: First packet must be either SR or RR \n"),
               DEBUG_EXCEPTION);
          return(FAILURE);
       }

       if (pHeader != pEnd) {
           DEBUG(PRINT("rtcpHeaderValidate: something wrong with packet format:end \n"),
                     DEBUG_EXCEPTION);
           return(FAILURE);
       }
       
       /* if Compound packet contains SR then store it's Timestamp in SRCInfoTable */
       if(bSR) {
           for(iCount=0; iCount < MAX_SOURCES; iCount++) {
             if((CheckElement(aucSITBitMask, MAX_SOURCES, iCount) == ELE_ALLOCATED)
                   && (gaSrcInfoTable[iCount].bValid) 
                             && (ulSSRC == gaSrcInfoTable[iCount].ulSSRC)) {
                 gaSrcInfoTable[iCount].bSRRecvd = TRUE; // KUMAR
                 gaSrcInfoTable[iCount].ulLastSRTimestamp = time32bit(ulNTPTimestampSec, ulNTPTimestampFrac);
                if(rtpGetTimeOfDay((INT32*)&ulNTPTimestampSec, (INT32*)&ulNTPTimestampFrac) > 0) {
         /* Error in rtpGetTimeOfDay() */
         DEBUG(PRINT("rtcpValidateHeader:Error getting timeofday\n"), DEBUG_EXCEPTION);
    }
               gaSrcInfoTable[iCount].ulLastSRRecvTimestamp =  time32bit(ulNTPTimestampSec, ulNTPTimestampFrac);
                 break;
             }
           } /* end of for loop */
       } /* end of if(bSR) */

       /* If packet contains BYE packet then remove this SSRC from the SRCInfoTable */
       if(bBYE) {
           for(iCount=0; iCount < MAX_SOURCES; iCount++) {
             if((CheckElement(aucSITBitMask, MAX_SOURCES, iCount) == ELE_ALLOCATED)
                     && (gaSrcInfoTable[iCount].bValid) 
                             && (ulSSRC == gaSrcInfoTable[iCount].ulSSRC)) {
                 if(ReleaseElement(aucSITBitMask, MAX_SOURCES, iCount) == FAILURE)
                 {
                     DEBUG(PRINT("Failed in releasing SRC bitmap\n"), DEBUG_EXCEPTION);
                 }
                 memset(&gaSrcInfoTable[iCount], 0, sizeof(gaSrcInfoTable[iCount]));
                 break;
             }
           } /* end of for loop */
       } /* end of if(bBYE) */

      return(SUCCESS);
}

/*****************************************************************************
Function : INT32 rtcpTimestamps(UINT32 *pulNTPTimestampSec, 
                 UINT32 *pulNTPTimestampFrac, *pulRTPTimestamp)


******************************************************************************/
INT32 rtcpTimestamps(UINT32 *pulNTPTimestampSec, UINT32 *pulNTPTimestampFrac, 
                 UINT32  *pulRTPTimestamp)
{
#ifdef NOT_NEEDED
    struct timeval  NTPTimestamp;

    if(rtpGetTimeOfDay(&NTPTimestamp, NULL) < 0) {
        DEBUG(PRINT("rtcpTimestamps(): Error in rtpGetTimeOfDay()\n"), 
                DEBUG_EXCEPTION);
        return(FAILURE);
    }

    *pulNTPTimestampSec = NTPTimestamp.tv_sec;
    *pulNTPTimestampFrac = NTPTimestamp.tv_usec;
    *pulRTPTimestamp = gulRTPTimestamp;
   
#endif
    return(SUCCESS);
}

/********************************************************************************
Function  : rtcpInterval(INT32 lMembers, INT32 lSenders, UINT32 ulRTCPBW, 
     BOOL bWeSent, INT32 lPacketSize, INT32 *plAvgRTCPSize, INT32 lInitial)

   The following function returns the time between transmissions of RTCP
   packets, measured in seconds. It should be called after sending one
   compound RTCP packet to calculate the delay until the next should be
   sent. This function should also be called to calculate the delay
   before sending the first RTCP packet upon startup rather than send
   the packet immediately. This avoids any burst of RTCP packets if an
   application is started at many sites simultaneously, for example as a
   result of a session announcement.
********************************************************************************/

INT32  rtcpInterval(INT32 lMembers, INT32 lSenders, UINT32 ulRTCPBW, 
          BOOL bWeSent, INT32 lPacketSize, INT32 *plAvgRTCPSize, INT32 lInitial)
{
       INT32   lInterval;     /* interval */
       INT32   lRTCPMinTime = RTCP_MIN_TIME;
       INT32   ln;           /* no. of members for computation */

       /*
        * Very first call at application start-up uses half the min
        * delay for quicker notification while still allowing some time
        * before reporting for randomization and to learn about other
        * sources so the report interval will converge to the correct
        * interval more quickly.  The average RTCP size is initialized
        * to 128 octets which is conservative (it assumes everyone else
        * is generating SRs instead of RRs: 20 IP + 8 UDP + 52 SR + 48
        * SDES CNAME).
        */

       if (lInitial) {
           lRTCPMinTime /= 2;
           *plAvgRTCPSize = 128;
       }

       /*
        * If there were active senders, give them at least a minimum
        * share of the RTCP bandwidth.  Otherwise all participants share
        * the RTCP bandwidth equally.
        */

       ln = lMembers;
       if (lSenders > 0 && lSenders < (lMembers / RTCP_SENDER_BW_DIVISER)) {
           if (bWeSent) {
               ulRTCPBW /= RTCP_SENDER_BW_DIVISER;
               ln = lSenders;
           } else {
               ulRTCPBW = ulRTCPBW - (ulRTCPBW / RTCP_SENDER_BW_DIVISER);
               ln -= lSenders;
           }
       }

       /*
        * Update the average size estimate by the size of the report
        * packet we just sent.
        */

        *plAvgRTCPSize += ((lPacketSize - *plAvgRTCPSize) / 16) ;

       /*
        * The effective number of sites times the average packet size is
        * the total number of octets sent when each site sends a report.
        * Dividing this by the effective bandwidth gives the time
        * interval over which those packets must be sent in order to
        * meet the bandwidth target, with a minimum enforced.  In that
        * time interval we send one report so this time is also our
        * average time between reports.
        */
 
       lInterval = ((*plAvgRTCPSize) * ln * 1000)/ ulRTCPBW;
       if(lInterval < lRTCPMinTime) {
           lInterval = lRTCPMinTime;
       }
       return(lInterval);

} /* end of RTCPInterval() */
UINT32 time32bit(UINT32 timeInSecs, UINT32 timeInUsecs)
{
	UINT32 ulTempVal;
	ulTempVal = 4294 * timeInUsecs;
	ulTempVal = ulTempVal >> 16;
	ulTempVal = (timeInSecs << 16) | ulTempVal;
	return(ulTempVal);
}

INT32 rtpGetTimeOfDay(INT32 *lSecs, INT32 *lUSecs)
{
  struct timeval tv;
  do_gettimeofday(&tv);
  *lSecs = (INT32) tv.tv_sec ; 
  *lUSecs =(INT32)  tv.tv_usec ;
  return (SUCCESS);
}

#endif

