#ifndef _RTPGDEF_H
#define _RTPGDEF_H

#define RTP_CNTRL_MSG_CNT 	10
#define RTP_UNICAST             0
#define RTP_MULTICAST        1
#define MAX_RTCP_SZ 500
#define SEND_RTCP_DATA   0x01
#define CREATE_SESSION 0x02
#define MODIFY_SESSION 0x03
#define DELETE_SESSION 0x04
#define MODIFY_SES_MODE   0x05
#define MODIFY_SES_CODEC   0x06

/* Different modes of session */
#define RTP_INACTIVE    0x01
#define RTP_SENDONLY    0x02
#define RTP_RECVONLY    0x03
#define RTP_SENDRECV    0x04
#define RTP_SESS_HOLD    0x05
#define RTP_SESS_XFER    0x06
#define RTP_SESS_CALLWAIT    0x07

/* Internal Data Structures */
#define NUM_EPS              MAX_VOICE_CHANNELS 
#define NUM_CONS             3
#define MAX_RTP_SESSIONS     NUM_EPS * NUM_CONS
#define MAX_SOURCES          NUM_EPS * NUM_CONS

/* RTP Port Range */
#define BASE_RTP_PORT  44000
#define LAST_RTP_PORT  49000

#define MAX_SESS_INFO_LEN     100
#define MAX_RFC2833_INFO_LEN  100

#define RTP_CMD      0xaa
#define RFC2833_CMD  0xab


/* Information pertaining each session */
typedef struct
{
  UINT32          ulDestIPAddr;		/* Destination IP Address.	   */
  UINT32          ulSrcIPAddr;		/* Source Address for this session */
  INT16           iSessionIndex;        /* Session Index.                  */
  UINT16          uiRTPPort; 		/* Src RTP port no of this session */
  UINT16          uiRTCPPort;           /* Src RTCP port no of this session*/
  UINT16          uiDestRTPPort;	/* Destination RTP port            */
  UINT16          uiDestRTCPPort;       /* Destination RTCP port           */
  UINT32          ulSSRC;		/* Source Indentifier.             */
  UINT16          uiSeqNum;             /* Sequence Number.		   */
  UINT32          ulTimestamp;          /* Time Stamp.                     */
  UINT32          ulOffset;             /* Offset.		           */
  UINT16          uiPayloadType;	/* Payload Type.		   */
  UINT16          uiDynPayloadType;	/* Dynamic Payload Type.           */
  INT16           iSessionBW;		/* RTP Session Band Width.	   */
  struct socket   *rtp_sock;            /* RTP socket structure.           */
  struct socket   *rtcp_sock;           /* RTCP Socket Structure.          */
  UINT32          ulTotalPacketsSent;   /* Total number of RTP Packets sent*/
  UINT32          ulTotalOctetsSent;    /* Total number of RTP Octets sent,*/
  BOOL            bPktSentInRTCPInterval;/* TRUE,if RTP Pkts are sent during 
                                            this RTCP Interval             */
  BOOL            bPktSentInLast2Intervals; /*TRUE,if RTP Pkts are sent in 
                                              last 2 RTCP Intervals        */
  UINT32          ulAvgRTCPSize;        /* Avg RTCP pkt size               */
  CHAR            cMulticast;           /* TRUE, then this session is      
                                           Multicast session               */
  CHAR            lRTCPTimerId;
  UINT32          ulRTCPInterval;       /* RTCP Time Interval.             */
  BOOL            bEndSession;          /* End Session Flag.               */
  BOOL            bVadSent;             /* VAD Packet flag.                */
  UCHAR           ucEpIndex;            /* Used to store endpoint index    */
#if SNMP_RTP
  UINT32          ulSessStart;          /* Session Start Time.             */
  UINT32          ulSenderJoins;        /* Total Number of Senders Joined. */
  UINT32          ulRcvrJoins;          /* Total Number of Receiver Joined.*/
  UINT32          ulNoOfByes;           /* Total Number of BYE.,           */
  UINT32          ulLastSRTime;         /* Last Send Report time.          */
  UINT32          ulLastRRTime;         /* Last Receiver Report Time.      */
#endif
  BOOL            bTimer;               /* RTCP Timer Flag.                */
  UCHAR           ucSigSessId;          /* Session ID.                     */
  UCHAR           ucMode;               /* Data Mode.                      */
  UCHAR           ucSessType;           /* Session Type.                   */
  BOOL            bRfc2833;             /* RFC 2833 Flag.                  */
  BOOL            bVirtualPort;         /* Virtual Flag.                   */
}RTPInfo_t;


/* RTP Session Structure. */
typedef struct
{
  UINT32  	ulSrcIPAddr;		/* Session Source IP Address.      */
  UINT32  	ulDestIPAddr;		/* Session Destination IP Address. */
  UINT16  	uiRTPPort;		/* RTP Port Number.		   */
  UINT16  	uiRTCPPort;		/* RTCP Port Number.		   */
  UINT16  	uiDestRTPPort;          /* Destinmation RTP Port.          */
  UINT16  	uiDestRTCPPort;	        /* Destination RTCP Port.          */
  INT16   	iSessionBW;             /* Session Band Width.             */
  UINT16  	uiPayloadType;		/* Payload Type.	           */
  UINT16  	uiDynPayloadType;       /* Dynamic Payload Type.           */
  UCHAR  	ucSigSessId;            /* Session Id.                     */
  UCHAR  	ucMode;                 /* Data Mode.                      */
  UCHAR  	rtp_cmd;                /* RTP Command.                    */
  UCHAR  	ucEpIndex;              /* Endpoiint Index.                */
  INT16  	iSessId;                /* Session Id.                     */
  UCHAR  	ucSessType;             /* Session Type.                   */
  BOOL   	bVirtualPort;           /* Virtual Port.                   */
}SessInfo_t;


/*structure used for passing parameters to rtpReceiveData and rtpPutInJItBuf fucntions*/
typedef struct {
        UCHAR *pcData;
        INT16 iPayloadLen;
        UINT16 uiSequenceNo;
        UINT32 ulRtpTimeStamp;
        UINT16 uiPayloadType;
        BOOL bMarkerBit;
        UINT16 uiSmpsPerFrm;
}Info_t;

/* Stuructures for CLI/SNMP/TTP */




/* Structure for Sender Report  Statistics*/
typedef struct ComSendRep {
     INT16     iSessionIndex;
     UINT32    ulSenderIpAddr  ;
     UINT32    ulSenderRTPPort  ;
     UINT32    ulTotalPacketsSent;
     UINT32    ulTotalOctetsSent;
     UINT32    ulTotalSRs;
     UINT32    ulLastSRTime;
     UINT16    uiPayloadType;
} ComSRInfo_t;


/* Structure for Receiver Report Statistics */
typedef struct RRInfo {
     INT16    iSessionIndex;
     UINT32   ulFromIpAddr;
     UINT32   ulFromRTPPort;
     UCHAR    aucCName;

     UINT32   ulCumLostPackets;
     UINT32   ulFractionLost;

     UINT32   ulRcvrJitter ;
     UINT32   ulRcvrRRTime   ;
     UINT32   ulTotalRcvdPkts;
     UINT32   ulTotalRcvdOctets;
}ComRRInfo_t;


typedef struct rtpConfig {
     UINT32 ulSrcIpAddr;
     UINT32 uiMaxRtpPort;
     UINT32 uiMinRtpPort;
}rtpConfig_t;

/* Interface structure among rtp, rfc2833, application */
typedef struct
{
   UINT32   ulMtype;
   CHAR     acMtext[MAX_SESS_INFO_LEN];
   UCHAR    aucRfc2833Info[MAX_RFC2833_INFO_LEN];
}RTPMessageInfo_t;



/* Table needed by the RTCP module, and global between RTP and RTCP modules.
 * This table maintains the status/statistics for each source (ie each SSRC).
 */

/* Per-source state information    */
/* Almost all of the fields are filled in by RTP module */
typedef struct PerSrcInfo_t 
{
       UINT16     uiMaxSeqNo;         /* highest seq. number seen */
       UINT32     ulCycles;           /* shifted count of seq. number cycles */
       UINT32     ulBaseSeqNo;        /* base seq number */
       UINT32     ulLastBadSeqNo;     /* last 'bad' seq number + 1 */
       UINT32     ulProbation;        /* sequ. packets till source is valid */
       UINT32     ulPktsReceived;     /* packets received */
       UINT32     ulOctetsReceived;   /* octets received */
       UINT32     ulPktsExpectedAtLastInterval;  /* packet expected at  last interval */
       UINT32     ulPktsReceivedAtLastInterval;  /* packet received at last  interval */
       UINT32     ulTransitTimeforPrevPkt;       /* relative transit time for prev pkt */
       UINT32     ulJitter;                      /* estimated jitter */
       UINT32     ulAvgTransmitDel;   /* AvgTrnansmissionDelay  */

}PerSrcInfo_t;

typedef struct SrcInfoTable_t 
{
       BOOL       bValid;                 /* If TRUE, then it's a Valid element */
       BOOL       bSRRecvd;
       INT16      iSessionIndex;
       UINT32     ulSSRC;
       PerSrcInfo_t  PerSrcInfo;
       UINT32     ulLastRecvdSeqNo;       /* Seq No of last RTP packet received */
       BOOL       bPktRecvdInRTCPInterval;/* TRUE, if RTP Packets are sent to this
                                                   SSRC during the RTCP Interval */
       UINT32     ulLastSRTimestamp;  /* Timestamp of Last SR received from this source */
       UINT32     ulLastSRRecvTimestamp;  /* Timestamp when SR received from this source */
       UINT32      ulStartTime;

} SrcInfoTable_t;


INT16 rtpSocketClose(struct socket *sock);
INT16 rtpCreateSession(SessInfo_t *SessInfo);
INT16 rtpSocketOpen(struct socket **sock,UINT32 ulSrcAddr, UINT32 ulSrcPort);
INT16 rtpSendData(struct socket *sock, CHAR *pcData, INT16 iLen,
                              UINT32 ulDestIPAddr, UINT16 uiDestRTPPort);
INT16 rtpGetSessId(SessInfo_t *pSessInfo);
INT16 rtpModifySession( SessInfo_t *SessInfo);
INT16 rtpDeleteSession(UINT16 uiSessId);
INT16 rtpModifySessMode(UINT16 uiSesId,UCHAR ucMode,UCHAR ucEpIndex);
INT16 rtpModifySessCodec(INT16 iSessId,UINT16 uiPayloadType, UCHAR ucEpIndex);
INT32 rtpFrameMsg(VoiceQ_t *pVoice, UCHAR ucEpIndex);
int rtpProcessControlMessage(unsigned long param);
int rtpCreateAPFlow(UINT32 src,
                    UINT16 sport,
                    UINT32 dst,
                    UINT16 dport,
                    UCHAR  bCreate);

INT32 rtcpSendData(RTPInfo_t *pRTPSessionInfo, UINT32 *pulPacketSize);
INT32  rtcpInterval(INT32 lMembers, INT32 lSenders, UINT32 ulRTCPBW,
          BOOL bWeSent, INT32 lPacketSize, INT32 *plAvgRTCPSize, INT32 lInitial);

VOID rtcpHandleIntervalTimer(unsigned long data);
INT32 rtcpBuildRR(RTPInfo_t *pRTPSessionInfo, INT16 iStartIndex, CHAR **pcRTCPPkt);
UINT32 time32bit(UINT32 timeInSecs, UINT32 timeInUsecs);
INT32 rtpGetTimeOfDay(INT32 *lSecs, INT32 *lUSecs);
INT32 rtcpBuildCompPkt(RTPInfo_t *pRTPSessionInfo, UINT32 *pulPacketSize);
int rtpInit(void);
INT32 rtcpBuildSR(RTPInfo_t *pRTPSessionInfo, CHAR **pcRTCPPkt);
INT32 rtcpHeaderValidate(CHAR *pcRTCPInBuf, INT32 lRTCPPktLen,RTPInfo_t *pRtpInfo);
VOID sendRTCPData(VOID *pRTPInfo);
INT32 rtpParseMsg(RTPInfo_t *pRtpInfo, UCHAR *pRTPData,
                                  UINT16 uiLen, VoiceNWPkt_t *pJitPkt);
#endif
