#ifndef __RTP_MSG_H
#define __RTP_MSG_H

#define RTP_VERSION          2         /* Current protocol version.   */
#define RTP_SEQ_MOD          (1<<16)
#define MAX_RR_BLOCKS        4
#define MAX_SDES_ITEMS       5
#define MAX_SSRC_LIST        3
#define UDP_HDR_SIZE         28
#define IP_HDR_SIZE          20
#define FRAME_HDR_SIZE       14
#define MAX_DROPOUT          300
#define MAX_MISORDERED       300
#define MIN_SEQUENTIAL       2
#define PROBATION            0
#define MAX_RTPPKT_SIZE      800

/* Excess number of RR Blocks over 31, inside SR */
#define EXCESS_RR_BLOCKS  0 //Changed on 30/03/2002-- kumar

/* Total number of RTCPPackets in a Compound pkt = 4 (ie. SR, RR, SDES, BYE)
 * + (Excess number of RR blocks from SR)
 */
#define RTCP_COMP_PKT_SIZE ( sizeof(struct SenderReport_t) + (sizeof(struct ReceptionReport_t) * ( EXCESS_RR_BLOCKS)) + sizeof(struct RTCPSDES_t) + sizeof(struct RTCPBYE_t) )    // -- kumar

/* Big-endian mask for version, padding bit and packet type pair  */
#define RTCP_VALID_MASK         (0xc000 | 0x2000 | 0xfe)
#define RTCP_VALID_VALUE        ((RTP_VERSION << 14) | RTCP_SR)

#define MAX_SDES_TEXT_SIZE   50
#define MAX_SDES_ITEMS   5

/* SDES item  */
typedef struct RTCPSDESItem {
UCHAR      ucType;                      /* type of item  */
UCHAR      ucLength;                    /* length of item (in octets) */
CHAR       acData[MAX_SDES_TEXT_SIZE];  /* text, not null-terminated , max 255 octets */
} RTCPSDESItem_t;


/* RTP Header */
typedef struct
{
    UINT16 uiVersion:2; /* protocol version */
    UINT16 uiP:1;       /* padding flag */
    UINT16 uiX:1;       /* header extension flag */
    UINT16 uiCC:4;      /* CSRC count */
    UINT16 uiM:1;       /* marker bit */
    UINT16 uiPt:7;      /* payload type */
    UINT16 uiSeq;       /* sequence number */
    UINT32 ulTs;        /* timestamp */
    UINT32 ulSSRC;      /* synchronization source */
} RTPHdr_t;

typedef enum RTCPPacketType_e 
{
	RTCP_SR   = 200,
	RTCP_RR   = 201,
	RTCP_SDES = 202,
	RTCP_BYE  = 203,
	RTCP_APP  = 204
} RTCPPacketType_e;


typedef enum RTCPSDESType_e 
{
	SDES_END,
	SDES_CNAME,
	SDES_NAME,
	SDES_EMAIL,
	SDES_PHONE,
	SDES_LOC,
	SDES_TOOL,
	SDES_NOTE,
	SDES_PRIV
} RTCPSDESType_e;


/* RTCP common header word for all types of Packets */
typedef struct RTCPCommonPktHeader_t {
UINT16     uiVersion:2;      /* protocol version */
UINT16     uiPadding:1;      /* padding flag */
UINT16     uiRRCount:5;      /* RR packet count in this packet varies by packet type */
UINT16     uiPacketType:8;   /* RTCP packet type */
UINT16     uiPacketLength;   /* pkt len in 32 bit words,  w/o this word */
} RTCPCommonPktHeader_t;


/* Reception Report block (RR Blocks) */
typedef struct RTCPRR_t {
UINT32     ulSSRC;                      /* data source being reported */
UINT16     uiFractionLost:8;            /* fraction lost since last SR/RR */
INT32      lCumulativeLost:24;          /* cumul. no. pkts lost (signed!) */
UINT32     ulLastSeqNo;                 /* extended last seq. no. received */
UINT32     ulJitter;                    /* interarrival jitter */
UINT32     ulLastSRTimestamp;           /* last SR packet from this source */
UINT32     ulDelaySinceLastSR;          /* delay since last SR packet */
} RTCPRR_t;


/* Sender Report (SR) Packet */
typedef struct SenderReport_t{
RTCPCommonPktHeader_t     Header;
UINT32                    ulSSRC;                  /* sender generating this report */
UINT32                    ulNTPTimestampSec;       /* NTP timestamp */
UINT32                    ulNTPTimestampFrac;
UINT32                    ulRTPTimestamp;          /* RTP timestamp */
UINT32                    ulTotalPacketsSent;      /* packets sent */
UINT32                    ulTotalOctetsSent;       /* octets sent */
RTCPRR_t                  aRRBlocks[MAX_RR_BLOCKS];  /* variable-length list */
}SenderReport_t;

/* Reception Report (RR) Packet */
typedef struct ReceptionReport_t{
RTCPCommonPktHeader_t     Header;
UINT32                    ulSSRC;                  /* receiver generating this report */
RTCPRR_t                  aRRBlocks[MAX_RR_BLOCKS];  /* variable-length list */
}ReceptionReport_t;

/* Source Description (SDES) Packet */
typedef struct RTCPSDES_t {
RTCPCommonPktHeader_t     Header;
UINT32                    ulSRC;                      /* first SSRC/CSRC */
RTCPSDESItem_t            aSDESItems[MAX_SDES_ITEMS];  /* list of SDES items */
} RTCPSDES_t;


/* BYE Packet */
typedef struct RTCPBYE_t {
RTCPCommonPktHeader_t     Header;
UINT32                    aulSRC[MAX_SSRC_LIST];       /* list of sources (SSRC/CSRC) */

} RTCPBYE_t;


#endif


