#include <voice_typedef.h>
#include <voicelib.h>
#if KRTP
#include <linux/netfilter.h>
#include <linux/spinlock.h>
#include <voice_bitmap.h>
#include <linux/in.h>
#include <dspipc.h>
#include "rtpgdef.h"
#include "rtpmsg.h"


/*************************************************************************
 * Function Name	:	rtpSocketOpen.				 *
 * Description          :	Function to open UDP socket.             *
 * Input		:	None.					 *
 * Output		:	Socket.					 *
 * Return Value		:	SUCCESS / FAILURE.			 *
 ************************************************************************/ 
INT16 rtpSocketOpen(struct socket **sock,UINT32 ulSrcAddr, UINT32 ulSrcPort)
{
  int status;
  struct sockaddr_in addr;
  status = sock_create(AF_INET,SOCK_DGRAM,IPPROTO_UDP, sock);
  if(status < 0)
    return FAILURE;
  addr.sin_port = ulSrcPort;
  addr.sin_addr.s_addr = ulSrcAddr;
  status  = (*sock)->ops->bind(*sock,(struct sockaddr *)&addr,sizeof(addr));
  if(status < 0)
    printk("Could not bind RTP socket %x/%d\n",ulSrcAddr,ulSrcPort);
    
  return SUCCESS;
}
/*************************************************************************
 * Function Name        :       rtpSocketClose.                          *
 * Description          :       Function to close UDP socket.            *
 * Input                :       Socket.                                  *
 * Output               :       None.                                    *
 * Return Value         :       SUCCESS / FAILURE.                       *
 ************************************************************************/
INT16 rtpSocketClose(struct socket *sock)
{
  if(sock == NULL)
    return FAILURE;
  sock_release(sock);
  return SUCCESS;
}
/*************************************************************************
 * Function Name        :       rtpSendData.                             *
 * Description          :       Function to send UDP packet.             *
 * Input                :       socket, data to send, length of data.    *
 *                              Destination address, destination Port.   *
 * Output               :       None.                                    *
 * Return Value         :       SUCCESS / FAILURE.                       *
 ************************************************************************/
INT16 rtpSendData(struct socket *sock, 
                  CHAR *pcData, 
                  INT16 iLen,
                  UINT32 ulDestIPAddr, 
                  UINT16 uiDestRTPPort)
{
  struct msghdr   msg;
  struct sockaddr_in DestAddr;
  struct iovec  iov;
  int error;
  if(sock == NULL)
    return FAILURE;
  DestAddr.sin_family=AF_INET;
  DestAddr.sin_port=htons(uiDestRTPPort);
  DestAddr.sin_addr.s_addr = htonl(ulDestIPAddr);
  iov.iov_base=(void *)pcData;
  iov.iov_len=iLen;
  msg.msg_name= &DestAddr;
  msg.msg_namelen=sizeof(DestAddr);
  msg.msg_iov=&iov;
  msg.msg_iovlen=1;
  msg.msg_control=NULL;
  msg.msg_controllen=0;
  msg.msg_flags = MSG_DONTWAIT;
#if RTP_SOCKETS_DEBUG_ON
  printk("ulDestIPAddr = %x\n",ulDestIPAddr);
  printk("uiDestRTPPort = %d\n",uiDestRTPPort);
  printk("len = %d\n",iLen);
#endif
  error = sock_sendmsg(sock, &msg, (size_t)iLen);
  if(error <0)
  {
    printk("Error in sending data %x %d\n",
                  DestAddr.sin_addr.s_addr, DestAddr.sin_port);
  }
  return SUCCESS;
}
#endif
