/* ============================================================================
 *
 * Copyright (C) 2001 Analog Devices Inc. All rights reserved
 *
 * The information and source code contained herein is the exclusive property
 * of Analog Devices and may not be disclosed, examined or reproduced in whole
 * or in part without the explicit written authorization from Analog Devices.
 *
 * ============================================================================
 */
#include <oswrapper.h>
#if LINUX_2_6
//#include <adi6843.h>
#include "../../../rg/os/linux-2.6/include/linux/sbi_defs.h"
#endif

UINT32 intrFlags;
static unsigned int dspIRQ = 3; /* Highest Priority to DSP Interrupts */
void osDSP1Tasklet(unsigned long param);
DECLARE_TASKLET(voip_dsp1_tasklet, osDSP1Tasklet, 0);
#if (MAX_CHANNELS_ISR == 1)
void osDSP2Tasklet(unsigned long param);
DECLARE_TASKLET(voip_dsp2_tasklet, osDSP2Tasklet, 0);
#endif

#if SLIC_MODULES
extern void  slicTasklet(unsigned long param);
DECLARE_TASKLET(voip_process_host_cmd_tasklet, slicTasklet, 0);
#endif

#if DRV_DEBUG_STAT
extern VoiceDebugStat_t VoiceDebugStat[];
#endif

#if DYNAMIC_CODEC_DOWNLOAD
UCHAR dspDynDownloadStart = 0;
DECLARE_WAIT_QUEUE_HEAD(dsp_dyn_dld_thread);
#endif

#if PROFILE
#define READ_REG_16(a,b) (*((unsigned short *)( (a) + (b) ) ))
#define CPE_TIMER1_BASE_ADDRESS 0xb9070000
int myTestValue[10000];
int PrevClk,CurrClk;
#endif

static int dspDeviceId1 = 0;
static int dspDeviceId2 = 1;

int myTestVal[2] = {0,0};
int myTestValueCnt = 0;
extern unsigned short dspTimeSlot0, dspTimeSlot1;

#if EXT_MODULE
extern dsp_regExtModForClock_str *dsp_regExtMod;
#endif

// Mbe (07/2009)
#if KERNEL_RTP_INTERFACE
extern dsp_regNetwork_t *pNWCallBack;
#endif
#if (MAX_CHANNELS_ISR == 1)
/*****************************************************************
 *  Function Name : osDSP2Tasklet.                               *
 *  Description   : Tasklet functionality for the tasklet.       *
 *  Input         : None.                                        *
 *  Output        : None.                                        *
 *  Return Value  : None.                                        *
 ****************************************************************/
void osDSP2Tasklet(unsigned long param)
{
#ifdef VOICE_TASKLET_MODE
  dspipcIsr(1);
#endif

#if DRV_DEBUG_STAT
  VoiceDebugStat[1].taskletCount++;
#endif
//Mbe (07/2009)
#if KERNEL_RTP_INTERFACE
  /* Careful need to be before dspipcGetVoicePktCnt */
  if ( (pNWCallBack != (dsp_regNetwork_t *)0) && (pNWCallBack->clockFromDsp != (dsp_network_clockFromDsp_f)0) ) {
     pNWCallBack->clockFromDsp(1);
  }
#endif
  dspipcGetVoicePktCnt(2);
#if T38
  dspipcGetT38PktCnt(2);
#endif
// dtr bugzilla miniV2 7901 :
// la commande dspWriteIntoDataMem(dspNumber,0x3fb2, 1) est supprime des tasklets osDSP1Tasklet(),
// osDSP2Tasklet() et introduite  la fin de la routine d'interruption dspipcIsr()
// pour supprimer la perte d'interruptions 10 ms des DSP en presence de trafic WAN/LAN
// (notamment download ftp ADSL/WIFI > 16/17 Mbps et upstream udp de paquets IP de qq d'10 d'octets
// ETH/ADSL et WIFI/ADSL).
//  dspWriteIntoDataMem(1,0x3fb2, 1);
}
#endif

/*****************************************************************
 *  Function Name : osDSP1Tasklet.                                *
 *  Description   : Tasklet functionality for the tasklet.       *
 *  Input         : None.                                        *
 *  Output        : None.                                        *
 *  Return Value  : None.                                        *
 ****************************************************************/
void osDSP1Tasklet(unsigned long param)
{
#if DRV_DEBUG_STAT
  VoiceDebugStat[0].taskletCount++;
#endif

#ifdef VOICE_TASKLET_MODE
  dspipcIsr(0);
#endif

// Mbe (07/2009)
#if KERNEL_RTP_INTERFACE
  /* Careful need to be before dspipcGetVoicePktCnt */
  if ( (pNWCallBack != (dsp_regNetwork_t *)0) && (pNWCallBack->clockFromDsp != (dsp_network_clockFromDsp_f)0) ) {
     pNWCallBack->clockFromDsp(0);
  }
#endif
  dspipcGetVoicePktCnt(2);
#if T38
  dspipcGetT38PktCnt(2);
#endif
// dtr bugzilla miniV2 7901 :
// la commande dspWriteIntoDataMem(dspNumber,0x3fb2, 1) est supprime des tasklets osDSP1Tasklet(),
// osDSP2Tasklet() et introduite  la fin de la routine d'interruption dspipcIsr()
// pour supprimer la perte d'interruptions 10 ms des DSP en presence de trafic WAN/LAN
// (notamment download ftp ADSL/WIFI > 16/17 Mbps et upstream udp de paquets IP de qq d'10 d'octets
// ETH/ADSL et WIFI/ADSL).
//  dspWriteIntoDataMem(0,0x3fb2, 1);
}

/*****************************************************************
 *  Function Name : osLockInterrupt.                             *
 *  Description   : Function to disable the core interrupts.     *
 *  Input         : None.                                        *
 *  Output        : None.                                        *
 *  Return Value  : None.                                        *
 ****************************************************************/
void osLockInterrupts(void)
{
#if LINUX_2_6
  local_irq_save(intrFlags);
#else
  save_flags(intrFlags);
  cli();
#endif
}

/*****************************************************************
 *  Function Name : osUnlockInterrupt.                           *
 *  Description   : Function to enable the core interrupts.      *
 *  Input         : None.                                        *
 *  Output        : None.                                        *
 *  Return Value  : None.                                        *
 ****************************************************************/
void osUnlockInterrupts(void)
{
#if LINUX_2_6
  local_irq_restore(intrFlags);
#else
  restore_flags(intrFlags);
#endif
}
void osLockTasklet(UINT16 PortNum)
{
  if(in_interrupt() == 1)
    return ;
  if(PortNum % 2)
  {
    /* DSP -1  */
    tasklet_disable(&voip_dsp2_tasklet);
  }
  else
    tasklet_disable(&voip_dsp1_tasklet);
}

void osUnlockTasklet(UINT16 PortNum)
{
  if(in_interrupt() == 1)
    return ;
  if(PortNum % 2)
  {
    /* DSP -1  */
    tasklet_enable(&voip_dsp2_tasklet);
  }
  else
    tasklet_enable(&voip_dsp1_tasklet);
}

/*****************************************************************
 *  Function Name : osDSPIntHandler.                             *
 *  Description   : Function to handle DSP Interrupt.            *
 *  Input         : None.                                        *
 *  Output        : None.                                        *
 *  Return Value  : 0 on success else error.                     *
 ****************************************************************/
int osDSPIntHandler(int irq, void *dev_id, struct pt_regs *regs)
{
  int index = *((int*)dev_id);
  if(index == 1)
  {
    /* Clear the latched PF4 bit */
    *(volatile UINT32*)DSP_LATCH_ADDR_HI = 0x00010000;
#ifndef VOICE_TASKLET_MODE
    dspipcIsr(1);
#endif
    tasklet_hi_schedule(&voip_dsp2_tasklet);
#if DRV_DEBUG_STAT
    if(VoiceDebugStat[1].bLog == TRUE)
      VoiceDebugStat[1].intCount++;
#endif
  }
  else
  {
#if DRV_DEBUG_STAT
    if(VoiceDebugStat[1].bLog == TRUE)
      VoiceDebugStat[0].intCount++;
#endif
    /* Clear the latched PF4 bit */
    *(volatile UINT32*)DSP_LATCH_ADDR_HI = 0x1;
#ifndef VOICE_TASKLET_MODE
    dspipcIsr(0);
    slicTasklet(0);
#endif
    tasklet_hi_schedule(&voip_dsp1_tasklet);
#ifdef VOICE_TASKLET_MODE
    tasklet_hi_schedule(&voip_process_host_cmd_tasklet);
#endif
  }

#if EXT_MODULE
  /* If external module registered */
  if ( (dsp_regExtMod) && (dsp_regExtMod->dspNumber == index) ) {
    /* Callback */
    dsp_regExtMod->callback(dsp_regExtMod->ref);
  }
#endif

#if LINUX_2_6
  return IRQ_HANDLED;
#else
  return 0;
#endif
}
/*****************************************************************
 *  Function Name : osAssignInterrupt.                           *
 *  Description   : Function to enable DSP Interrupts.           *
 *  Input         : DSP Number.                                  *
 *  Output        : None.                                        *
 *  Return Value  : 0 on success else error.                     *
 ****************************************************************/
int osAssignInterrupt(int dsp_num)
{
  UINT32 result;
  int *pDevice;
  UCHAR *pName;
  if(dsp_num ==0 )
  {
    dspDeviceId1 = 0;
    pDevice = &dspDeviceId1;
    pName = DSP1_DEVICE_NAME;
#if LINUX_2_6
    dspIRQ = DSP0_FL0_INT;
#endif
  }
  else
  {
    dspDeviceId2 =1 ;
    pDevice = &dspDeviceId2;
    pName = DSP2_DEVICE_NAME;
#if LINUX_2_6
    dspIRQ  = DSP1_FL0_INT;
#endif
  }
  /* Register IRQ */
  result = request_irq(dspIRQ,
                       osDSPIntHandler,
#if LINUX_2_6
                       SA_INTERRUPT,
#else
                       SA_SHIRQ | SA_INTERRUPT,
#endif
                       pName,
                       pDevice);

  switch(result)
  {
    case -EBUSY:
      printk(KERN_ERR ": IRQ %d busy\n", dspIRQ);
      return -EBUSY;
    case -EINVAL:
      printk(KERN_ERR ": Bad irq number or handler\n");
      return -EINVAL;
  }
#if !LINUX_2_6
  enable_irq(dspIRQ);
#endif
  printk(	"%s: Enable IRQ(%d) for DSP \n",
		__FUNCTION__,
		dspIRQ);

  DEBUG(PRINT("\n Enable IRQ for DSP \n"),DEBUG_INFO);
  /* Clear DSP Lateched PF4 Bit */
  return 0;
}
void osDisableSystem(void)
{
  /* Wait for the tasklet */
  osLockTasklet(0);
  osLockTasklet(1);
  osLockInterrupts();
  tasklet_kill(&voip_dsp1_tasklet);
  tasklet_kill(&voip_dsp2_tasklet);
  /* Reset DSP */
  *((UINT32 *)CPE_PER_INT) = 0x2000200;
  osUnlockInterrupts();
}
/**********************************************************************************
 *  Function Name  : osDownloadDSPImage.                                          *
 *  Description    : Function to Download DSP Image.                              *
 *  Input          : DSP Image Data, DSP Number, type of download                 *
 *  Output         : None.                                                        *
 *  Return Value   : None.                                                        *
 *********************************************************************************/
int osDownloadDSPImage(UINT16 *pDSPData, int dspNum)
{
  UINT32 k = 0;
  UINT16 control_word1;
  UINT16 control_word2;
  UINT16 count;
  k = 0;
  count  = pDSPData[k];
  while (count != 0xFFFF)
  {
    k = k+1;
    control_word1 = pDSPData[k];
    k = k+1;
    control_word2 = pDSPData[k];
    k = k+1;
    dspDownloadMem(dspNum,control_word1, control_word2, &pDSPData[k], count);
    k = k+count;
    count = pDSPData[k];
#if DYNAMIC_CODEC_DOWNLOAD
    if(dspipcGetVoicePktCnt(1) != 0)
    {
        dspDynDownloadStart = 1;
        interruptible_sleep_on_timeout(&dsp_dyn_dld_thread,2);
        dspDynDownloadStart = 0;
    }
#endif
  }
  return 0;
}
#if DYNAMIC_CODEC_DOWNLOAD
void wakeupDynDownload(void)
{
  wake_up_interruptible(&dsp_dyn_dld_thread);
}
#endif

#if PROFILE
void voxTimer2Init(void)
{
  unsigned long addr;
  unsigned long period;
  unsigned long width;
  period = 0xFFFFFFFF;
  width = period - 1;
  printk("Initializing VOX Timer - 2 \n");
  osLockInterrupts();
  addr = CPE_TIMER1_BASE_ADDRESS + 0x24;
  outw( 0x0d, addr );
  /* Write top half of period register */
  addr = CPE_TIMER1_BASE_ADDRESS + 0x34;
  outw( (period >> 16), addr );
  /* Write bottom half of period register */
  addr = CPE_TIMER1_BASE_ADDRESS + 0x30;
  outw( (period & 0x00FFFF), addr );
  /* Write top half of width register */
  addr = CPE_TIMER1_BASE_ADDRESS + 0x3c;
  outw( (width >> 16), addr );
  /* Write bottom half of width register */
  addr = CPE_TIMER1_BASE_ADDRESS + 0x38;
  outw( (width & 0x00FFFF), addr );
  /* Set TIMEN0 and clear any lingering interrupts */
  addr = CPE_TIMER1_BASE_ADDRESS + 0x20;
  outw( 0x0400, addr );
  osUnlockInterrupts();
}



void myProfileTime(int index)
{
  CurrClk =       READ_REG_16(CPE_TIMER1_BASE_ADDRESS, 0x28)
                 | (READ_REG_16(CPE_TIMER1_BASE_ADDRESS, 0x2c) << 16);
  myTestValue[index] = CurrClk - PrevClk;
  PrevClk = CurrClk;

}

#endif
