/*======================================================================
 * File Name	:	legerity88211.h
 * Description	:	codec related definations
 *======================================================================
 */

/*======================================================================
 * History
 *
 *    Date		      Author			           Comment
 * 11/23/2004       Samuel Weng      	       Initial version
 * 12/01/2004       Samuel Weng                V1.1
 *======================================================================
 */

#ifndef _LEGERITY88211_H_
#define _LEGERITY88211_H_

#ifdef __cplusplus
extern "C" {
#endif

#ifdef LEGERITY_UNIT
void UT_CODEC_MODESETTING(int iPort,int iMode);
void UT_CODEC_HOOKSTATUS(int iPort);
void UT_CODEC_READ_REG(unsigned char ucAddr,int length);
#endif

#define LEGERITY_UNIT 1 /* For testing */

/* Legerity registers */
#define CODEC_REG_HW_RESET 			0x04
#define CODEC_REG_CFG_W    			0x46
#define CODEC_REG_CFG_R				0x47
#define CODEC_REG_REGULATOR_CTL_W	0xE6
#define CODEC_REG_REGULATOR_CTL_R	0xE7
#define CODEC_REG_REGULATOR_PARA_W	0xE4
#define CODEC_REG_REGULATOR_PARA_R	0xE5
#define CODEC_REG_TS_EDGE_W         0x44
#define CODEC_REG_TS_EDGE_R         0x45
#define CODEC_REG_CH_CHAN_ENABLE_W	0x4A
#define CODEC_REG_CH_CHAN_ENABLE_R  0x4B
#define CODEC_REG_READ_SIGNAL_POLL	0x4D
#define CODEC_REG_READ_SIGANL_INTR	0x4F
#define CODEC_REG_R_FLITER_W        0x8A
#define CODEC_REG_R_FLITER_R        0x8B
#define CODEC_REG_Z_IIR_FLITER_W    0x9A
#define CODEC_REG_Z_IIR_FLITER_R    0x9B
#define CODEC_REG_Z_FIR_FILITER_W	0x98
#define CODEC_REG_Z_FIR_FILITER_R	0x99
#define CODEC_REG_VOICE_GAIN_W      0x50
#define CODEC_REG_VOICE_GAIN_R      0x51
#define CODEC_REG_GX_FLITER_W 		0x80
#define CODEC_REG_GX_FLITER_R 		0x81
#define CODEC_REG_GR_FLITER_W		0x82
#define CODEC_REG_GR_FLITER_R		0x83
#define CODEC_REG_B_FIR_FLITER_W	0x86
#define CODEC_REG_B_FIR_FLITER_R	0x87
#define CODEC_REG_X_FLITER_W		0x88
#define CODEC_REG_X_FLITER_R		0x89
#define CODEC_REG_B_IIR_FLITER_W	0x96
#define CODEC_REG_B_IIR_FLITER_R	0x97
#define CODEC_REG_DISN_W			0xCA
#define CODEC_REG_DISN_R			0xCB
#define CODEC_REG_OPR_CONDITION_W	0x70
#define CODEC_REG_OPR_CONDITION_R	0x71
#define CODEC_REG_OPR_CONDITON      0x71
#define CODEC_REG_OPR_FUN_W			0x60
#define CODEC_REG_OPR_FUN_R			0x61
#define CODEC_REG_TX_TS_W			0x40
#define CODEC_REG_TX_TS_R			0x41
#define CODEC_REG_RX_TS_W			0x42
#define CODEC_REG_RX_TS_R			0x43
#define CODEC_REG_SIG_GEN_PARA		0xD2
#define CODEC_REG_CANDECE_TIMER		0xE0
#define CODEC_REG_SIG_GEN_CTL		0xDE
#define CODEC_REG_LOOP_SUPERVISION_W 0xC2
#define CODEC_REG_LOOP_SUPERVISION_R 0xC3
#define CODEC_REG_DC_FEED_W   		0xC6
#define CODEC_REG_DC_FEED_R   		0xC7
#define CODEC_REG_SYS_STATE_W		0x56
#define CODEC_REG_SYS_STATE_R		0x57
#define CODEC_REG_IO_DIRECTION		0x54
#define CODEC_REG_IO_DATA			0x52

/* Codec state */
#define CODEC_STATE_DISCONNECT		0x00
#define CODEC_STATE_IDLE			0x04
#define CODEC_STATE_BALANCE_RINGING	0x07
#define CODEC_STATE_ACTIVE			0x23

                                               
/*************************************************************************/
/*                         COUNTRY RELATIVE DATA                         */
/*************************************************************************/

/* China-A law COEFFICIENTS for 50 OHM fuse : 600 ohm, A-D=0dB, D-A=-6dB */
/* Reg Addr: 0x8A */
unsigned char CHINA_ALAW_R_COFF[] = {0x1D,0x01,0x01,0x11,0x01,0x90,0x01,\
                            0x90,0x01,0x90,0x01,0x90,0x01,0x90};

/* Reg Addr: 0x9A */
unsigned char CHINA_ALAW_Z_IIR_COFF[] = {0x23,0xCA,0x26,0x9F,0x01};

/* Reg Addr: 0x50 */
unsigned char CHINA_ALAW_VOICE_GAIN[] = {0x10};

/* Reg Addr: 0x80 */
unsigned char CHINA_ALAW_TX_GAIN[] = {0x32,0x10};

/* Reg Addr: 0x82 */
unsigned char CHINA_ALAW_RX_GAIN[] = {0x22,0xE0};

/* Reg Addr: 0x86 */
unsigned char CHINA_ALAW_B_FIR_COFF[] = {0x3A,0x52,0xA2,0x3B,0x22,0xA4,0xC9, \
	                         0x7C,0x8F,0xA9,0xFA,0x8F,0x29,0xF0};

/* Reg Addr: 0x88 */
unsigned char CHINA_ALAW_X_COFF[] = {0xD8,0x70,0x7A,0x87,0x23,0x3F, \
	                        0x4A,0x97,0x5A,0xA7,0x5A,0xAF};

/* Reg Addr: 0x96 */
unsigned char CHINA_ALAW_B_IIR_COFF[] = {0x2E,0x01};

/* Reg Addr: 0x98 */
unsigned char CHINA_ALAW_Z_FIR_COFF[] = {0xCA,0xB9,0xA2,0x4C,0x2B, \
	                        0xA2,0xA3,0xA2,0xAE,0x2B};  
/* Reg Addr: 0xCA */
unsigned char CHINA_ALAW_DIMDENCE[] = {0xDA};

/* Reg Addr: 0x70 */
unsigned char CHINA_ALAW_OPCOND[] = {0x00};

/* Reg Addr: 0x60 */
unsigned char CHINA_ALAW_OPFUN[] = {0x7F};

/* China ring cadence Sin/25HZ/65Vpk/no Bais 2s on, 2s off */
/* Reg Addr: 0xD2 */
unsigned char CHINA_RING_SIGNAL_GEN[] = {0x00,0x00,0x00,0x00,0x44,0x35,
                                0xE3,0x00,0x00,0x00,0x00};
/* Reg Addr: 0xE0 */
unsigned char CHINA_RING_CADENCE[] = {0x01,0x90,0x01,0x90};


/*************************************************************************/
/*                           GENERAL DATA                                */
/*************************************************************************/
unsigned char REGULATOR_PARA_88221[] = {0x00,0x04,0x11};
unsigned char LOOP_SUPERVISION_88221[] = {0x1B,0x84,0xC6,0x05}; 
unsigned char DC_FEED_88221[] = {0x10,0x0b};


#ifdef __cplusplus
}
#endif

#endif /* _LEGERITY88211_H_ */
