/******************************************************************************
 *
 *   Copyright (C) 2002 Analog Devices India (P) Ltd. All rights are reserved.
 *
 *   The information and source code contained herein is the exclusive
 *   property of Analog Devices India (P) Ltd. and may not be disclosed,
 *   examined or reproduced in whole or in part without explicit written
 *   authorization from the compnay.
 *
 ******************************************************************************/

#ifndef __BELGIUM_CONST_H
#define __BELGIUM_CONST_H

//****************************************************************************
//		Plan de transmission du FXS BELGIUM : CPLX +2.5dBIn/-6.5dBout
//****************************************************************************
// SAGEM CFe : changement du plan de transmission du FXS : +2.5dBIn/-6.5dBout (au lieu de +3dBIn/-7dBout) 
// atténuation FXS totale en transmission de 2,5 dB avec les atténuations de 3.5 dB d'ATX, de 1.63 dB du mode linéaire et gain ADCG de 2.63dB
// atténuation FXS totale en réception de 6,5 dB avec les atténuations de 3.5 dB d'ARX, de 0.15 dB intrinsèque et 2.85dB de DACG 

// Transmit Path Digital To Analog Converter Gain/Attenuation = +2.63dB.
#define BE_XMIT_DIGITAL_GAIN_CPLX_IR14 		0x56A2
// Receive Path Digital To Analog Converter Gain/Attenuation = -2.85dB.
#define BE_RECV_DIGITAL_GAIN_CPLX_IR13 		0x2E19


//****************************************************************************
//		BELGIUM Ring specifications : Ring 50Hz 35V 1.5s On 3.5s Off
//****************************************************************************
// Ringing Active Timer = 125uS * 0x2EE0 = 1.5S
#define BE_RING_ON_1S5_LSB_DR48 			0xE0
#define BE_RING_ON_1S5_MSB_DR49 			0x2E
// Ringing Inactive Timer = 125uS * 0x6D60 = 3.5S
#define BE_RING_OFF_3S5_LSB_DR50 			0x60
#define BE_RING_OFF_3S5_MSB_DR51 			0x6D

// Ring Oscillator Frequency Coefficient for 50Hz.
#define BE_RING_OSC_FREQ_50HZ_IR7 			0x79BC
// Ring Oscillator Amplitude Register for 50Hz and 35V.
#define BE_RING_OSC_AMP_50HZ_IR8 			0x030C
// Ring Oscillator Initial Phase Register for 50Hz and 35V.
#define BE_RING_OSC_INITPHASE_50HZ_IR9 		0x0000
// Data sheet SI3216.
#define BE_RING_TRIP_THRESH_50HZ_IR16 		0x3600
#define BE_RING_TRIP_FILTER_50HZ_IR23 		0x06A8
// Ring Trip Detect Debounce Interval = 1.25mS * 0x05 = 6,25mS.
#define BE_RT_DEBOUNCE_50HZ_DR70 			0x05

//********************************************************
//		BELGIUM Country Tones specifications
//********************************************************

#define BE_FREQ_330Hz		        0x7EED
#define BE_F330Hz_AMP_M5dBm0	    0x00D1

#define	BE_FREQ_425Hz 				0x7E39
#define	BE_F425Hz_AMP_M6dBm0	 	0x00F0
#define	BE_F425Hz_AMP_M10dBm05 		0x008F
#define BE_F425Hz_AMP_M16dBm0	    0x004C

#define BE_FREQ_440Hz		        0x7E18
#define BE_F440Hz_AMP_M8dBm05	    0x00BA

#define BE_FREQ_740Hz              	0x3580
#define BE_F740Hz_AMP_M8dBm05      	0x0282

#define BE_FREQ_950Hz		        0x7732
#define BE_F950Hz_AMP_M8dBm05	    0x0196

#define BE_FREQ_1000Hz             	0x2D40
#define BE_F1000Hz_AMP_M8dBm05     	0x0379

#define	BE_FREQ_2130Hz 				0x55C6
#define	BE_F2130Hz_AMP_M9dBm0 		0x0386

#define	BE_FREQ_2750Hz 				0x3C57
#define	BE_F2750Hz_AMP_M9dBm0 		0x04C0


// Belgium OSC1	425Hz -6dBm0 Continuous
#define	BE_DIAL_IR0                 BE_FREQ_425Hz
#define	BE_DIAL_IR1                 BE_F425Hz_AMP_M6dBm0

// Belgium OSC1	425Hz -6dBm0 1s On 3s Off
#define	BE_RINGBACK_IR0             BE_FREQ_425Hz
#define	BE_RINGBACK_IR1             BE_F425Hz_AMP_M6dBm0
#define	BE_RINGBACK_DR36            0x40
#define	BE_RINGBACK_DR37            0x1F    // 0x1F40 = 1s
#define	BE_RINGBACK_DR38            0XC0
#define	BE_RINGBACK_DR39            0X5D    // 0x5DC0 = 3s

// Belgium OSC1	425Hz -6dBm0 0.5s On 0.5s Off
#define	BE_BUSY_IR0                 BE_FREQ_425Hz
#define	BE_BUSY_IR1                 BE_F425Hz_AMP_M6dBm0
#define	BE_BUSY_DR36                0xA0
#define	BE_BUSY_DR37                0x0F    // 0x0FA0 = 0.5s
#define	BE_BUSY_DR38                0xA0
#define	BE_BUSY_DR39                0x0F    // 0x0FA0 = 0.5s

// Belgium OSC1	425Hz -6dBm0 50ms On 50ms Off
#define	BE_ROUTING_TONE_C0_IR0 	BE_FREQ_425Hz
#define	BE_ROUTING_TONE_C0_IR1 	BE_F425Hz_AMP_M10dBm05
#define	BE_ROUTING_TONE_C0_DR36 	0x90
#define	BE_ROUTING_TONE_C0_DR37 	0x01
#define	BE_ROUTING_TONE_C0_DR38 	0x90
#define	BE_ROUTING_TONE_C0_DR39 	0x01

// Belgium OSC1	425Hz -16Bm0 0.100s On 0.100s Off
#define	BE_CALL_WAIT_C0_IR0         BE_FREQ_425Hz
#define	BE_CALL_WAIT_C0_IR1         BE_F425Hz_AMP_M16dBm0
#define	BE_CALL_WAIT_C0_DR36        0x20
#define	BE_CALL_WAIT_C0_DR37        0x03    //0x0320 = 100ms
#define	BE_CALL_WAIT_C0_DR38        0x20
#define	BE_CALL_WAIT_C0_DR39        0x03    //0x0320 = 100ms
// Belgium OSC1 0.100 On 3s Off
#define	BE_CALL_WAIT_C1_IR0         BE_FREQ_425Hz
#define	BE_CALL_WAIT_C1_IR1         BE_F425Hz_AMP_M16dBm0
#define	BE_CALL_WAIT_C1_DR36        0x20
#define	BE_CALL_WAIT_C1_DR37        0x03    //0x0320 = 100ms
#define	BE_CALL_WAIT_C1_DR38        0xC0
#define	BE_CALL_WAIT_C1_DR39        0x5D    //0x5DC0 = 3s

// Belgium OSC1	425Hz -6dBm0 Continuous
#define	BE_VOIP_DIAL_C0_IR0         BE_FREQ_425Hz
#define	BE_VOIP_DIAL_C0_IR1         BE_F425Hz_AMP_M6dBm0
#define	BE_VOIP_DIAL_C0_DR36        0x00
#define	BE_VOIP_DIAL_C0_DR37        0x00
#define	BE_VOIP_DIAL_C0_DR38        0x00
#define	BE_VOIP_DIAL_C0_DR39        0x00
// Belgium OSC1 not used
#define	BE_VOIP_DIAL_C1_IR0         0x00
#define	BE_VOIP_DIAL_C1_IR1         0x00
#define	BE_VOIP_DIAL_C1_DR36        0x00
#define	BE_VOIP_DIAL_C1_DR37        0x00
#define	BE_VOIP_DIAL_C1_DR38        0x00
#define	BE_VOIP_DIAL_C1_DR39        0x00
// Belgium OSC1	not used
#define	BE_VOIP_DIAL_C2_IR0         0x00
#define	BE_VOIP_DIAL_C2_IR1         0x00
#define	BE_VOIP_DIAL_C2_DR36        0x00
#define	BE_VOIP_DIAL_C2_DR37        0x00
#define	BE_VOIP_DIAL_C2_DR38        0x00
#define	BE_VOIP_DIAL_C2_DR39        0x00
// Belgium OSC1	not used
#define	BE_VOIP_DIAL_C3_IR0         0x00
#define	BE_VOIP_DIAL_C3_IR1         0x00
#define	BE_VOIP_DIAL_C3_DR36        0x00
#define	BE_VOIP_DIAL_C3_DR37        0x00
#define	BE_VOIP_DIAL_C3_DR38        0x00
#define	BE_VOIP_DIAL_C3_DR39        0x00

// Belgium OSC1	Msg Waiting Tone 1200 ms on 40 ms off
#define	BE_MSG_WAITING_C0_IR0       BE_DIAL_IR0
#define	BE_MSG_WAITING_C0_IR1       BE_DIAL_IR1
#define	BE_MSG_WAITING_C0_DR40      0xC0
#define	BE_MSG_WAITING_C0_DR41      0x26    // 0x26C0 = 1.24s
#define	BE_MSG_WAITING_C0_DR42      0x40
#define	BE_MSG_WAITING_C0_DR43      0x01    // 0x0140 = 0.04s
// Belgium OSC1	425Hz -6dBm0 / 40 ms on 40 ms off
#define	BE_MSG_WAITING_C1_IR0       BE_FREQ_425Hz
#define	BE_MSG_WAITING_C1_IR1       BE_F425Hz_AMP_M6dBm0
#define	BE_MSG_WAITING_C1_DR40      0x40
#define	BE_MSG_WAITING_C1_DR41      0x01    // 0x0140 = 0.04s
#define	BE_MSG_WAITING_C1_DR42      0x40
#define	BE_MSG_WAITING_C1_DR43      0x01    // 0x0140 = 0.04s

// OSC1	2130Hz -9dBm0 80ms On
#define	BE_ALERTING_IR0           BE_FREQ_2130Hz
#define	BE_ALERTING_IR1           BE_F2130Hz_AMP_M9dBm0
#define	BE_ALERTING_DR36          0x80
#define	BE_ALERTING_DR37          0x02
#define	BE_ALERTING_DR38          0x00
#define	BE_ALERTING_DR39          0x00
// OSC2	2750Hz -9dBm0 80ms On
#define	BE_ALERTING_IR3           BE_FREQ_2750Hz
#define	BE_ALERTING_IR4           BE_F2750Hz_AMP_M9dBm0
#define	BE_ALERTING_DR40          0x80
#define	BE_ALERTING_DR41          0x02
#define	BE_ALERTING_DR42          0x00
#define	BE_ALERTING_DR43          0x00


// OSC1	440Hz -8.5dBm0 550ms On 20ms Off
#define	BE_VOIP_MWI_C0_IR0         BE_FREQ_440Hz
#define	BE_VOIP_MWI_C0_IR1         BE_F440Hz_AMP_M8dBm05
#define	BE_VOIP_MWI_C0_DR36        0x30
#define	BE_VOIP_MWI_C0_DR37        0x11
#define	BE_VOIP_MWI_C0_DR38        0xA0
#define	BE_VOIP_MWI_C0_DR39        0x00
// OSC1	1000Hz -8.5dBm0 250ms On 20ms Off
#define	BE_VOIP_MWI_C1_IR0         BE_FREQ_1000Hz
#define	BE_VOIP_MWI_C1_IR1         BE_F1000Hz_AMP_M8dBm05
#define	BE_VOIP_MWI_C1_DR36        0xD0
#define	BE_VOIP_MWI_C1_DR37        0x07
#define	BE_VOIP_MWI_C1_DR38        0xA0
#define	BE_VOIP_MWI_C1_DR39        0x00
// OSC1	740Hz -8.5dBm0 250ms On 20ms Off
#define	BE_VOIP_MWI_C2_IR0         BE_FREQ_740Hz
#define	BE_VOIP_MWI_C2_IR1         BE_F740Hz_AMP_M8dBm05
#define	BE_VOIP_MWI_C2_DR36        0xD0
#define	BE_VOIP_MWI_C2_DR37        0x07
#define	BE_VOIP_MWI_C2_DR38        0xA0
#define	BE_VOIP_MWI_C2_DR39        0x00
// OSC1	440Hz -8.5dBm0 1650ms On 20ms Off
#define	BE_VOIP_MWI_C3_IR0         BE_FREQ_440Hz
#define	BE_VOIP_MWI_C3_IR1         BE_F440Hz_AMP_M8dBm05
#define	BE_VOIP_MWI_C3_DR36        0x90
#define	BE_VOIP_MWI_C3_DR37        0x33
#define	BE_VOIP_MWI_C3_DR38        0xA0
#define	BE_VOIP_MWI_C3_DR39        0x00
// OSC2	330Hz -5dBm0 50ms On 150ms Off
#define	BE_VOIP_MWI_C0_IR3       BE_FREQ_330Hz
#define	BE_VOIP_MWI_C0_IR4       BE_F330Hz_AMP_M5dBm0
#define	BE_VOIP_MWI_C0_DR40      0x90
#define	BE_VOIP_MWI_C0_DR41      0x01
#define	BE_VOIP_MWI_C0_DR42      0xB0
#define	BE_VOIP_MWI_C0_DR43      0x04
// OSC2	330Hz -5dBm0 50ms On 860ms Off
#define	BE_VOIP_MWI_C1_IR3       BE_FREQ_330Hz
#define	BE_VOIP_MWI_C1_IR4       BE_F330Hz_AMP_M5dBm0
#define	BE_VOIP_MWI_C1_DR40      0x90
#define	BE_VOIP_MWI_C1_DR41      0x01
#define	BE_VOIP_MWI_C1_DR42      0xE0
#define	BE_VOIP_MWI_C1_DR43      0x1A
// OSC2	330Hz -5dBm0 50ms On 150ms Off
#define	BE_VOIP_MWI_C2_IR3       BE_FREQ_330Hz
#define	BE_VOIP_MWI_C2_IR4       BE_F330Hz_AMP_M5dBm0
#define	BE_VOIP_MWI_C2_DR40      0x90
#define	BE_VOIP_MWI_C2_DR41      0x01
#define	BE_VOIP_MWI_C2_DR42      0xB0
#define	BE_VOIP_MWI_C2_DR43      0x04
// OSC2	330Hz -5dBm0 50ms On 300ms Off
#define	BE_VOIP_MWI_C3_IR3       BE_FREQ_330Hz
#define	BE_VOIP_MWI_C3_IR4       BE_F330Hz_AMP_M5dBm0
#define	BE_VOIP_MWI_C3_DR40      0x90
#define	BE_VOIP_MWI_C3_DR41      0x01
#define	BE_VOIP_MWI_C3_DR42      0x60
#define	BE_VOIP_MWI_C3_DR43      0x09
// OSC2	330Hz -5dBm0 50ms On 150ms Off
#define	BE_VOIP_MWI_C4_IR3       BE_FREQ_330Hz
#define	BE_VOIP_MWI_C4_IR4       BE_F330Hz_AMP_M5dBm0
#define	BE_VOIP_MWI_C4_DR40      0x90
#define	BE_VOIP_MWI_C4_DR41      0x01
#define	BE_VOIP_MWI_C4_DR42      0xB0
#define	BE_VOIP_MWI_C4_DR43      0x04
// OSC2	330Hz -5dBm0 50ms On 300ms Off
#define	BE_VOIP_MWI_C5_IR3       BE_FREQ_330Hz
#define	BE_VOIP_MWI_C5_IR4       BE_F330Hz_AMP_M5dBm0
#define	BE_VOIP_MWI_C5_DR40      0x90
#define	BE_VOIP_MWI_C5_DR41      0x01
#define	BE_VOIP_MWI_C5_DR42      0x60
#define	BE_VOIP_MWI_C5_DR43      0x09
// OSC2	330Hz -5dBm0 50ms On 150ms Off
#define	BE_VOIP_MWI_C6_IR3       BE_FREQ_330Hz
#define	BE_VOIP_MWI_C6_IR4       BE_F330Hz_AMP_M5dBm0
#define	BE_VOIP_MWI_C6_DR40      0x90
#define	BE_VOIP_MWI_C6_DR41      0x01
#define	BE_VOIP_MWI_C6_DR42      0xB0
#define	BE_VOIP_MWI_C6_DR43      0x04
// OSC2	330Hz -5dBm0 50ms On 860ms Off
#define	BE_VOIP_MWI_C7_IR3       BE_FREQ_330Hz
#define	BE_VOIP_MWI_C7_IR4       BE_F330Hz_AMP_M5dBm0
#define	BE_VOIP_MWI_C7_DR40      0x90
#define	BE_VOIP_MWI_C7_DR41      0x01
#define	BE_VOIP_MWI_C7_DR42      0xE0
#define	BE_VOIP_MWI_C7_DR43      0x1A



//********************************************************
//		BELGIUM Country DTMF Tones specifications
//********************************************************
#define	BE_DTMF_on_80ms_DR36          0x80
#define	BE_DTMF_on_80ms_DR37          0x02
#define	BE_DTMF_on_80ms_DR40          0x80
#define	BE_DTMF_on_80ms_DR41          0x02

#define BE_FREQ_697Hz		        0x7B3C
#define BE_F697Hz_AMP_M11dBm0	    0x00DD

#define BE_FREQ_770Hz		        0x7A31
#define BE_F770Hz_AMP_M11dBm0	    0x00F5

#define BE_FREQ_852Hz		        0x78E6
#define BE_F852Hz_AMP_M11dBm0	    0x0110

#define BE_FREQ_941Hz		        0x775C
#define BE_F941Hz_AMP_M11dBm0	    0x012D

#define BE_FREQ_1209Hz		        0x71D7
#define BE_F1209Hz_AMP_M9dBm0	    0x01EA

#define BE_FREQ_1336Hz		        0x6EC8
#define BE_F1336Hz_AMP_M9dBm0	    0x0220

#define BE_FREQ_1477Hz		        0x6B10
#define BE_F1477Hz_AMP_M9dBm0	    0x025D

#define BE_FREQ_1633Hz		        0x6692
#define BE_F1633Hz_AMP_M9dBm0	    0x02A1

typedef enum
{
	BE_NULL_TONE		        	= 0,
	BE_DIAL_TONE		        	= 1,
	BE_ROUTING_TONE		 			= 2,
	BE_RING_BACK_TONE		    	= 3,
	BE_BUSY_TONE		        	= 4,
	BE_MSG_WAITING_TONE		    	= 5,
	BE_CALL_WAITING_TONE	    	= 6,
	BE_ALERTING_TONE		    	= 7,
	BE_VOIP_DIAL_TONE		    	= 8,
	BE_VOIP_MWI_TONE	        	= 9,
	BE_DTMF_DIG_0_TONE_LOCAL		= 10,
	BE_DTMF_DIG_1_TONE_LOCAL		= 11,
	BE_DTMF_DIG_2_TONE_LOCAL		= 12,
	BE_DTMF_DIG_3_TONE_LOCAL        = 13,
	BE_DTMF_DIG_4_TONE_LOCAL        = 14,
	BE_DTMF_DIG_5_TONE_LOCAL        = 15,
	BE_DTMF_DIG_6_TONE_LOCAL        = 16,
	BE_DTMF_DIG_7_TONE_LOCAL        = 17,
	BE_DTMF_DIG_8_TONE_LOCAL        = 18,
	BE_DTMF_DIG_9_TONE_LOCAL        = 19,
	BE_DTMF_DIG_A_TONE_LOCAL        = 20,
	BE_DTMF_DIG_B_TONE_LOCAL        = 21,
	BE_DTMF_DIG_C_TONE_LOCAL        = 22,
	BE_DTMF_DIG_D_TONE_LOCAL        = 23,
	BE_DTMF_DIG_STAR_TONE_LOCAL     = 24,
	BE_DTMF_DIG_HASH_TONE_LOCAL     = 25,
	BE_TONE_END			        	= 26
} enumBelgiumTone;

#define BE_MAX_NUM_CADENCE        8

extern const ringStruct belgiumRingTable;
extern const toneCadenceTables belgiumToneCadenceTables[BE_TONE_END][BE_MAX_NUM_CADENCE];


#endif
