/* ============================================================================
 *
 * Copyright (C) 2001 Analog Devices Inc. All rights reserved
 *
 * The information and source code contained herein is the exclusive property
 * of Analog Devices and may not be disclosed, examined or reproduced in whole
 * or in part without the explicit written authorization from Analog Devices.
 *
 * ============================================================================
 */

#include<oswrapper.h>

#if CALLER_IDTx
 extern CidTxType_  SMM_CidTx[MAX_VOICE_CHANNELS];
 extern dtmfCidType_ dtmfCidGen[MAX_VOICE_CHANNELS];
 extern void dtmfCidGeneration(Word16 ucPortNo);
#endif

unsigned short prev_status = 0x0;
SLICInfo_t gaSlicInfo[MAX_SLIC_CHANNELS];
INT16 testVariable;
extern unsigned short dspTimeSlot0,dspTimeSlot1;

#ifdef FIRMWARE_CONF_COUNTRY_NL
unsigned char voltmonitor;
#endif

// Sagem 20080226
unsigned short offhookDebounceValue = 15;   // OFF DEBOUNCE is 150 msecs 
unsigned short onhookDebounceValue = 80;   // ON HOOK DEBOUNCE is 800 msecs

#define MAX_HOST_CMD_Q (4*MAX_SLIC_CHANNELS)
SLICHostCmd_t slicHostCmds[MAX_HOST_CMD_Q];
UCHAR slicHostCmdWriteIdx = 0, slicHostCmdReadIdx = 0;

/* FLASH is between 50 msecs to 350 msecs for FRANCE
   0:USA, 1: FRANCE
*/
// Sagem 20080226
unsigned short flashhookDebounceMinValue[NUM_COUNTRY] = {5,	// USA  
                                                         5,	// FRANCE 
														 7,	// NETHERLANDS
                                                         5,	// POLAND
                                                         5,	// SPAIN
                                                         5	// BELGIUM
                                                         };   

unsigned short flashhookDebounceMaxValue[NUM_COUNTRY] = {60,  // USA  
                                                         35,  // FRANCE  
                                                         20,  // NETHERLANDS 
                                                         52,  // POLAND 
                                                         35,  // SPAIN  
                                                         35   // BELGIUM  
                                                         }; 

slicHandlers_t slicHandlers;

extern void slicModeSetting_88111(int codecNo,int chNo,int mode);
extern void slicModeSetting_88221(int codecNo,int chNo,int mode);
extern void slicModeSetting_78d11(int codecNo,int chNo,int mode);
extern void slicModeSetting_silabs(int codecNo,int chNo,int mode);

extern unsigned short codecReadStatus_88111(int codecNo);
extern unsigned short codecReadStatus_88221(int codecNo);
extern unsigned short codecReadStatus_78d11(int codecNo);
extern unsigned short codecReadStatus_silabs(int codecNo);

extern void writeDirectReg(unsigned char codecNo, unsigned char address, unsigned char data);
extern unsigned char readDirectReg(unsigned char codecNo, unsigned char address);

/*********************************************************************************
 *  Function Name        : slicGetCodecNo                                        *
 *  Description          : Get the codec Number.                                 *
 *  Input                : Port Number.                                          *
 *  Output               : None.                                                 *
 *  Return Value         : Codec Number.                                         *
 ********************************************************************************/
static UINT16 slicGetCodecNo(unsigned short portNum)
{
  unsigned short codecNo = 0;
  switch (portNum)
  {
    case 0: /* FXS Tphone1: DSP0(G711+G729)=>TimeSlot=0 */
      codecNo = 0;
    break;
    case 1: /* FXS Tphone2: DSP0(G711+G729)=>TimeSlot=4 */
      if(slicHandlers.codecType == CODEC_SILABS)
        codecNo = 1;
      else if(slicHandlers.codecType == CODEC_LE78d11)
        codecNo = 0;
      else if(slicHandlers.codecType == CODEC_LE88111)
        codecNo = 1;
      else if(slicHandlers.codecType == CODEC_LE88221)
        codecNo = 0;
     break;
     case 2:
#ifdef SILABS_1FXS_1FXO
      codecNo = 2;
#endif
     break;
   }
 return codecNo;
}

/*********************************************************************************
 *  Function Name        : slicInit                                              *
 *  Description          : initialize the slics.                                 *
 *  Input                : Slic Port Number.                                     *
 *  Output               : None.                                                 *
 *  Return Value         : None.                                                 *
 ********************************************************************************/
void slicInit(unsigned short portNum)
{
#if SLIC_MODULES

#ifdef FIRMWARE_CONF_COUNTRY_NL
voltmonitor = LINE_MONITORING_YES;
#endif

  unsigned char codecNo =  slicGetCodecNo(portNum);
  osLockInterrupts();
  switch(codecNo)
  {
    case 0:
    case 1:
      if(slicHandlers.codecType == CODEC_SILABS)
        si3216_initialize(codecNo);
      else if(slicHandlers.codecType == CODEC_LE78d11)
        legerity78d11_init(codecNo);
      else if(slicHandlers.codecType == CODEC_LE88111)
        legerity88111_init(codecNo); 
      else if(slicHandlers.codecType == CODEC_LE88221)
        legerity88221_init(codecNo); 
      break;
    case 2:
#ifdef SILABS_1FXS_1FXO
      si3050_initialize();
#endif
      break;
   }
  osUnlockInterrupts();
#ifndef JUNGO /* { */
  /* SAGEM -> [ Hard non V2 (if reset SLIC = reset switch) ] */
  if( *(volatile UINT32*)0xb900003c != 0x6836) {
    /* re-init switch on RG1, for which phone and switch resets are common. */
    *(volatile unsigned int *)0xb9110058 = 0x107f;
    *(volatile unsigned int *)0xb911005c = 0x0f04;
    mdelay(100);
    *(volatile unsigned int *)0xb9110058 = 0x107f;
    *(volatile unsigned int *)0xb911005c = 0x0f24;
    mdelay(100);
    *(volatile unsigned int *)0xb9110058 = 0x107f;
    *(volatile unsigned int *)0xb911005c = 0x0f44;
    mdelay(100);
  }
#endif /* JUNGO } */
#endif
}

/*********************************************************************************
 *  Function Name        : slicIdlePol                                           *
 *  Description          : Change current polarity .                             *
 *  Input                : Channel Number.                                       *
 *  Output               : None.                                                 *
 *  Return Value         : None.                                                 *
 ********************************************************************************/
void slicIdlePol(unsigned short portNum)
{
//printk("\n slicIdlePol\n");

#if SLIC_MODULES
  gaSlicInfo[portNum].slicPolarity = IDLE_POL;
#endif
}

/*********************************************************************************
 *  Function Name        : slicActivate                                          *
 *  Description          : Activates the slics .                                 *
 *  Input                : Channel Number.                                       *
 *  Output               : None.                                                 *
 *  Return Value         : None.                                                 *
 ********************************************************************************/
void slicActivate(unsigned short portNum)
{
#if SLIC_MODULES
  UINT16 codecNum = slicGetCodecNo(portNum);
  osLockInterrupts();
  if (gaSlicInfo[portNum].slicPolarity == REVERSE_POL)
	slicHandlers.slicCmdHandler(codecNum, portNum, SLIC_REVERSE_ACTIVE);
  else
  	slicHandlers.slicCmdHandler(codecNum, portNum, SLIC_ACTIVE);
  osUnlockInterrupts();
#endif
}
/*********************************************************************************
 *  Function Name        : slicRing                                              *
 *  Description          : Provide Ring to Slic.                                 *
 *  Input                : Channel Number.                                       *
 *  Output               : None.                                                 *
 *  Return Value         : None.                                                 *
 ********************************************************************************/
void slicRing(unsigned short portNum)
{
#if SLIC_MODULES
  UINT16 codecNum = slicGetCodecNo(portNum);
  DEBUG(PRINT("slicRing:  Port: %d\n",portNum), DEBUG_SLIC);
  osLockInterrupts();
  slicHandlers.slicCmdHandler(codecNum, portNum, SLIC_RING);
  osUnlockInterrupts();
#endif
}
/*********************************************************************************
 *  Function Name        : slicReverseActivate                                   *
 *  Description          : Deactivate to Slic.                                   *
 *  Input                : Channel Number.                                       *
 *  Output               : None.                                                 *
 *  Return Value         : None.                                                 *
 ********************************************************************************/
void slicReverseActivate(unsigned short portNum)
{
#if SLIC_MODULES
  UINT16 codecNum = slicGetCodecNo(portNum);
  gaSlicInfo[portNum].slicPolarity = REVERSE_POL;
  osLockInterrupts();
  slicHandlers.slicCmdHandler(codecNum, portNum, SLIC_REVERSE_ACTIVE);
  osUnlockInterrupts();
#endif
}

/*********************************************************************************
 *  Function Name        : slicGainCtrl                                          *
 *  Description          : change gain for receive path.                         *
 *  Input                : Channel Number.                                       *
 *  Input                : gain value.                                           *
 *  Output               : None.                                                 *
 *  Return Value         : None.                                                 *
 ********************************************************************************/
void slicGainCtrl(unsigned char portNum, unsigned int level)
{
#if SLIC_MODULES
  osLockInterrupts();
  si3216_gain_ctrl(portNum,level);
  osUnlockInterrupts();
#endif
}

/*********************************************************************************
 *  Function Name        : slicRingOff                                           *
 *  Description          : Stop Ring to Slic.                                    *
 *  Input                : Channel Number.                                       *
 *  Output               : None.                                                 *
 *  Return Value         : None.                                                 *
 ********************************************************************************/
void slicRingOff(unsigned short portNum)
{
#if SLIC_MODULES
  INT16 codecNum = slicGetCodecNo(portNum);
  DEBUG(PRINT("slicRingOff:  Port: %d\n",portNum), DEBUG_SLIC);
  osLockInterrupts();
  if (gaSlicInfo[portNum].slicPolarity == REVERSE_POL)
	slicHandlers.slicCmdHandler(codecNum, portNum, SLIC_REVERSE_RINGOFF);
  else
  slicHandlers.slicCmdHandler(codecNum, portNum, SLIC_RINGOFF);
  osUnlockInterrupts();
#endif
}
/*********************************************************************************
 *  Function Name        : slicStandby                                           *
 *  Description          : Standby Slic.                                         *
 *  Input                : Channel Number.                                       *
 *  Output               : None.                                                 *
 *  Return Value         : None.                                                 *
 ********************************************************************************/
void slicStandby(unsigned short portNum)
{
#if SLIC_MODULES
  UINT16 codecNum = slicGetCodecNo(portNum);
  slicHandlers.slicCmdHandler(codecNum, portNum, SLIC_ACTIVE);
#endif
}
/*********************************************************************************
 *  Function Name        : slicLoopBack.                                         *
 *  Description          : Slic Loopback.                                        *
 *  Input                : Channel Number.                                       *
 *  Output               : None.                                                 *
 *  Return Value         : None.                                                 *
 ********************************************************************************/
void slicLoopBack(unsigned short portNum)
{
#if SLIC_MODULES
  UINT16 codecNum = slicGetCodecNo(portNum);
  slicHandlers.slicCmdHandler(codecNum, portNum, SLIC_LOOPBACK);
#endif
}
/*********************************************************************************
 *  Function Name        : slicDeviceInitialize.                                 *
 *  Description          : Function to probe the slic Card and initialize.       * 
 *  Input                : None.                                                 *
 *  Output               : None.                                                 *
 *  Return Value         : None.                                                 *
 ********************************************************************************/
void slicDeviceInitialize(void)
{
#if SLIC_MODULES
  UINT32 temp;
  UINT16 temp1;
  UCHAR  product_code[2];
  UINT16 slicGPIOReset;

  if( *(volatile UINT32*)0xb900003c == 0x6836)
  {
#if TARGET_ALASKA
    slicGPIOReset = 11;
#else
  #if ( defined(CONFIG_SAGEM_BOXER) && ! defined(CONFIG_SAGEM_GPIO_ULT_FOR_BOXER_FW) ) || ( defined(CONFIG_SAGEM_LBV2ULT) && defined(CONFIG_SAGEM_GPIO_BOXER_FOR_ULT_FW) )
	printk("SLIC board BOXER_P1\n");
    slicGPIOReset = 3;
  #else
	printk("SLIC board FAST_3202\n");
    slicGPIOReset = 10;
  #endif
#endif
  }
/* SAGEM -> [Hard V1] { */
/* LUA -> [ Bug 1661 ] { */
  else if ( (*(volatile UINT32*)0xb900003c == 0x6834) || (*(volatile UINT32*)0xb900003c == 0x6833) )
    slicGPIOReset = 15;
/* } */
/* } */
  else
    slicGPIOReset = 6;

  if(slicGPIOReset > 15)
  {
    /* H/W Reset to the codec */
    temp1 = *(volatile UINT16*)GPIO_DIR_EXT_REG;
    temp1 |= (1 << (slicGPIOReset - 16));
    *(volatile UINT16*)GPIO_DIR_EXT_REG = temp1;
  }
  else 
  {
    temp1 = *(volatile UINT16*)GPIO_DIR_REG;
    temp1 |= (1 << slicGPIOReset);
    *(volatile UINT16*)GPIO_DIR_REG = temp1;
  }


  /* Program SPI to drive GPIO Muxed Pins */
  temp = *(volatile UINT32*)MISC_CTRL_REG;
  temp |= (1 << SPI_CS1_EN);
//  temp = (temp ) | (1 << SPI_CS2_EN);
#if FXO_MODULE
/* SAGEM -> [Hard V2] { */
  if( *(volatile UINT32*)0xb900003c == 0x6836)
/* } */
    temp = (temp ) | (1 << SPI_CS2_EN);
#endif
  *(volatile UINT32*)MISC_CTRL_REG = temp;

  /* Set Bit No 25 in Interrupt Register */
  temp = *(volatile UINT32*)CPE_PER_INT;
  temp = temp | 0x08000000;
  *(volatile UINT32*)CPE_PER_INT = temp;

  /* Initialize SPI Module */
  DEBUG(PRINT("Initializing SPI Module \n"), DEBUG_INFO);
  spiInit(SPI_BAUD);

  /* Probe the SLIC Card */
  /* First of all look for the legerity devices */
  /* 88 series */
  /* Read 0x73 Register */
  read2codecArray(0,0x73,product_code,2);
  printk("Product Code: %x %x\n",product_code[0],product_code[1]);
  if(product_code[0] == 0xE1)
  {
    /* Legerity78d11 Card */
    DEBUG(PRINT("Slic Card Detected : => Legerity78d11 \n"), DEBUG_INFO);
    slicHandlers.codecType = CODEC_LE78d11;
    slicHandlers.slicCmdHandler = slicModeSetting_78d11;
    slicHandlers.codecStatus = codecReadStatus_78d11;
    slicHandlers.codecCnt = 1;
  }  
  else
  {
    if(product_code[0] == 0x02 && product_code[1] == 0x0A)
    {
      /* Legerity 88111 */
      DEBUG(PRINT("Slic Card Detected : => Legerity88111 \n"), DEBUG_INFO);
      slicHandlers.codecCnt = 2;
      slicHandlers.codecType = CODEC_LE88111;
      slicHandlers.slicCmdHandler = slicModeSetting_88111;
      slicHandlers.codecStatus = codecReadStatus_88111;
      
    }
    else if(product_code[0] == 0x01 && product_code[1] == 0xb2)
    {
      /* Legerity 88221 */
      slicHandlers.codecCnt = 1;
      DEBUG(PRINT("Slic Card Detected : => Legerity88221 \n"), DEBUG_INFO);
      slicHandlers.codecType = CODEC_LE88221;
      slicHandlers.slicCmdHandler = slicModeSetting_88221;
      slicHandlers.codecStatus = codecReadStatus_88221;
    }
    else 
    {
      /* Assume it as Silabs */
      DEBUG(PRINT("Assuming SLIC card as SILABS \n"), DEBUG_INFO);
      slicHandlers.codecCnt = 2;
      slicHandlers.codecType = CODEC_SILABS;
      slicHandlers.slicCmdHandler = slicModeSetting_silabs;
      slicHandlers.codecStatus = codecReadStatus_silabs;
    }
  }
#endif
  mdelay(300);
  /* initialize slic codecs */
  DEBUG(PRINT("Initializing SLICs \n"), DEBUG_INFO);
  for(temp1 = 0; temp1 < slicHandlers.codecCnt; temp1++)
  {
    slicInit(temp1);
    /* Give 100 ms delay */
    mdelay(300);
  }
#if FXO_MODULE
  if(slicHandlers.codecType == CODEC_SILABS)
    si3050_initialize();
#endif

  DEBUG(PRINT("Activate SLICs => %d\n",temp1), DEBUG_INFO);
  for(temp1 = 0; temp1 < MAX_SLIC_CHANNELS; temp1++)
  {
    /* Activate SLIC */
    slicActivate(temp1);
    mdelay(300);
    slicResetRingDSPStateInfo(temp1);
    gaSlicInfo[temp1].slicOffTheHook = SLIC_ONHOOK;
    gaSlicInfo[temp1].ucSlicState = SLIC_NORING_ONHOOK;
    gaSlicInfo[temp1].bFirstCadence  = FALSE;
    gaSlicInfo[temp1].slicPolarity = IDLE_POL;
  }
  /* Just to know the codec status */
  slicIsr();
}

/* Clear SLIC Stuff */
/**********************************************************************************
 *  Function Name  : slicResetRingDSPInfo.                                        *
 *  Description    : Function to be reset DSPIPC Slic State Info.                 *
 *  Input          : Port Number.                                                 *
 *  Output         : None.                                                        *
 *  Return Value   : None.                                                        *
 *********************************************************************************/
void slicResetRingDSPStateInfo(UCHAR ucPort)
{
  SLICInfo_t *pSLIC = &gaSlicInfo[ucPort];
  pSLIC->uiT4SCntr = 0;
  pSLIC->uiT2SCntr = 0;
  pSLIC->ucNumRings = 0;
  pSLIC->bFirstCadence = FALSE;
}

/* SLIC Tasklet function */
void slicTasklet(unsigned long param)
{
  UINT16 uiCountry;
  int index;
  SLICInfo_t *pSLIC = NULL;
  SLICHostCmd_t *pCmd = NULL;

  confGetCountryOpts(&uiCountry);

  if(slicHostCmdWriteIdx != slicHostCmdReadIdx)
  {
    slicHostCmdReadIdx = (slicHostCmdReadIdx + 1) % MAX_HOST_CMD_Q;
    pCmd = &slicHostCmds[slicHostCmdReadIdx];
    switch(pCmd->cmd)
    {
      case DSP_SLIC_RING:
// LCe OpenRG Hollande
#ifdef FIRMWARE_CONF_COUNTRY_NL
        gaSlicInfo[pCmd->port].bFirstCadence = FALSE;
        if (dtmfCidGen[pCmd->port].ucDigState == 0)
            slicRing(pCmd->port);
#else 
        gaSlicInfo[pCmd->port].bFirstCadence = TRUE;
        slicRing(pCmd->port);
#endif
// LCe OpenRG Hollande

        gaSlicInfo[pCmd->port].ucSlicState = SLIC_RINGING_ON;
        break;
      case DSP_SLIC_RING_OFF:
// LCe OpenRG Hollande
#ifdef FIRMWARE_CONF_COUNTRY_NL
        if (gaSlicInfo[pCmd->port].slicOffTheHook == SLIC_ONHOOK)
		slicIdlePol(pCmd->port);
#endif
// LCe OpenRG Hollande
        slicRingOff(pCmd->port);
        slicResetRingDSPStateInfo(pCmd->port);
        gaSlicInfo[pCmd->port].ucSlicState = SLIC_NORING_ONHOOK;
        break;
      case SLIC_RING_USER_CONTROLLED:
        slicRing(pCmd->port);
        break;
      case SLIC_RING_OFF_USER_CONTROLLED:
        slicRingOff(pCmd->port);
        break;
    }
  }
  for(index = 0; index < MAX_SLIC_CHANNELS; index++)
  {
    pSLIC = &gaSlicInfo[index];

#ifdef FIRMWARE_CONF_COUNTRY_NL
    if (dtmfCidGen[index].ucDigState == 1)
	pSLIC->bFirstCadence = FALSE;

    if ((pSLIC->ucSlicState == SLIC_RINGING_ON) && (dtmfCidGen[index].ucDigState == 0))
#else
    if(pSLIC->ucSlicState == SLIC_RINGING_ON)
#endif
    {
      UINT16 slic_ring_on_time;
      pSLIC->uiT2SCntr++;
      if(pSLIC->bFirstCadence)
      {
        slic_ring_on_time = confFirstCadenceOn(uiCountry);
      }
      else
        slic_ring_on_time = confCadenceOn(uiCountry);
      if(pSLIC->uiT2SCntr >= slic_ring_on_time)
      {
        pSLIC->ucSlicState = SLIC_RINGING_OFF;
#if SLIC_MODULES
        slicRingOff(index);
#endif
        pSLIC->uiT4SCntr = 0;
      }
    }
    else if(pSLIC->ucSlicState == SLIC_RINGING_OFF)
    {
      UINT16 slic_ring_off_time;
      pSLIC->uiT4SCntr++;
      if(pSLIC->bFirstCadence)
      {
        slic_ring_off_time = confFirstCadenceOff(uiCountry);
      }
      else
        slic_ring_off_time = confCadenceOff(uiCountry);
#if CALLER_IDTx
      if(pSLIC->bFirstCadence && (SMM_CidTx[index].bCaller == 1) && (pSLIC->uiT4SCntr >= 60))
      {
        //printk("slic_ring_off_time:%d\n",pSLIC->uiT4SCntr);
        SMM_CidTx[index].CidTxState = CID_TX_STATE2;
        SMM_CidTx[index].TimerCntr = 0;
        SMM_CidTx[index].bCaller = 0;
      }
#endif
      if(pSLIC->uiT4SCntr >= slic_ring_off_time )
      {
        if(pSLIC->ucNumRings == -1) /* Infinite Rings */
        {
          pSLIC->bFirstCadence = FALSE;
          pSLIC->ucSlicState = SLIC_RINGING_ON;
#if SLIC_MODULES
          slicRing(index);
#endif
          pSLIC->uiT2SCntr = 0;
        }
        else if(pSLIC->ucNumRings)
        {
          pSLIC->bFirstCadence = FALSE;
          pSLIC->ucNumRings--;
          pSLIC->ucSlicState = SLIC_RINGING_ON;
#if SLIC_MODULES
          slicRing(index);
#endif
          pSLIC->uiT2SCntr = 0;
        }
        else
        {
          dspipcReportIndication(index,DSPIPC_RING_EXPIRED);
          slicResetRingDSPStateInfo(index);
          pSLIC->ucSlicState = SLIC_NORING_ONHOOK;
        }
      }
    }
#ifdef FIRMWARE_CONF_COUNTRY_NL
    // DTMF  Caller ID generation
    if (dtmfCidGen[index].ucDigState != 0)
	{
	dtmfCidGen[index].ucIpcDigCntr++;
	dtmfCidGeneration(index);
	}

    if (dtmfCidGen[index].ucDigState == 4)	// first ring
	{
	dtmfCidGen[index].ucDigState = 0;
	slicRing(index);
	}
#endif
  }
  slicIsr();
}
/**********************************************************************************
 *  Function Name  : slicVoiceRing.                                               *
 *  Description    : Function to be provide Ring Tone to the port.                *
 *  Input          : Port Number.                                                 *
 *  Output         : None.                                                        *
 *  Return Value   : None.                                                        *
 *********************************************************************************/
void slicVoiceRing(UCHAR ucPort, int ringCnt)
{
#if SLIC_MODULES
  int index = (slicHostCmdWriteIdx + 1) % MAX_HOST_CMD_Q;
  SLICHostCmd_t *pCmd = &slicHostCmds[index];
  pCmd->cmd = DSP_SLIC_RING;
  pCmd->port = ucPort;
  gaSlicInfo[ucPort].uiT4SCntr = 0;
  gaSlicInfo[ucPort].uiT2SCntr = 0;
  if(ringCnt == 0)
    ringCnt = 11; /* Default value of the ring count is 12 */
  if(ringCnt != -1)
    gaSlicInfo[ucPort].ucNumRings = ringCnt - 1;
  else 
    gaSlicInfo[ucPort].ucNumRings = ringCnt;
  slicHostCmdWriteIdx = index;
#endif
}

/**********************************************************************************
 *  Function Name  : slicUserControlRing.                                         *
 *  Description    : Function to be provide SLIC ring interface controlled by user*
 *  Input          : Port Number.                                                 *
 *  Output         : None.                                                        *
 *  Return Value   : None.                                                        *
 *********************************************************************************/
void slicUserControlRing(UCHAR ucPort)
{
#if SLIC_MODULES
  int index = (slicHostCmdWriteIdx + 1) % MAX_HOST_CMD_Q;
  SLICHostCmd_t *pCmd = &slicHostCmds[index];
  pCmd->cmd = SLIC_RING_USER_CONTROLLED;
  pCmd->port = ucPort;
  slicHostCmdWriteIdx = index;
#endif
}

/**********************************************************************************
 *  Function Name  : slicUserControlRingOff.                                      *
 *  Description    : Function to be provide SLIC ring off interface controlled by *
 *                   user                                                         *
 *  Input          : Port Number.                                                 *
 *  Output         : None.                                                        *
 *  Return Value   : None.                                                        *
 *********************************************************************************/
void slicUserControlRingOff(UCHAR ucPort)
{
#if SLIC_MODULES
  int index = (slicHostCmdWriteIdx + 1) % MAX_HOST_CMD_Q;
  SLICHostCmd_t *pCmd = &slicHostCmds[index];
  pCmd->cmd = SLIC_RING_OFF_USER_CONTROLLED;
  pCmd->port = ucPort;
  slicHostCmdWriteIdx = index;
#endif
}



/**********************************************************************************
 *  Function Name  : slicGetHookStateInfo.                                        *
 *  Description    : Function to get FXS Ring and Hook State Information.         *
 *  Input          : SlicStateInfo_t                                              *
 *  Output         : None.                                                        *
 *  Return Value   : None.                                                        *
 *********************************************************************************/
void slicGetHookStateInfo(SlicStateInfo_t *pHookInfo)
{
  pHookInfo->slicState = gaSlicInfo[pHookInfo->ucPortNum].ucSlicState;
  pHookInfo->hookState = gaSlicInfo[pHookInfo->ucPortNum].slicOffTheHook;
}


/**********************************************************************************
 *  Function Name  : slicVoiceRingOff.                                            *
 *  Description    : Function to be stop Ring Tone to the port   .                *
 *  Input          : Port Number.                                                 *
 *  Output         : None.                                                        *
 *  Return Value   : None.                                                        *
 *********************************************************************************/
void slicVoiceRingOff(UCHAR ucPort, BOOL bActivate)
{
#if SLIC_MODULES
  int index = (slicHostCmdWriteIdx + 1) % MAX_HOST_CMD_Q;
  SLICHostCmd_t *pCmd = &slicHostCmds[index];
  pCmd->cmd = DSP_SLIC_RING_OFF;
  pCmd->port = ucPort;
  slicHostCmdWriteIdx = index;
#endif
}
#if FXO_MODULE
/**********************************************************************************
 *  Function Name  : slicFXOCmd.                                                  *
 *  Description    : Function to be execute commands on FXO.                      *
 *  Input          : FXS Port Number. command to execute.                         *
 *  Output         : None.                                                        *
 *  Return Value   : None.                                                        *
 *********************************************************************************/
void slicFXOCmd(UCHAR ucFXSPort, UCHAR cmd)
{
  osLockInterrupts();
  /* Change the time slot based on the FXS slot */
  if(slicHandlers.codecType == CODEC_SILABS)
  {
    si3050PCMSlotChange(ucFXSPort);
    si3050_cmd(cmd);
  }
  osUnlockInterrupts();
}
/**********************************************************************************
 *  Function Name  : slicGetFXOState.                                             *
 *  Description    : Function to get FXO Line status.                             *
 *  Input          : None.                                                        *
 *  Output         : None.                                                        *
 *  Return Value   : FXO Line Status.                                             *
 *********************************************************************************/
int slicGetFXOState(void)
{
  if(slicHandlers.codecType == CODEC_SILABS)
    return si3050_getFXOState();
  return -1;
}

#endif

/* Indicate Hook State Change */
void slicHookStateChanged(UINT16 portNum,INT32 hookState)
{
  if(hookState == SLIC_OFFHOOK_EVENT)
  {
    gaSlicInfo[portNum].uiT2SCntr = 0;
    gaSlicInfo[portNum].uiT4SCntr = 0;
    gaSlicInfo[portNum].ucSlicState = SLIC_NORING_OFFHOOK;
    SMM_CidTx[portNum].CidTxState    =  CID_TX_STATE0;
    SMM_CidTx[portNum].bCaller    =  0;
    dtmfCidGen[portNum].ucDigState = 0;
  }
  else if(hookState == SLIC_ONHOOK_EVENT)
  {
    gaSlicInfo[portNum].ucSlicState = SLIC_NORING_ONHOOK;
    gaSlicInfo[portNum].slicPolarity = IDLE_POL;
    slicActivate(portNum);
  }
  /* Call Handler */
  dspipcReportIndication(portNum,hookState);
}

/*======================================================================
 * Function Name        : slicIsr
 * Description          : slic isr to indicate the hook state changed
 *======================================================================
 */
void slicIsr(void)
{
#if SLIC_MODULES
  unsigned short uiCountry;
  unsigned short codec_status = 0x00;
  unsigned short i;
  unsigned char  curSlicState[MAX_SLIC_CHANNELS];
  unsigned char  prevSlicState[MAX_SLIC_CHANNELS];
  unsigned char codec_pol    = 0x00;

  confGetCountryOpts(&uiCountry);
  
  /* read the hook status for the codecs */
  for(i=0; i<slicHandlers.codecCnt; i++)
  {
    codec_status |= (slicHandlers.codecStatus(i));
  }
   
  for(i=0; i < MAX_SLIC_CHANNELS; i++)
  {
    curSlicState[i]  = (codec_status >> i ) & 0x1;
    prevSlicState[i] = (prev_status >> i ) & 0x1;


    if (curSlicState[i] == prevSlicState[i])
    {
      /* We have Slic State Change */
      if(gaSlicInfo[i].slicStateChanged == TRUE)
      {
        if(curSlicState[i] == SLIC_ONHOOK)
        {
          if(gaSlicInfo[i].slicOffTheHook == SLIC_OFFHOOK)
          {
//            if(gaSlicInfo[i].slicCountOnhookDebounce >= flashhookDebounceMaxValue[uiCountry])
            if(gaSlicInfo[i].slicCountOnhookDebounce >= onhookDebounceValue)
            { 
              DEBUG(PRINT("slicIsr: SLIC_ONHOOK on Port: %d\n",i), DEBUG_SLIC);
              slicHookStateChanged(i, SLIC_ONHOOK_EVENT);
              gaSlicInfo[i].slicOffTheHook = SLIC_ONHOOK;
              gaSlicInfo[i].slicCountOnhookDebounce = 0;
              gaSlicInfo[i].slicStateChanged = FALSE;

              // SAGEM LCe 16-11-05: tension raccroche a 42 V en tenant compte de la polarite
#ifdef FIRMWARE_CONF_COUNTRY_NL
              if (voltmonitor==LINE_MONITORING_YES)
		{ // in HOLLANDE, set by default, maybe change by CLI
#endif
                codec_pol = (readDirectReg(i, 72) & 0x40);
                if (codec_pol == 0x40)
                  //writeDirectReg(i, 72, 0x5C);
                  writeDirectReg(i, 72, 0x5F); // Modification for the output voltage
                else
                  //writeDirectReg(i, 72, 0x1C);
                  writeDirectReg(i, 72, 0x1F);// Modification of the output voltage.
#ifdef FIRMWARE_CONF_COUNTRY_NL
		}
#endif
            }
            else 
              gaSlicInfo[i].slicCountOnhookDebounce++;
          }
//          else 
//          {
//            gaSlicInfo[i].slicStateChanged = FALSE;
//            gaSlicInfo[i].slicCountOffhookDebounce = 0;
//          }
        }
        else if(curSlicState[i] == SLIC_OFFHOOK)
        {
          if(gaSlicInfo[i].slicOffTheHook == SLIC_OFFHOOK)
          {
            if ((gaSlicInfo[i].slicCountOnhookDebounce >= flashhookDebounceMinValue[uiCountry] ) &&
                (gaSlicInfo[i].slicCountOnhookDebounce <  flashhookDebounceMaxValue[uiCountry] ) )
            { 
              slicHookStateChanged(i, SLIC_FLASHHOOK_EVENT);
              DEBUG(PRINT("slicIsr: SLIC_FLASHHOOK_EVENT on Port: %d\n",i), DEBUG_SLIC);
              gaSlicInfo[i].slicCountOnhookDebounce = 0;
            }
            else 
            {
              gaSlicInfo[i].slicCountOnhookDebounce = 0;
              gaSlicInfo[i].slicStateChanged = FALSE;
            }
          }
          else 
          {
            if(gaSlicInfo[i].slicCountOffhookDebounce >= offhookDebounceValue) 
            {
              slicHookStateChanged(i, SLIC_OFFHOOK_EVENT);
              gaSlicInfo[i].slicOffTheHook = SLIC_OFFHOOK;
              gaSlicInfo[i].slicCountOffhookDebounce = 0;
              gaSlicInfo[i].slicStateChanged = FALSE;

                // SAGEM LCe 16-11-05: tension decroche a 31.5 V en tenant compte de la polarite
                codec_pol = (readDirectReg(i, 72) & 0x40);
                if (codec_pol == 0x40)
                  writeDirectReg(i, 72, 0x55);
                else
                  writeDirectReg(i, 72, 0x15);


              DEBUG(PRINT("slicIsr: SLIC_OFFHOOK on Port: %d\n",i), DEBUG_SLIC);
            }
            else 
              gaSlicInfo[i].slicCountOffhookDebounce++;
          }
        }
      }
    }
    else
    {
      gaSlicInfo[i].slicStateChanged = TRUE;
      if(curSlicState[i] == SLIC_ONHOOK)
      {
        gaSlicInfo[i].slicCountOnhookDebounce = 0;    // 1 ==> Phone is onhook
      }
      else
      {
        gaSlicInfo[i].slicCountOffhookDebounce = 0;   // 0 ==> Phone is offhook
      }
    }
//Sagem 20080226 : FXS_ToneDtmf_ACTIVE
#ifdef FXS_ToneDtmf_ACTIVE
    si3216NextTone(i);
#endif
  }
  /* save the codec status for future use */
  prev_status = codec_status;
#if FXO_MODULE
  si3050_isr();
#endif 
#endif
}

